#!/usr/bin/env node
/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	var __webpack_modules__ = ({

/***/ "./apps/interpreter/src/prerequisites.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.assertNodeVersion = void 0;
/**
 * Asserts that a certain node version is installed on the executing machine.
 * Exits the process if this prerequisite is not fulfilled.
 */
function assertNodeVersion() {
    const requiredNodeMajorVersion = 17;
    const currentNodeMajorVersion = getNodeMajorVersion();
    if (currentNodeMajorVersion < requiredNodeMajorVersion) {
        console.error(`Jayvee requires node version ${requiredNodeMajorVersion}.0.0 or higher.`);
        console.info(`Your current node version is ${currentNodeMajorVersion} - please upgrade!`);
        process.exit(1);
    }
}
exports.assertNodeVersion = assertNodeVersion;
/**
 * Returns the node version of the executing machine.
 * Exits the process if no node process is running.
 */
function getNodeMajorVersion() {
    var _a;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const nodeVersion = (_a = process === null || process === void 0 ? void 0 : process.versions) === null || _a === void 0 ? void 0 : _a.node;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (nodeVersion === undefined) {
        console.error('Could not find a nodejs runtime.');
        process.exit(1);
    }
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    return +nodeVersion.split('.')[0];
}


/***/ }),

/***/ "./apps/interpreter/src/run-action.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.runAction = void 0;
const tslib_1 = __webpack_require__("tslib");
const process = __webpack_require__("process");
const jayvee_interpreter_lib_1 = __webpack_require__("./libs/interpreter-lib/src/index.ts");
function runAction(fileName, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const extractAstNodeFn = (services, loggerFactory) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield (0, jayvee_interpreter_lib_1.extractAstNodeFromFile)(fileName, services, loggerFactory.createLogger());
        });
        if (options.parseOnly === true) {
            const { model, services } = yield (0, jayvee_interpreter_lib_1.parseModel)(extractAstNodeFn, options);
            const exitCode = model != null && services != null ? 0 : 1;
            process.exit(exitCode);
        }
        const exitCode = yield (0, jayvee_interpreter_lib_1.interpretModel)(extractAstNodeFn, options);
        process.exit(exitCode);
    });
}
exports.runAction = runAction;


/***/ }),

/***/ "./libs/execution/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/index.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-execution-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.logExecutionDuration = exports.executeBlock = exports.executeBlocks = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const types_1 = __webpack_require__("./libs/execution/src/lib/types/index.ts");
// eslint-disable-next-line import/no-cycle
const block_executor_registry_1 = __webpack_require__("./libs/execution/src/lib/blocks/block-executor-registry.ts");
const R = __webpack_require__("./libs/execution/src/lib/blocks/execution-result.ts");
/**
 * Executes an ordered list of blocks in sequence, using outputs from previous blocks as inputs for downstream blocks.
 *
 * @param executionContext The context the blocks are executed in, e.g., a pipeline or composite block
 * @param executionOrder An ordered list of blocks so that blocks that need inputs are after blocks that produce these inputs
 * @param initialInputValue An initial input that was produced outside of this block chain, e.g., as input to a composite block
 *
 * @returns The ordered blocks and their produced outputs or an error on failure
 */
function executeBlocks(executionContext, executionOrder, initialInputValue = undefined) {
    var _a, _b;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let isFirstBlock = true;
        for (const blockData of executionOrder) {
            const block = blockData.block;
            const parentData = (0, jayvee_language_server_1.collectParents)(block).map((parent) => executionOrder.find((blockData) => parent === blockData.block));
            let inputValue = ((_a = parentData[0]) === null || _a === void 0 ? void 0 : _a.value) === undefined ? types_1.NONE : (_b = parentData[0]) === null || _b === void 0 ? void 0 : _b.value;
            const useExternalInputValueForFirstBlock = isFirstBlock && inputValue === types_1.NONE && initialInputValue !== undefined;
            if (useExternalInputValueForFirstBlock) {
                inputValue = initialInputValue;
            }
            executionContext.enterNode(block);
            const executionResult = yield executeBlock(inputValue, block, executionContext);
            if (R.isErr(executionResult)) {
                return executionResult;
            }
            const blockResultData = executionResult.right;
            blockData.value = blockResultData;
            executionContext.exitNode(block);
            isFirstBlock = false;
        }
        return R.ok(executionOrder);
    });
}
exports.executeBlocks = executeBlocks;
function executeBlock(inputValue, block, executionContext) {
    var _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (inputValue == null) {
            executionContext.logger.logInfoDiagnostic(`Skipped execution because parent block emitted no value.`, { node: block, property: 'name' });
            return R.ok(null);
        }
        const blockExecutor = (0, block_executor_registry_1.createBlockExecutor)(block);
        const startTime = new Date();
        let result;
        try {
            result = yield blockExecutor.execute(inputValue, executionContext);
        }
        catch (unexpectedError) {
            return R.err({
                message: `An unknown error occurred: ${unexpectedError instanceof Error
                    ? (_a = unexpectedError.stack) !== null && _a !== void 0 ? _a : unexpectedError.message
                    : JSON.stringify(unexpectedError)}`,
                diagnostic: { node: block, property: 'name' },
            });
        }
        logExecutionDuration(startTime, executionContext.logger);
        return result;
    });
}
exports.executeBlock = executeBlock;
function logExecutionDuration(startTime, logger) {
    const endTime = new Date();
    const executionDurationMs = Math.round(endTime.getTime() - startTime.getTime());
    logger.logDebug(`Execution duration: ${executionDurationMs} ms.`);
}
exports.logExecutionDuration = logExecutionDuration;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-executor-class.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-executor-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createBlockExecutor = exports.getRegisteredBlockExecutors = exports.registerBlockExecutor = exports.blockExecutorRegistry = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
// eslint-disable-next-line import/no-cycle
const composite_block_executor_1 = __webpack_require__("./libs/execution/src/lib/blocks/composite-block-executor.ts");
exports.blockExecutorRegistry = new jayvee_language_server_1.Registry();
function registerBlockExecutor(executorClass) {
    exports.blockExecutorRegistry.register(executorClass.type, executorClass);
}
exports.registerBlockExecutor = registerBlockExecutor;
function getRegisteredBlockExecutors() {
    return [...exports.blockExecutorRegistry.getAll()];
}
exports.getRegisteredBlockExecutors = getRegisteredBlockExecutors;
function createBlockExecutor(block) {
    const blockType = block.type.ref;
    (0, assert_1.strict)(blockType !== undefined);
    if (!exports.blockExecutorRegistry.get(blockType.name) &&
        (0, jayvee_language_server_1.isCompositeBlocktypeDefinition)(block.type.ref)) {
        const executorClass = (0, composite_block_executor_1.createCompositeBlockExecutor)((0, composite_block_executor_1.getInputType)(block.type.ref), (0, composite_block_executor_1.getOutputType)(block.type.ref), block);
        exports.blockExecutorRegistry.register(block.type.ref.name, executorClass);
    }
    const blockExecutor = exports.blockExecutorRegistry.get(blockType.name);
    (0, assert_1.strict)(blockExecutor !== undefined, `No executor was registered for block type ${blockType.name}`);
    return new blockExecutor();
}
exports.createBlockExecutor = createBlockExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/block-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AbstractBlockExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const debug_configuration_1 = __webpack_require__("./libs/execution/src/lib/debugging/debug-configuration.ts");
const debug_log_visitor_1 = __webpack_require__("./libs/execution/src/lib/debugging/debug-log-visitor.ts");
const R = __webpack_require__("./libs/execution/src/lib/blocks/execution-result.ts");
class AbstractBlockExecutor {
    constructor(inputType, outputType) {
        this.inputType = inputType;
        this.outputType = outputType;
    }
    execute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const executionResult = yield this.doExecute(input, context);
            if (R.isOk(executionResult)) {
                this.logBlockResult(executionResult.right, context);
            }
            return executionResult;
        });
    }
    logBlockResult(result, context) {
        if (!context.runOptions.isDebugMode) {
            return;
        }
        if (result == null) {
            return;
        }
        const currentNode = context.getCurrentNode();
        (0, assert_1.strict)((0, jayvee_language_server_1.isBlockDefinition)(currentNode));
        const isBlockTargeted = (0, debug_configuration_1.isBlockTargetedForDebugLogging)(currentNode, context);
        if (!isBlockTargeted) {
            return;
        }
        result.acceptVisitor(new debug_log_visitor_1.DebugLogVisitor(context.runOptions.debugGranularity, 'Output', context.logger));
    }
}
exports.AbstractBlockExecutor = AbstractBlockExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/composite-block-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getOutputType = exports.getInputType = exports.createCompositeBlockExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const strict_1 = __webpack_require__("assert/strict");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const types_1 = __webpack_require__("./libs/execution/src/lib/types/index.ts");
// eslint-disable-next-line import/no-cycle
const block_execution_util_1 = __webpack_require__("./libs/execution/src/lib/blocks/block-execution-util.ts");
const block_executor_1 = __webpack_require__("./libs/execution/src/lib/blocks/block-executor.ts");
const R = __webpack_require__("./libs/execution/src/lib/blocks/execution-result.ts");
function createCompositeBlockExecutor(inputType, outputType, block) {
    var _a;
    (0, strict_1.strict)(block.type.ref, `Blocktype reference missing for block ${block.name}.`);
    (0, strict_1.strict)((0, jayvee_language_server_1.isCompositeBlocktypeDefinition)(block.type.ref), `Blocktype is not a composite block for block ${block.name}.`);
    const blockTypeReference = block.type.ref;
    return _a = class extends block_executor_1.AbstractBlockExecutor {
            constructor() {
                super(inputType, outputType);
            }
            doExecute(input, context) {
                var _a, _b, _c, _d;
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    context.logger.logDebug(`Executing composite block of type ${(_b = (_a = block.type.ref) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : 'undefined'}`);
                    this.addVariablesToContext(block, blockTypeReference.properties, context);
                    const executionOrder = (0, jayvee_language_server_1.getBlocksInTopologicalSorting)(blockTypeReference).map((block) => {
                        return { block: block, value: types_1.NONE };
                    });
                    const executionResult = yield (0, block_execution_util_1.executeBlocks)(context, executionOrder, input);
                    if (R.isErr(executionResult)) {
                        const diagnosticError = executionResult.left;
                        context.logger.logErrDiagnostic(diagnosticError.message, diagnosticError.diagnostic);
                    }
                    this.removeVariablesFromContext(blockTypeReference.properties, context);
                    if (R.isOk(executionResult)) {
                        // The last block always pipes into the output if it exists
                        const pipeline = getPipeline(blockTypeReference);
                        const lastBlock = pipeline.blocks.at(-1);
                        const blockExecutionResult = R.okData(executionResult).find((result) => { var _a; return result.block.name === ((_a = lastBlock === null || lastBlock === void 0 ? void 0 : lastBlock.ref) === null || _a === void 0 ? void 0 : _a.name); });
                        (0, strict_1.strict)(blockExecutionResult, `No execution result found for composite block ${(_d = (_c = block.type.ref) === null || _c === void 0 ? void 0 : _c.name) !== null && _d !== void 0 ? _d : 'undefined'}`);
                        return R.ok(blockExecutionResult.value);
                    }
                    return R.ok(null);
                });
            }
            removeVariablesFromContext(properties, context) {
                properties.forEach((prop) => context.evaluationContext.deleteValueForReference(prop.name));
            }
            addVariablesToContext(block, properties, context) {
                properties.forEach((blocktypeProperty) => {
                    const valueType = (0, jayvee_language_server_1.createValuetype)(blocktypeProperty.valueType);
                    (0, strict_1.strict)(valueType, `Can not create valuetype for blocktype property ${blocktypeProperty.name}`);
                    const propertyValue = this.getPropertyValueFromBlockOrDefault(blocktypeProperty.name, valueType, block, properties, context.evaluationContext);
                    (0, strict_1.strict)(propertyValue !== undefined, `Can not get value for blocktype property ${blocktypeProperty.name}`);
                    context.evaluationContext.setValueForReference(blocktypeProperty.name, propertyValue);
                });
            }
            getPropertyValueFromBlockOrDefault(name, valueType, block, properties, evaluationContext) {
                const propertyFromBlock = block.body.properties.find((property) => property.name === name);
                if (propertyFromBlock !== undefined) {
                    const value = (0, jayvee_language_server_1.evaluatePropertyValue)(propertyFromBlock, evaluationContext, valueType);
                    if (value !== undefined) {
                        return value;
                    }
                }
                const propertyFromBlockType = properties.find((property) => property.name === name);
                if ((propertyFromBlockType === null || propertyFromBlockType === void 0 ? void 0 : propertyFromBlockType.defaultValue) === undefined) {
                    return;
                }
                return (0, jayvee_language_server_1.evaluateExpression)(propertyFromBlockType.defaultValue, evaluationContext);
            }
        },
        _a.type = blockTypeReference.name,
        _a;
}
exports.createCompositeBlockExecutor = createCompositeBlockExecutor;
function getPipeline(block) {
    (0, strict_1.strict)(block.pipes[0], `Composite block ${block.name} must have exactly one pipeline.`);
    return block.pipes[0];
}
function getInputType(block) {
    (0, strict_1.strict)(block.inputs.length === 1, `Composite block ${block.name} must have exactly one input.`);
    return block.inputs[0] ? (0, jayvee_language_server_1.getIOType)(block.inputs[0]) : jayvee_language_server_1.IOType.NONE;
}
exports.getInputType = getInputType;
function getOutputType(block) {
    (0, strict_1.strict)(block.outputs.length === 1, `Composite block ${block.name} must have exactly one output.`);
    return block.outputs[0] ? (0, jayvee_language_server_1.getIOType)(block.outputs[0]) : jayvee_language_server_1.IOType.NONE;
}
exports.getOutputType = getOutputType;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/execution-result.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.okData = exports.isErr = exports.isOk = exports.err = exports.ok = void 0;
const E = __webpack_require__("fp-ts/lib/Either");
/**
 * Creates an @Ok object from a data object typed T.
 * @param data the data object
 * @returns the created @Ok object
 */
function ok(data) {
    return E.right(data);
}
exports.ok = ok;
/**
 * Creates an @Err object from a @ExecutionErrorDetails object.
 * @param details the @ExecutionErrorDetails object
 * @returns the created @Err object
 */
function err(details) {
    return E.left(details);
}
exports.err = err;
/**
 * Type guard for @see Ok
 */
function isOk(result) {
    return E.isRight(result);
}
exports.isOk = isOk;
/**
 * Type guard for @see Err
 */
function isErr(result) {
    return E.isLeft(result);
}
exports.isErr = isErr;
/**
 * Convenience method to get wrapped data of an @see Ok object.
 */
function okData(ok) {
    return ok.right;
}
exports.okData = okData;


/***/ }),

/***/ "./libs/execution/src/lib/blocks/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-executor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-executor-class.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-executor-registry.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/execution-result.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/block-execution-util.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/constraints/constraint-executor-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createConstraintExecutor = exports.getRegisteredConstraintExecutors = exports.registerConstraintExecutor = exports.registerDefaultConstraintExecutors = exports.constraintExecutorRegistry = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const langium_1 = __webpack_require__("langium");
const allowlist_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/allowlist-constraint-executor.ts");
const denylist_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/denylist-constraint-executor.ts");
const expression_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/expression-constraint-executor.ts");
const length_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/length-constraint-executor.ts");
const range_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/range-constraint-executor.ts");
const regex_constraint_executor_1 = __webpack_require__("./libs/execution/src/lib/constraints/executors/regex-constraint-executor.ts");
exports.constraintExecutorRegistry = new jayvee_language_server_1.Registry();
function registerDefaultConstraintExecutors() {
    registerConstraintExecutor(allowlist_constraint_executor_1.AllowlistConstraintExecutor);
    registerConstraintExecutor(denylist_constraint_executor_1.DenylistConstraintExecutor);
    registerConstraintExecutor(regex_constraint_executor_1.RegexConstraintExecutor);
    registerConstraintExecutor(length_constraint_executor_1.LengthConstraintExecutor);
    registerConstraintExecutor(range_constraint_executor_1.RangeConstraintExecutor);
}
exports.registerDefaultConstraintExecutors = registerDefaultConstraintExecutors;
function registerConstraintExecutor(executorClass) {
    exports.constraintExecutorRegistry.register(executorClass.type, executorClass);
}
exports.registerConstraintExecutor = registerConstraintExecutor;
function getRegisteredConstraintExecutors() {
    return exports.constraintExecutorRegistry.getAll();
}
exports.getRegisteredConstraintExecutors = getRegisteredConstraintExecutors;
function createConstraintExecutor(constraint) {
    var _a;
    if ((0, jayvee_language_server_1.isTypedConstraintDefinition)(constraint)) {
        const constraintType = (_a = constraint.type.ref) === null || _a === void 0 ? void 0 : _a.name;
        (0, assert_1.strict)(constraintType !== undefined, `Could not resolve reference to constraint type of ${constraint.name}`);
        const constraintExecutor = exports.constraintExecutorRegistry.get(constraintType);
        (0, assert_1.strict)(constraintExecutor !== undefined, `No executor was registered for constraint type ${constraintType}`);
        return new constraintExecutor();
    }
    else if ((0, jayvee_language_server_1.isExpressionConstraintDefinition)(constraint)) {
        return new expression_constraint_executor_1.ExpressionConstraintExecutor(constraint);
    }
    (0, langium_1.assertUnreachable)(constraint);
}
exports.createConstraintExecutor = createConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/allowlist-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AllowlistConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let AllowlistConstraintExecutor = class AllowlistConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const allowlist = context.getPropertyValue('allowlist', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.Text));
        return allowlist.includes(value);
    }
};
AllowlistConstraintExecutor.type = 'AllowlistConstraint';
AllowlistConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], AllowlistConstraintExecutor);
exports.AllowlistConstraintExecutor = AllowlistConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/denylist-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DenylistConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let DenylistConstraintExecutor = class DenylistConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const denylist = context.getPropertyValue('denylist', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.Text));
        return !denylist.includes(value);
    }
};
DenylistConstraintExecutor.type = 'DenylistConstraint';
DenylistConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], DenylistConstraintExecutor);
exports.DenylistConstraintExecutor = DenylistConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/expression-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ExpressionConstraintExecutor = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class ExpressionConstraintExecutor {
    constructor(astNode) {
        this.astNode = astNode;
    }
    isValid(value, context) {
        const expression = this.astNode.expression;
        context.evaluationContext.setValueForValueKeyword(value);
        const result = (0, jayvee_language_server_1.evaluateExpression)(expression, context.evaluationContext);
        (0, assert_1.strict)(jayvee_language_server_1.PrimitiveValuetypes.Boolean.isInternalValueRepresentation(result));
        context.evaluationContext.deleteValueForValueKeyword();
        return result;
    }
}
exports.ExpressionConstraintExecutor = ExpressionConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/length-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LengthConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let LengthConstraintExecutor = class LengthConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const minLength = context.getPropertyValue('minLength', jayvee_language_server_1.PrimitiveValuetypes.Integer);
        const maxLength = context.getPropertyValue('maxLength', jayvee_language_server_1.PrimitiveValuetypes.Integer);
        return minLength <= value.length && value.length <= maxLength;
    }
};
LengthConstraintExecutor.type = 'LengthConstraint';
LengthConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], LengthConstraintExecutor);
exports.LengthConstraintExecutor = LengthConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/range-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RangeConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let RangeConstraintExecutor = class RangeConstraintExecutor {
    isValid(value, context) {
        let numericValue;
        if (typeof value === 'string') {
            numericValue = Number.parseFloat(value);
        }
        else if (typeof value === 'number') {
            numericValue = value;
        }
        else {
            return false;
        }
        const lowerBound = context.getPropertyValue('lowerBound', jayvee_language_server_1.PrimitiveValuetypes.Decimal);
        const lowerBoundInclusive = context.getPropertyValue('lowerBoundInclusive', jayvee_language_server_1.PrimitiveValuetypes.Boolean);
        const upperBound = context.getPropertyValue('upperBound', jayvee_language_server_1.PrimitiveValuetypes.Decimal);
        const upperBoundInclusive = context.getPropertyValue('upperBoundInclusive', jayvee_language_server_1.PrimitiveValuetypes.Boolean);
        const lowerBoundFulfilled = lowerBoundInclusive
            ? lowerBound <= numericValue
            : lowerBound < numericValue;
        const upperBoundFulfilled = upperBoundInclusive
            ? numericValue <= upperBound
            : numericValue < upperBound;
        return lowerBoundFulfilled && upperBoundFulfilled;
    }
};
RangeConstraintExecutor.type = 'RangeConstraint';
RangeConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], RangeConstraintExecutor);
exports.RangeConstraintExecutor = RangeConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/executors/regex-constraint-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RegexConstraintExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const implements_static_decorator_1 = __webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts");
let RegexConstraintExecutor = class RegexConstraintExecutor {
    isValid(value, context) {
        if (typeof value !== 'string') {
            return false;
        }
        const regex = context.getPropertyValue('regex', jayvee_language_server_1.PrimitiveValuetypes.Regex);
        return regex.test(value);
    }
};
RegexConstraintExecutor.type = 'RegexConstraint';
RegexConstraintExecutor = tslib_1.__decorate([
    (0, implements_static_decorator_1.implementsStatic)()
], RegexConstraintExecutor);
exports.RegexConstraintExecutor = RegexConstraintExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/constraints/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/constraints/constraint-executor-registry.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/debugging/debug-configuration.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isBlockTargetedForDebugLogging = exports.DefaultDebugTargetsValue = exports.isDebugGranularity = exports.DebugGranularityValues = exports.DefaultGranularityValue = void 0;
exports.DefaultGranularityValue = 'minimal';
exports.DebugGranularityValues = [
    'peek',
    'exhaustive',
    exports.DefaultGranularityValue,
];
function isDebugGranularity(obj) {
    return obj === 'exhaustive' || obj === 'peek' || obj === 'minimal';
}
exports.isDebugGranularity = isDebugGranularity;
exports.DefaultDebugTargetsValue = 'all';
function isBlockTargetedForDebugLogging(block, context) {
    return (context.runOptions.debugTargets === exports.DefaultDebugTargetsValue ||
        context.runOptions.debugTargets.includes(block.name));
}
exports.isBlockTargetedForDebugLogging = isBlockTargetedForDebugLogging;


/***/ }),

/***/ "./libs/execution/src/lib/debugging/debug-log-visitor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DebugLogVisitor = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class DebugLogVisitor {
    constructor(debugGranularity, logPrefix, logger) {
        this.debugGranularity = debugGranularity;
        this.logPrefix = logPrefix;
        this.logger = logger;
        this.PEEK_NUMBER_OF_WORKBOOKS = 5;
        this.PEEK_NUMBER_OF_ROWS = 10;
        this.PEEK_NUMBER_OF_BYTES = 100;
        this.PEEK_NUMBER_OF_LINES = 10;
    }
    visitTable(table) {
        if (this.debugGranularity === 'minimal') {
            return;
        }
        const numberOfRows = table.getNumberOfRows();
        this.log(`Table with ${numberOfRows} rows and ${table.getNumberOfColumns()} columns.`);
        const headers = [...table.getColumns().entries()]
            .map(([columnName, column]) => {
            return `${columnName} (${column.valuetype.getName()})`;
        })
            .join(' | ');
        this.log(`[Header] ${headers}`);
        for (let i = 0; i < numberOfRows; ++i) {
            if (this.debugGranularity === 'peek' && i >= this.PEEK_NUMBER_OF_ROWS) {
                break;
            }
            const row = table.getRow(i);
            const rowData = [...row.values()]
                .map((cell) => (0, jayvee_language_server_1.internalValueToString)(cell))
                .join(' | ');
            this.log(`[Row ${i}] ${rowData}`);
        }
        this.logPeekComment();
    }
    visitSheet(sheet) {
        if (this.debugGranularity === 'minimal') {
            return;
        }
        this.log(`Sheet with ${sheet.getNumberOfRows()} rows and ${sheet.getNumberOfColumns()} columns.`);
        const rowsAsString = sheet
            .getData()
            .filter((_, rowIndex) => {
            if (this.debugGranularity === 'peek') {
                return rowIndex < this.PEEK_NUMBER_OF_ROWS;
            }
            return true;
        })
            .map((row) => `${row.map((cell) => `"${cell}"`).join(', ')}`);
        rowsAsString.forEach((row, i) => {
            this.log(`[Row ${i}] ${row}`);
        });
        this.logPeekComment();
    }
    visitNone() {
        if (this.debugGranularity === 'minimal') {
            return;
        }
        this.log('<None>');
    }
    visitFileSystem(fileSystem) {
        var _a, _b;
        if (this.debugGranularity === 'minimal') {
            return;
        }
        this.log((_b = (_a = fileSystem.getFile('/')) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '<found no root file>');
    }
    visitBinaryFile(binaryFile) {
        if (this.debugGranularity === 'minimal') {
            return;
        }
        const buffer = binaryFile.content.slice(0, this.PEEK_NUMBER_OF_BYTES);
        const hexString = [...new Uint8Array(buffer)]
            .map((x) => x.toString(16).padStart(2, '0').toUpperCase())
            .join('');
        this.log(`<hex> ${hexString}`);
        this.logPeekComment();
    }
    visitTextFile(binaryFile) {
        var _a;
        if (this.debugGranularity === 'minimal') {
            return;
        }
        for (let i = 0; i < binaryFile.content.length; ++i) {
            if (i > this.PEEK_NUMBER_OF_LINES) {
                break;
            }
            this.log(`[Line ${i}] ${(_a = binaryFile.content[i]) !== null && _a !== void 0 ? _a : '<undefined>'}`);
        }
        this.logPeekComment();
    }
    logPeekComment() {
        if (this.debugGranularity === 'minimal') {
            return;
        }
        if (this.debugGranularity !== 'peek') {
            return;
        }
        this.log('... (omitted in peek mode)');
    }
    visitWorkbook(workbook) {
        if (this.debugGranularity === 'minimal') {
            return;
        }
        const workbookSheets = workbook.getSheets();
        const keys = Array.from(workbookSheets.keys());
        const numberOfSheets = workbookSheets.size;
        if (numberOfSheets === 0) {
            this.log(`Empty Workbook`);
            return;
        }
        this.log(`Workbook with ${numberOfSheets} Sheets.`);
        this.log(`Sheets in WorkBook:`);
        for (let i = 0; i < numberOfSheets; ++i) {
            if (this.debugGranularity === 'peek' &&
                i >= this.PEEK_NUMBER_OF_WORKBOOKS) {
                break;
            }
            const currentWorkbookName = keys[i];
            if (currentWorkbookName === undefined) {
                continue;
            }
            this.log(`WorkSheet: ${currentWorkbookName}`);
        }
        this.logPeekComment();
    }
    log(text) {
        this.logger.logDebug(`[${this.logPrefix}] ${text}`);
    }
}
exports.DebugLogVisitor = DebugLogVisitor;


/***/ }),

/***/ "./libs/execution/src/lib/debugging/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/debugging/debug-configuration.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/debugging/debug-log-visitor.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/execution-context.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ExecutionContext = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const langium_1 = __webpack_require__("langium");
class ExecutionContext {
    constructor(pipeline, logger, runOptions, evaluationContext) {
        this.pipeline = pipeline;
        this.logger = logger;
        this.runOptions = runOptions;
        this.evaluationContext = evaluationContext;
        this.stack = [];
        logger.setLoggingContext(pipeline.name);
    }
    enterNode(node) {
        this.stack.push(node);
        this.updateLoggingContext();
    }
    exitNode(node) {
        const poppedNode = this.stack.pop();
        (0, assert_1.strict)(poppedNode === node);
        this.updateLoggingContext();
    }
    /**
     * @returns the latest stack node. Returns the pipeline if the stack is empty.
     */
    getCurrentNode() {
        const currentNode = this.stack[this.stack.length - 1];
        if (currentNode === undefined) {
            return this.pipeline;
        }
        return currentNode;
    }
    updateLoggingContext() {
        this.logger.setLoggingDepth(this.stack.length);
        this.logger.setLoggingContext(this.getCurrentNode().name);
    }
    getPropertyValue(propertyName, valuetype) {
        const property = this.getProperty(propertyName);
        if (property === undefined) {
            return this.getDefaultPropertyValue(propertyName, valuetype);
        }
        const propertyValue = (0, jayvee_language_server_1.evaluatePropertyValue)(property, this.evaluationContext, valuetype);
        (0, assert_1.strict)(propertyValue !== undefined);
        return propertyValue;
    }
    getProperty(propertyName) {
        const currentNode = this.getCurrentNode();
        if ((0, jayvee_language_server_1.isPipelineDefinition)(currentNode) ||
            (0, jayvee_language_server_1.isExpressionConstraintDefinition)(currentNode)) {
            return undefined;
        }
        const body = currentNode.body;
        if (!(0, jayvee_language_server_1.isPropertyBody)(body)) {
            return undefined;
        }
        return body.properties.find((property) => property.name === propertyName);
    }
    getOrFailProperty(propertyName) {
        const property = this.getProperty(propertyName);
        (0, assert_1.strict)(property !== undefined);
        return property;
    }
    getDefaultPropertyValue(propertyName, valuetype) {
        const wrapper = this.getWrapperOfCurrentNode();
        const propertySpec = wrapper.getPropertySpecification(propertyName);
        (0, assert_1.strict)(propertySpec !== undefined);
        const defaultValue = propertySpec.defaultValue;
        (0, assert_1.strict)(defaultValue !== undefined);
        (0, assert_1.strict)(valuetype.isInternalValueRepresentation(defaultValue));
        return defaultValue;
    }
    getWrapperOfCurrentNode() {
        var _a, _b, _c, _d;
        const currentNode = this.getCurrentNode();
        (0, assert_1.strict)(!(0, jayvee_language_server_1.isPipelineDefinition)(currentNode));
        (0, assert_1.strict)(!(0, jayvee_language_server_1.isExpressionConstraintDefinition)(currentNode));
        (0, assert_1.strict)(!(0, jayvee_language_server_1.isTransformDefinition)(currentNode));
        (0, assert_1.strict)((0, langium_1.isReference)(currentNode.type));
        if ((0, jayvee_language_server_1.isTypedConstraintDefinition)(currentNode)) {
            (0, assert_1.strict)(jayvee_language_server_1.ConstraintTypeWrapper.canBeWrapped(currentNode.type), `ConstraintType ${(_b = (_a = currentNode.type.ref) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : '<unresolved reference>'} cannot be wrapped`);
            return new jayvee_language_server_1.ConstraintTypeWrapper(currentNode.type);
        }
        else if ((0, jayvee_language_server_1.isBlockDefinition)(currentNode)) {
            (0, assert_1.strict)(jayvee_language_server_1.BlockTypeWrapper.canBeWrapped(currentNode.type), `Blocktype ${(_d = (_c = currentNode.type.ref) === null || _c === void 0 ? void 0 : _c.name) !== null && _d !== void 0 ? _d : '<unresolved reference>'} cannot be wrapped`);
            return new jayvee_language_server_1.BlockTypeWrapper(currentNode.type);
        }
        (0, langium_1.assertUnreachable)(currentNode);
    }
}
exports.ExecutionContext = ExecutionContext;


/***/ }),

/***/ "./libs/execution/src/lib/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.useExtension = void 0;
const block_executor_registry_1 = __webpack_require__("./libs/execution/src/lib/blocks/block-executor-registry.ts");
function useExtension(extension) {
    extension.getBlockExecutors().forEach(block_executor_registry_1.registerBlockExecutor);
}
exports.useExtension = useExtension;


/***/ }),

/***/ "./libs/execution/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/blocks/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/constraints/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/transforms/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/util/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/debugging/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/execution-context.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/extension.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/logging/index.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/logging/cached-logger.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CachedLogger = void 0;
const chalk = __webpack_require__("chalk");
const default_logger_1 = __webpack_require__("./libs/execution/src/lib/logging/default-logger.ts");
const log_cache_1 = __webpack_require__("./libs/execution/src/lib/logging/log-cache.ts");
const logger_1 = __webpack_require__("./libs/execution/src/lib/logging/logger.ts");
class CachedLogger extends default_logger_1.DefaultLogger {
    constructor(enableDebugLogging, loggingContext, printLogs = true, depth = 0, cacheSize = 200) {
        super(enableDebugLogging, loggingContext, depth);
        this.printLogs = printLogs;
        this.logCache = new log_cache_1.LogCache(cacheSize);
    }
    /**
     * Gets all log entries with the specified log levels.
     * If no log level given, returns logs on all log levels.
     */
    getLogs(...logLevel) {
        const filterLevels = logLevel.length === 0 ? Object.values(logger_1.DiagnosticSeverity) : logLevel;
        return this.logCache
            .getLogs()
            .filter((log) => filterLevels.includes(log.severity));
    }
    clearLogs() {
        this.logCache.clearLogs();
    }
    logInfo(message) {
        const msg = `${chalk.bold(this.getContext())}${message}`;
        this.logCache.insertLogMessage(msg, logger_1.DiagnosticSeverity.INFO);
        this.printMessageToPrintFnIfEnabled(msg, console.log);
    }
    logDebug(message) {
        if (this.enableDebugLogging) {
            const msg = `${chalk.bold(this.getContext())}${message}`;
            this.logCache.insertLogMessage(msg, logger_1.DiagnosticSeverity.DEBUG);
            this.printMessageToPrintFnIfEnabled(msg, console.log);
        }
    }
    logErr(message) {
        const msg = `${chalk.bold(this.getContext())}${chalk.red(message)}`;
        this.logCache.insertLogMessage(msg, logger_1.DiagnosticSeverity.ERROR);
        this.printMessageToPrintFnIfEnabled(msg, console.error);
    }
    logDiagnostic(severity, message, range, document) {
        const printFn = (msg) => {
            const basePrintFn = this.inferPrintFunction(severity);
            this.logCache.insertLogMessage(msg, severity);
            this.printMessageToPrintFnIfEnabled(msg, basePrintFn);
        };
        const colorFn = this.inferChalkColor(severity);
        this.logDiagnosticMessage(severity, message, printFn, colorFn);
        this.logDiagnosticInfo(range, document, printFn, colorFn);
        printFn('');
    }
    printMessageToPrintFnIfEnabled(message, printFn) {
        if (this.printLogs) {
            printFn(message);
        }
    }
}
exports.CachedLogger = CachedLogger;


/***/ }),

/***/ "./libs/execution/src/lib/logging/default-logger.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DefaultLogger = void 0;
const chalk = __webpack_require__("chalk");
const errors_1 = __webpack_require__("langium/lib/utils/errors");
const vscode_languageserver_types_1 = __webpack_require__("vscode-languageserver-types");
const logger_1 = __webpack_require__("./libs/execution/src/lib/logging/logger.ts");
class DefaultLogger extends logger_1.Logger {
    constructor(enableDebugLogging, loggingContext, depth = 0) {
        super();
        this.enableDebugLogging = enableDebugLogging;
        this.loggingContext = loggingContext;
        this.depth = depth;
        this.TAB_TO_SPACES = 4;
    }
    logInfo(message) {
        console.log(`${this.getDepthTabs()}${chalk.bold(this.getContext())}${message}`);
    }
    logDebug(message) {
        if (this.enableDebugLogging) {
            console.log(`${this.getDepthTabs()}${chalk.bold(this.getContext())}${message}`);
        }
    }
    logErr(message) {
        console.error(`${this.getDepthTabs()}${chalk.bold(this.getContext())}${chalk.red(message)}`);
    }
    setLoggingContext(loggingContext) {
        this.loggingContext = loggingContext;
    }
    setLoggingDepth(depth) {
        this.depth = depth;
    }
    getDepthTabs() {
        return '\t'.repeat(this.depth);
    }
    getContext() {
        return this.loggingContext !== undefined
            ? chalk.grey(`[${this.loggingContext}] `)
            : '';
    }
    logDiagnostic(severity, message, range, document) {
        const printFn = this.inferPrintFunction(severity);
        const colorFn = this.inferChalkColor(severity);
        this.logDiagnosticMessage(severity, message, printFn, colorFn);
        this.logDiagnosticInfo(range, document, printFn, colorFn);
        printFn('');
    }
    logDiagnosticMessage(severityName, message, printFn, colorFn) {
        printFn(`${this.getDepthTabs()}${chalk.bold(colorFn(severityName))}: ${message}`);
    }
    logDiagnosticInfo(range, document, printFn, colorFn) {
        const startLineNumber = range.start.line + 1;
        const endLineNumber = range.end.line + 1;
        const fullRange = {
            start: {
                line: range.start.line,
                character: 0,
            },
            end: {
                line: range.end.line,
                character: vscode_languageserver_types_1.uinteger.MAX_VALUE,
            },
        };
        const text = document.textDocument.getText(fullRange).trimEnd();
        const lines = text.split('\n');
        const lineNumberLength = Math.floor(Math.log10(endLineNumber)) + 1;
        printFn(`${this.getDepthTabs()}$In ${document.uri.path}:${startLineNumber}:${range.start.character + 1}`);
        lines.forEach((line, i) => {
            const lineNumber = startLineNumber + i;
            const paddedLineNumber = String(lineNumber).padStart(lineNumberLength, ' ');
            printFn(`${this.getDepthTabs()}${chalk.grey(`${paddedLineNumber} |`)} ${line.replace(/\t/g, ' '.repeat(this.TAB_TO_SPACES))}`);
            let underlineFrom = 0;
            let underlineTo = line.length;
            if (lineNumber === startLineNumber) {
                underlineFrom = range.start.character;
            }
            if (lineNumber === endLineNumber) {
                underlineTo = range.end.character;
            }
            const underlineIndent = this.repeatCharAccordingToString(' ', line.substring(0, underlineFrom), this.TAB_TO_SPACES);
            const underline = this.repeatCharAccordingToString('^', line.substring(underlineFrom, underlineTo), this.TAB_TO_SPACES);
            printFn(`${this.getDepthTabs()}${chalk.grey(`${' '.repeat(lineNumberLength)} |`)} ${underlineIndent}${colorFn(underline)}`);
        });
    }
    /**
     * Repeats {@link charToRepeat} as many times as {@link accordingTo} is long.
     * For each occurrence of \t in {@link accordingTo},
     * {@link charToRepeat} is repeated {@link tabRepeats} times instead of once.
     */
    repeatCharAccordingToString(charToRepeat, accordingTo, tabRepeats) {
        return Array.from(accordingTo).reduce((prev, cur) => {
            const repeatedChar = cur === '\t' ? charToRepeat.repeat(tabRepeats) : charToRepeat;
            return `${prev}${repeatedChar}`;
        }, '');
    }
    inferPrintFunction(severity) {
        switch (severity) {
            case logger_1.DiagnosticSeverity.ERROR:
                return console.error;
            case logger_1.DiagnosticSeverity.WARNING:
                return console.warn;
            case logger_1.DiagnosticSeverity.INFO:
            case logger_1.DiagnosticSeverity.HINT:
            case logger_1.DiagnosticSeverity.DEBUG:
                return console.info;
            default:
                (0, errors_1.assertUnreachable)(severity);
        }
    }
    inferChalkColor(severity) {
        switch (severity) {
            case logger_1.DiagnosticSeverity.ERROR:
                return chalk.red;
            case logger_1.DiagnosticSeverity.WARNING:
                return chalk.yellow;
            case logger_1.DiagnosticSeverity.INFO:
                return chalk.green;
            case logger_1.DiagnosticSeverity.HINT:
                return chalk.blue;
            case logger_1.DiagnosticSeverity.DEBUG:
                return chalk.grey;
            default:
                (0, errors_1.assertUnreachable)(severity);
        }
    }
}
exports.DefaultLogger = DefaultLogger;


/***/ }),

/***/ "./libs/execution/src/lib/logging/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/logging/logger.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/logging/default-logger.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/logging/cached-logger.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/logging/log-cache.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LogCache = void 0;
class LogCache {
    constructor(maxCacheSize = Number.POSITIVE_INFINITY) {
        this.logs = [];
        if (maxCacheSize <= 0) {
            throw new Error('maxCacheSize needs to be greater than 0');
        }
        this.logCacheMaxSize = maxCacheSize;
    }
    insertLogMessage(message, severity) {
        this.logs.push({ message, severity });
        this.removeOldestLogsIfSizeExceeded();
    }
    removeOldestLogsIfSizeExceeded() {
        if (this.logs.length > this.logCacheMaxSize) {
            this.logs = this.logs.slice(0, 1);
        }
    }
    getLogMessages() {
        return this.logs.map((e) => e.message);
    }
    getLogs() {
        return this.logs;
    }
    clearLogs() {
        this.logs = [];
    }
}
exports.LogCache = LogCache;


/***/ }),

/***/ "./libs/execution/src/lib/logging/logger.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Logger = exports.DiagnosticSeverity = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ls = __webpack_require__("vscode-languageserver");
var DiagnosticSeverity;
(function (DiagnosticSeverity) {
    DiagnosticSeverity["ERROR"] = "error";
    DiagnosticSeverity["WARNING"] = "warning";
    DiagnosticSeverity["INFO"] = "info";
    DiagnosticSeverity["HINT"] = "hint";
    DiagnosticSeverity["DEBUG"] = "debug";
})(DiagnosticSeverity = exports.DiagnosticSeverity || (exports.DiagnosticSeverity = {}));
class Logger {
    logLangiumDiagnostic(severity, message, diagnostic) {
        this.logDiagnostic(severity, message, (0, langium_1.getDiagnosticRange)(diagnostic), (0, langium_1.getDocument)(diagnostic.node));
    }
    logErrDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.ERROR, message, diagnostic);
    }
    logWarnDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.WARNING, message, diagnostic);
    }
    logInfoDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.INFO, message, diagnostic);
    }
    logHintDiagnostic(message, diagnostic) {
        this.logLangiumDiagnostic(DiagnosticSeverity.HINT, message, diagnostic);
    }
    logLanguageServerDiagnostic(lsDiagnostic, document) {
        (0, assert_1.strict)(lsDiagnostic.severity !== undefined, 'The diagnostic severity is assumed to be present');
        this.logDiagnostic(this.toDiagnosticSeverity(lsDiagnostic.severity), lsDiagnostic.message, lsDiagnostic.range, document);
    }
    toDiagnosticSeverity(severity) {
        switch (severity) {
            case ls.DiagnosticSeverity.Error:
                return DiagnosticSeverity.ERROR;
            case ls.DiagnosticSeverity.Warning:
                return DiagnosticSeverity.WARNING;
            case ls.DiagnosticSeverity.Information:
                return DiagnosticSeverity.INFO;
            case ls.DiagnosticSeverity.Hint:
                return DiagnosticSeverity.HINT;
        }
    }
}
exports.Logger = Logger;


/***/ }),

/***/ "./libs/execution/src/lib/transforms/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/transforms/transform-executor.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/transforms/transform-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TransformExecutor = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const types_1 = __webpack_require__("./libs/execution/src/lib/types/index.ts");
class TransformExecutor {
    constructor(transform) {
        this.transform = transform;
    }
    getInputDetails() {
        return this.getPortDetails('from');
    }
    getOutputDetails() {
        const portDetails = this.getPortDetails('to');
        (0, assert_1.strict)(portDetails.length === 1);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return portDetails[0];
    }
    getPortDetails(kind) {
        const ports = this.transform.body.ports.filter((x) => x.kind === kind);
        const portDetails = ports.map((port) => {
            const valuetypeNode = port.valueType;
            const valuetype = (0, jayvee_language_server_1.createValuetype)(valuetypeNode);
            (0, assert_1.strict)(valuetype !== undefined);
            return {
                port: port,
                valuetype: valuetype,
            };
        });
        return portDetails;
    }
    getOutputAssignment() {
        const outputAssignments = this.transform.body.outputAssignments;
        (0, assert_1.strict)(outputAssignments.length === 1);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return outputAssignments[0];
    }
    executeTransform(columns, numberOfRows, context) {
        context.enterNode(this.transform);
        const result = this.doExecuteTransform(columns, numberOfRows, context);
        context.exitNode(this.transform);
        return result;
    }
    doExecuteTransform(columns, numberOfRows, context) {
        const inputDetailsList = this.getInputDetails();
        const outputDetails = this.getOutputDetails();
        const newColumn = [];
        const rowsToDelete = [];
        for (let rowIndex = 0; rowIndex < numberOfRows; ++rowIndex) {
            this.addVariablesToContext(inputDetailsList, columns, rowIndex, context);
            let newValue = undefined;
            try {
                newValue = (0, jayvee_language_server_1.evaluateExpression)(this.getOutputAssignment().expression, context.evaluationContext);
            }
            catch (e) {
                if (e instanceof Error) {
                    context.logger.logDebug(e.message);
                }
                else {
                    context.logger.logDebug(String(e));
                }
            }
            if (newValue === undefined) {
                context.logger.logDebug(`Dropping row ${rowIndex + 1}: Could not evaluate transform expression`);
                rowsToDelete.push(rowIndex);
            }
            else if (!(0, types_1.isValidValueRepresentation)(newValue, outputDetails.valuetype, context)) {
                (0, assert_1.strict)(typeof newValue === 'string' ||
                    typeof newValue === 'boolean' ||
                    typeof newValue === 'number');
                context.logger.logDebug(`Invalid value in row ${rowIndex + 1}: "${newValue.toString()}" does not match the type ${outputDetails.valuetype.getName()}`);
                rowsToDelete.push(rowIndex);
            }
            else {
                newColumn.push(newValue);
            }
            this.removeVariablesFromContext(inputDetailsList, context);
        }
        return {
            rowsToDelete: rowsToDelete,
            resultingColumn: {
                values: newColumn,
                valuetype: outputDetails.valuetype,
            },
        };
    }
    removeVariablesFromContext(inputDetailsList, context) {
        for (const inputDetails of inputDetailsList) {
            context.evaluationContext.deleteValueForReference(inputDetails.port.name);
        }
    }
    addVariablesToContext(inputDetailsList, columns, rowIndex, context) {
        for (const inputDetails of inputDetailsList) {
            const variableName = inputDetails.port.name;
            const column = columns.get(variableName);
            (0, assert_1.strict)(column !== undefined);
            const variableValue = column.values[rowIndex];
            (0, assert_1.strict)(variableValue !== undefined);
            context.evaluationContext.setValueForReference(variableName, variableValue);
        }
    }
}
exports.TransformExecutor = TransformExecutor;


/***/ }),

/***/ "./libs/execution/src/lib/types/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/index.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-inmemory.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.InMemoryFileSystem = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const filesystem_node_directory_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-directory.ts");
const filesystem_node_file_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file.ts");
class InMemoryFileSystem {
    constructor() {
        this.ioType = jayvee_language_server_1.IOType.FILE_SYSTEM;
        this.rootDirectory = new filesystem_node_directory_1.FileSystemDirectory('');
    }
    /**
     * Retrieves a file from the file system. Parent directory indicators (../) are resolved to up to root
     * @function getFile
     * @param {string} path - The absolute path to the file starting with "/..."
     * @returns {FileSystemFile<unknown> | null} - The file or null if the node does not exist.
     */
    getFile(path) {
        const processedParts = this.processPath(path);
        if (processedParts != null) {
            const node = this.rootDirectory.getNode(processedParts);
            if (node instanceof filesystem_node_file_1.FileSystemFile) {
                return node;
            }
        }
        return null;
    }
    /**
     * Saves a file to the file system.
     * @function putNode
     * @param {string} path - The absolute path to the file starting with "/..."
     * @param { FileSystemFile<unknown>} file - The file to save.
     * @returns {FileSystem | null} - The FileSystem where file was inserted or null if the file failed to insert
     */
    putFile(path, file) {
        const processedParts = this.processPath(path);
        if (processedParts != null) {
            const node = this.rootDirectory.putNode(processedParts, file);
            if (node instanceof filesystem_node_file_1.FileSystemFile) {
                return this;
            }
        }
        return null;
    }
    static getPathSeparator() {
        return InMemoryFileSystem.PATH_SEPARATOR;
    }
    processPath(path) {
        if (!path.startsWith(InMemoryFileSystem.getPathSeparator())) {
            return null;
        }
        const parts = path
            .split(InMemoryFileSystem.getPathSeparator())
            .filter((p) => p !== ''); // Process paths like "folder1//folder1" to "folder1/folder2"
        const processedParts = [];
        for (const part of parts) {
            if (part === InMemoryFileSystem.CURRENT_DIR) {
                continue; // Skip current dirs in path
            }
            if (part === InMemoryFileSystem.PARENT_DIR) {
                const poppedPath = processedParts.pop(); // Go level up in folder hierarchy, max level up is root dir
                // If Path ascend beyond root, error
                if (poppedPath === undefined) {
                    return null;
                }
            }
            else {
                processedParts.push(part);
            }
        }
        // Add path part for root to processedParts and return
        return ['', ...processedParts];
    }
    acceptVisitor(visitor) {
        return visitor.visitFileSystem(this);
    }
}
exports.InMemoryFileSystem = InMemoryFileSystem;
InMemoryFileSystem.PATH_SEPARATOR = '/';
InMemoryFileSystem.CURRENT_DIR = '.';
InMemoryFileSystem.PARENT_DIR = '..';


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-node-directory.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.FileSystemDirectory = void 0;
// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
const filesystem_node_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node.ts");
const filesystem_node_file_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file.ts");
class FileSystemDirectory extends filesystem_node_1.FileSystemNode {
    constructor(name) {
        super(name);
        this.name = name;
        this.children = [];
    }
    getNode(pathParts) {
        const [firstPart, ...rest] = pathParts;
        // Base case: Called a wrong node
        if (firstPart !== this.name) {
            return null;
        }
        // Base case: Called the right node
        if (rest.length === 0) {
            return this;
        }
        // Recursion case: Traverse child nodes
        for (const child of this.children) {
            const f = child.getNode(rest);
            if (f) {
                return f;
            }
        }
        return null;
    }
    putNode(pathParts, node) {
        const [firstPart, ...rest] = pathParts;
        if (firstPart !== this.name) {
            return null;
        }
        // Base case: One path part left (which is the filename)
        if (rest.length === 1) {
            if (!this.nodeHasAlreadyChildFileWithSameName(rest) &&
                node.name === rest[0]) {
                this.addChild(node);
                return node;
            }
            return null;
        }
        // Case: We need to add directory, because it does not exist
        if (!this.nodeHasAlreadyChildDirectoryWithSameName(rest) &&
            rest[0] != null) {
            const newdir = new FileSystemDirectory(rest[0]);
            this.addChild(newdir);
            return newdir.putNode(rest, node);
        }
        // Recursion case: Traverse child nodes
        for (const child of this.children) {
            const f = child.putNode(rest, node);
            if (f) {
                return f;
            }
        }
        return null;
    }
    addChild(fileSystemNode) {
        this.children.push(fileSystemNode);
        return fileSystemNode;
    }
    nodeHasAlreadyChildDirectoryWithSameName(rest) {
        const children = this.children.filter((child) => child instanceof FileSystemDirectory && child.name === rest[0]);
        return children.length !== 0;
    }
    nodeHasAlreadyChildFileWithSameName(rest) {
        const children = this.children.filter((child) => child instanceof filesystem_node_file_1.FileSystemFile && child.name === rest[0]);
        return children.length !== 0;
    }
    toDirectoryString(indentation = 0) {
        return ('\t'.repeat(indentation) +
            this.name +
            '\n' +
            this.children
                .map((child) => child.toDirectoryString(indentation + 2))
                .join('\n'));
    }
}
exports.FileSystemDirectory = FileSystemDirectory;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-node-file-binary.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BinaryFile = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const filesystem_node_file_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file.ts");
class BinaryFile extends filesystem_node_file_1.FileSystemFile {
    constructor() {
        super(...arguments);
        this.ioType = jayvee_language_server_1.IOType.FILE;
    }
    acceptVisitor(visitor) {
        return visitor.visitBinaryFile(this);
    }
}
exports.BinaryFile = BinaryFile;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-node-file-text.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextFile = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const filesystem_node_file_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file.ts");
class TextFile extends filesystem_node_file_1.FileSystemFile {
    constructor() {
        super(...arguments);
        this.ioType = jayvee_language_server_1.IOType.TEXT_FILE;
    }
    acceptVisitor(visitor) {
        return visitor.visitTextFile(this);
    }
}
exports.TextFile = TextFile;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-node-file.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MimeType = exports.FileExtension = exports.FileSystemFile = void 0;
const filesystem_node_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node.ts");
class FileSystemFile extends filesystem_node_1.FileSystemNode {
    constructor(
    /**
     * The name of the file with extension
     * @property {string} name
     */
    name, 
    /**
     * The file extension in lower case, NONE / empty string for unknown or missing file extensions.
     * @property {FileExtension} extension
     */
    extension, 
    /**
     * The MIME type of the file taken from the Content-Type header (for HTTP requests only),
     * Otherwise inferred from the file extension, default application/octet-stream for unknown or missing file extensions.
     * @property {MimeType} mimeType
     */
    mimeType, content) {
        super(name);
        this.name = name;
        this.extension = extension;
        this.mimeType = mimeType;
        this.content = content;
    }
    getNode(pathParts) {
        const [firstPart, ...rest] = pathParts;
        if (firstPart === this.name && rest.length === 0) {
            return this;
        }
        return null;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    putNode(path) {
        return null;
    }
    toDirectoryString(indentation) {
        return ' '.repeat(indentation) + this.name + ` (${this.mimeType})`;
    }
}
exports.FileSystemFile = FileSystemFile;
/**
 * An enumeration of common file extensions. New extensions for Files need to be registered here.
 *
 * @enum {string}
 */
var FileExtension;
(function (FileExtension) {
    FileExtension["ZIP"] = "zip";
    FileExtension["TXT"] = "txt";
    FileExtension["CSV"] = "csv";
    FileExtension["NONE"] = "";
})(FileExtension = exports.FileExtension || (exports.FileExtension = {}));
/**
 * An enumeration of common MIME types.
 *
 * @enum {string}
 */
var MimeType;
(function (MimeType) {
    MimeType["APPLICATION_ZIP"] = "application/zip";
    MimeType["APPLICATION_OCTET_STREAM"] = "application/octet-stream";
    MimeType["TEXT_CSV"] = "text/csv";
    MimeType["TEXT_PLAIN"] = "text/plain";
})(MimeType = exports.MimeType || (exports.MimeType = {}));


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem-node.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.FileSystemNode = void 0;
class FileSystemNode {
    constructor(name) {
        this.name = name;
    }
}
exports.FileSystemNode = FileSystemNode;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/filesystem.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-inmemory.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/io-type-implementation.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/none.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/workbook.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/sheet.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/table.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file-binary.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-file-text.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/io-types/filesystem-node-directory.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/io-type-implementation.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/none.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.NONE = exports.None = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class None {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    constructor() {
        this.ioType = jayvee_language_server_1.IOType.NONE;
    }
    static get singletonInstance() {
        if (this._instance === undefined) {
            this._instance = new this();
        }
        return this._instance;
    }
    acceptVisitor(visitor) {
        return visitor.visitNone(this);
    }
}
exports.None = None;
exports.NONE = None.singletonInstance;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/sheet.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Sheet = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class Sheet {
    constructor(data) {
        this.data = data;
        this.ioType = jayvee_language_server_1.IOType.SHEET;
        this.numberOfRows = data.length;
        this.numberOfColumns = data.reduce((prev, curr) => {
            return curr.length > prev ? curr.length : prev;
        }, 0);
    }
    getData() {
        return this.data;
    }
    getNumberOfRows() {
        return this.numberOfRows;
    }
    getNumberOfColumns() {
        return this.numberOfColumns;
    }
    getHeaderRow() {
        (0, assert_1.strict)(this.getNumberOfRows() > 0, 'The sheet is expected to be non-empty and have a header row');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.data[0];
    }
    iterateRows(callbackFn) {
        this.data.forEach(callbackFn);
    }
    clone() {
        return new Sheet(structuredClone(this.data));
    }
    deleteRow(row) {
        (0, assert_1.strict)(this.isInBounds(row));
        row = this.resolveRelativeIndexes(row);
        const rowIndex = (0, jayvee_language_server_1.getRowIndex)(row);
        this.data.splice(rowIndex, 1);
        this.numberOfRows--;
    }
    deleteColumn(column) {
        (0, assert_1.strict)(this.isInBounds(column));
        column = this.resolveRelativeIndexes(column);
        const columnIndex = (0, jayvee_language_server_1.getColumnIndex)(column);
        this.data.forEach((row) => {
            row.splice(columnIndex, 1);
        });
        this.numberOfColumns--;
    }
    writeCell(absoluteCell, content) {
        (0, assert_1.strict)(absoluteCell.isInBounds(this.getBounds()));
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        this.data[absoluteCell.rowIndex][absoluteCell.columnIndex] = content;
    }
    selectRange(range) {
        (0, assert_1.strict)(this.isInBounds(range));
        range = this.resolveRelativeIndexes(range);
        this.data = this.data.reduce((previous, row, rowIndex) => {
            if (rowIndex < range.from.rowIndex || range.to.rowIndex < rowIndex) {
                return previous;
            }
            return [
                ...previous,
                row.slice(range.from.columnIndex, range.to.columnIndex + 1),
            ];
        }, []);
        this.numberOfColumns = range.to.columnIndex - range.from.columnIndex + 1;
        this.numberOfRows = range.to.rowIndex - range.from.rowIndex + 1;
    }
    isInBounds(range) {
        const bounds = this.getBounds();
        return range.isInBounds(bounds);
    }
    resolveRelativeIndexes(range) {
        if (range.hasRelativeIndexes()) {
            const bounds = this.getBounds();
            return range.resolveRelativeIndexes(bounds);
        }
        return range;
    }
    enumerateCellIndexes(range) {
        const resolvedRange = this.resolveRelativeIndexes(range);
        const result = [];
        for (let row = resolvedRange.from.rowIndex; row <= resolvedRange.to.rowIndex; ++row) {
            for (let column = resolvedRange.from.columnIndex; column <= resolvedRange.to.columnIndex; ++column) {
                result.push(new jayvee_language_server_1.CellIndex(column, row));
            }
        }
        return result;
    }
    getBounds() {
        return {
            lastColumnIndex: this.numberOfColumns - 1,
            lastRowIndex: this.numberOfRows - 1,
        };
    }
    acceptVisitor(visitor) {
        return visitor.visitSheet(this);
    }
}
exports.Sheet = Sheet;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/table.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Table = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const sql_column_type_visitor_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/sql-column-type-visitor.ts");
const sql_value_representation_visitor_1 = __webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/sql-value-representation-visitor.ts");
/**
 * Invariant: the shape of the table is always a rectangle.
 * This means all columns must have the same size.
 */
class Table {
    constructor(numberOfRows = 0) {
        this.ioType = jayvee_language_server_1.IOType.TABLE;
        this.numberOfRows = 0;
        this.columns = new Map();
        this.numberOfRows = numberOfRows;
    }
    addColumn(name, column) {
        (0, assert_1.strict)(column.values.length === this.numberOfRows);
        this.columns.set(name, column);
    }
    /**
     * Tries to add a new row to this table.
     * NOTE: This method will only add the row if the table has at least one column!
     * @param row data of this row for each column
     */
    addRow(row) {
        const rowLength = Object.keys(row).length;
        (0, assert_1.strict)(rowLength === this.columns.size, `Added row has the wrong dimension (expected: ${this.columns.size}, actual: ${rowLength})`);
        if (rowLength === 0) {
            return;
        }
        (0, assert_1.strict)(Object.keys(row).every((x) => this.hasColumn(x)), 'Added row does not fit the columns in the table');
        Object.entries(row).forEach(([columnName, value]) => {
            const column = this.columns.get(columnName);
            (0, assert_1.strict)(column !== undefined);
            (0, assert_1.strict)(column.valuetype.isInternalValueRepresentation(value));
            column.values.push(value);
        });
        this.numberOfRows++;
    }
    dropRow(rowId) {
        (0, assert_1.strict)(rowId < this.numberOfRows);
        this.columns.forEach((column) => {
            column.values.splice(rowId, 1);
        });
        this.numberOfRows--;
    }
    dropRows(rowIds) {
        rowIds
            .sort((a, b) => b - a) // delete descending to avoid messing up row indices
            .forEach((rowId) => {
            this.dropRow(rowId);
        });
    }
    getNumberOfRows() {
        return this.numberOfRows;
    }
    getNumberOfColumns() {
        return this.columns.size;
    }
    hasColumn(name) {
        return this.columns.has(name);
    }
    getColumns() {
        return this.columns;
    }
    getColumn(name) {
        return this.columns.get(name);
    }
    getRow(rowId) {
        const numberOfRows = this.getNumberOfRows();
        if (rowId >= numberOfRows) {
            throw new Error(`Trying to access table row ${rowId} (of ${numberOfRows} rows)`);
        }
        const row = new Map();
        [...this.columns.entries()].forEach(([columnName, column]) => {
            const value = column.values[rowId];
            (0, assert_1.strict)(value !== undefined);
            row.set(columnName, value);
        });
        return row;
    }
    static generateDropTableStatement(tableName) {
        return `DROP TABLE IF EXISTS "${tableName}";`;
    }
    generateInsertValuesStatement(tableName) {
        const valueRepresentationVisitor = new sql_value_representation_visitor_1.SQLValueRepresentationVisitor();
        const columnNames = [...this.columns.keys()];
        const formattedRowValues = [];
        for (let rowIndex = 0; rowIndex < this.numberOfRows; ++rowIndex) {
            const rowValues = [];
            for (const columnName of columnNames) {
                const column = this.columns.get(columnName);
                const entry = column === null || column === void 0 ? void 0 : column.values[rowIndex];
                (0, assert_1.strict)(entry !== undefined);
                const formattedValue = column === null || column === void 0 ? void 0 : column.valuetype.acceptVisitor(valueRepresentationVisitor)(entry);
                (0, assert_1.strict)(formattedValue !== undefined);
                rowValues.push(formattedValue);
            }
            formattedRowValues.push(`(${rowValues.join(',')})`);
        }
        const formattedColumns = columnNames.map((c) => `"${c}"`).join(',');
        return `INSERT INTO "${tableName}" (${formattedColumns}) VALUES ${formattedRowValues.join(', ')}`;
    }
    generateCreateTableStatement(tableName) {
        const columnTypeVisitor = new sql_column_type_visitor_1.SQLColumnTypeVisitor();
        const columns = [...this.columns.entries()];
        const columnStatements = columns.map(([columnName, column]) => {
            return `"${columnName}" ${column.valuetype.acceptVisitor(columnTypeVisitor)}`;
        });
        return `CREATE TABLE IF NOT EXISTS "${tableName}" (${columnStatements.join(',')});`;
    }
    clone() {
        const cloned = new Table();
        cloned.numberOfRows = this.numberOfRows;
        [...this.columns.entries()].forEach(([columnName, column]) => {
            cloned.addColumn(columnName, {
                values: structuredClone(column.values),
                valuetype: column.valuetype,
            });
        });
        return cloned;
    }
    acceptVisitor(visitor) {
        return visitor.visitTable(this);
    }
}
exports.Table = Table;


/***/ }),

/***/ "./libs/execution/src/lib/types/io-types/workbook.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Workbook = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const sheet_1 = __webpack_require__("./libs/execution/src/lib/types/io-types/sheet.ts");
class Workbook {
    constructor() {
        this.ioType = jayvee_language_server_1.IOType.WORKBOOK;
        this.sheets = new Map();
    }
    getSheets() {
        return this.sheets;
    }
    getSheetByName(sheetName) {
        return this.sheets.get(sheetName);
    }
    getUniqueDefaultNameForNewSheet() {
        let defaultSheetNumber = this.sheets.size;
        let defaultName = `Sheet${++defaultSheetNumber}`;
        while (this.getSheetByName(defaultName) !== undefined) {
            defaultName = `Sheet${++defaultSheetNumber}`;
        }
        return defaultName;
    }
    addSheet(data, sheetName) {
        const sheetNameOrDefault = sheetName !== null && sheetName !== void 0 ? sheetName : this.getUniqueDefaultNameForNewSheet();
        if (this.sheets.get(sheetNameOrDefault) !== undefined) {
            throw new Error(`Sheet with name ${sheetNameOrDefault} already exists in Workbook.`);
        }
        this.sheets.set(sheetNameOrDefault, new sheet_1.Sheet(data));
        return this;
    }
    acceptVisitor(visitor) {
        return visitor.visitWorkbook(this);
    }
}
exports.Workbook = Workbook;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/internal-representation-parsing.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/value-representation-validity.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/internal-representation-parsing.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.parseValueToInternalRepresentation = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const NUMBER_REGEX = /^[+-]?([0-9]*[,.])?[0-9]+([eE][+-]?\d+)?$/;
const TRUE_REGEX = /^true$/i;
const FALSE_REGEX = /^false$/i;
function parseValueToInternalRepresentation(value, valuetype) {
    const visitor = new InternalRepresentationParserVisitor(value);
    const result = valuetype.acceptVisitor(visitor);
    if (!valuetype.isInternalValueRepresentation(result)) {
        return undefined;
    }
    return result;
}
exports.parseValueToInternalRepresentation = parseValueToInternalRepresentation;
class InternalRepresentationParserVisitor extends jayvee_language_server_1.ValuetypeVisitor {
    constructor(value) {
        super();
        this.value = value;
    }
    visitBoolean() {
        if (TRUE_REGEX.test(this.value)) {
            return true;
        }
        else if (FALSE_REGEX.test(this.value)) {
            return false;
        }
        return undefined;
    }
    visitDecimal() {
        if (!NUMBER_REGEX.test(this.value)) {
            return undefined;
        }
        return Number.parseFloat(this.value.replace(',', '.'));
    }
    visitInteger() {
        /**
         * Reuse decimal number parsing to capture valid scientific notation
         * of integers like 5.3e3 = 5300. In contrast to decimal, if the final number
         * is not a valid integer, returns undefined.
         */
        const decimalNumber = this.visitDecimal();
        if (decimalNumber === undefined) {
            return undefined;
        }
        const integerNumber = Math.trunc(decimalNumber);
        if (decimalNumber !== integerNumber) {
            return undefined;
        }
        return integerNumber;
    }
    visitText() {
        return this.value;
    }
    visitAtomicValuetype(valuetype) {
        const supertype = valuetype.getSupertype();
        (0, assert_1.strict)(supertype !== undefined);
        return supertype.acceptVisitor(this);
    }
    visitCellRange() {
        return undefined;
    }
    visitCollection() {
        return undefined;
    }
    visitConstraint() {
        return undefined;
    }
    visitRegex() {
        return undefined;
    }
    visitTransform() {
        return undefined;
    }
    visitValuetypeAssignment() {
        return undefined;
    }
}


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/value-representation-validity.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isValidValueRepresentation = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const constraint_executor_registry_1 = __webpack_require__("./libs/execution/src/lib/constraints/constraint-executor-registry.ts");
function isValidValueRepresentation(value, valuetype, context) {
    const visitor = new ValueRepresentationValidityVisitor(value, context);
    return valuetype.acceptVisitor(visitor);
}
exports.isValidValueRepresentation = isValidValueRepresentation;
class ValueRepresentationValidityVisitor extends jayvee_language_server_1.ValuetypeVisitor {
    constructor(value, context) {
        super();
        this.value = value;
        this.context = context;
    }
    visitAtomicValuetype(valuetype) {
        const supertype = valuetype.getSupertype();
        (0, assert_1.strict)(supertype !== undefined);
        if (!supertype.acceptVisitor(this)) {
            return false;
        }
        const constraints = valuetype.getConstraints(this.context.evaluationContext);
        for (const constraint of constraints) {
            const constraintExecutor = (0, constraint_executor_registry_1.createConstraintExecutor)(constraint);
            this.context.enterNode(constraint);
            const valueFulfilledConstraint = constraintExecutor.isValid(this.value, this.context);
            this.context.exitNode(constraint);
            if (!valueFulfilledConstraint) {
                return false;
            }
        }
        return true;
    }
    visitBoolean(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitDecimal(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitInteger(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitText(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitRegex(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitCellRange(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitConstraint(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitValuetypeAssignment(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitCollection(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    visitTransform(valuetype) {
        return this.isValidForPrimitiveValuetype(valuetype);
    }
    isValidForPrimitiveValuetype(valuetype) {
        return valuetype.isInternalValueRepresentation(this.value);
    }
}


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/visitors/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/sql-column-type-visitor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/types/valuetypes/visitors/sql-value-representation-visitor.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/visitors/sql-column-type-visitor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLColumnTypeVisitor = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class SQLColumnTypeVisitor extends jayvee_language_server_1.ValuetypeVisitor {
    visitBoolean() {
        return 'boolean';
    }
    visitDecimal() {
        return 'real';
    }
    visitInteger() {
        return 'integer';
    }
    visitText() {
        return 'text';
    }
    visitAtomicValuetype(valuetype) {
        const supertype = valuetype.getSupertype();
        (0, assert_1.strict)(supertype !== undefined);
        return supertype.acceptVisitor(this);
    }
    visitRegex() {
        throw new Error('No visit implementation given for regex. Cannot be the type of a column.');
    }
    visitCellRange() {
        throw new Error('No visit implementation given for cell ranges. Cannot be the type of a column.');
    }
    visitConstraint() {
        throw new Error('No visit implementation given for constraints. Cannot be the type of a column.');
    }
    visitValuetypeAssignment() {
        throw new Error('No visit implementation given for valuetype assignments. Cannot be the type of a column.');
    }
    visitCollection() {
        throw new Error('No visit implementation given for collections. Cannot be the type of a column.');
    }
    visitTransform() {
        throw new Error('No visit implementation given for transforms. Cannot be the type of a column.');
    }
}
exports.SQLColumnTypeVisitor = SQLColumnTypeVisitor;


/***/ }),

/***/ "./libs/execution/src/lib/types/valuetypes/visitors/sql-value-representation-visitor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLValueRepresentationVisitor = void 0;
const assert_1 = __webpack_require__("assert");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
class SQLValueRepresentationVisitor extends jayvee_language_server_1.ValuetypeVisitor {
    visitBoolean(valuetype) {
        return (value) => {
            (0, assert_1.strict)(valuetype.isInternalValueRepresentation(value));
            return value ? `'true'` : `'false'`;
        };
    }
    visitDecimal(valuetype) {
        return (value) => {
            (0, assert_1.strict)(valuetype.isInternalValueRepresentation(value));
            return value.toString();
        };
    }
    visitInteger(valuetype) {
        return (value) => {
            (0, assert_1.strict)(valuetype.isInternalValueRepresentation(value));
            return value.toString();
        };
    }
    visitText(valuetype) {
        return (value) => {
            (0, assert_1.strict)(valuetype.isInternalValueRepresentation(value));
            const escapedValue = escapeSingleQuotes(value);
            return `'${escapedValue}'`;
        };
    }
    visitAtomicValuetype(valuetype) {
        const supertype = valuetype.getSupertype();
        (0, assert_1.strict)(supertype !== undefined);
        return supertype.acceptVisitor(this);
    }
    visitRegex() {
        throw new Error('No visit implementation given for regex. Cannot be the type of a column.');
    }
    visitCellRange() {
        throw new Error('No visit implementation given for cell ranges. Cannot be the type of a column.');
    }
    visitConstraint() {
        throw new Error('No visit implementation given for constraints. Cannot be the type of a column.');
    }
    visitValuetypeAssignment() {
        throw new Error('No visit implementation given for valuetype assignments. Cannot be the type of a column.');
    }
    visitCollection() {
        throw new Error('No visit implementation given for collections. Cannot be the type of a column.');
    }
    visitTransform() {
        throw new Error('No visit implementation given for transforms. Cannot be the type of a column.');
    }
}
exports.SQLValueRepresentationVisitor = SQLValueRepresentationVisitor;
function escapeSingleQuotes(value) {
    return value.replace(/'/g, `''`);
}


/***/ }),

/***/ "./libs/execution/src/lib/util/file-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.inferFileExtensionFromContentTypeString = exports.inferFileExtensionFromFileExtensionString = exports.inferMimeTypeFromFileExtensionString = void 0;
const mime = __webpack_require__("mime-types");
const types_1 = __webpack_require__("./libs/execution/src/lib/types/index.ts");
function inferMimeTypeFromFileExtensionString(fileExtension) {
    if (fileExtension !== undefined) {
        const inferredMimeType = mime.lookup(fileExtension);
        if (Object.values(types_1.MimeType).includes(inferredMimeType)) {
            return inferredMimeType;
        }
    }
    return undefined;
}
exports.inferMimeTypeFromFileExtensionString = inferMimeTypeFromFileExtensionString;
function inferFileExtensionFromFileExtensionString(fileExtension) {
    if (fileExtension !== undefined) {
        const inferredFileExtension = fileExtension.replace('.', '');
        if (Object.values(types_1.FileExtension).includes(inferredFileExtension)) {
            return inferredFileExtension;
        }
    }
    return undefined;
}
exports.inferFileExtensionFromFileExtensionString = inferFileExtensionFromFileExtensionString;
function inferFileExtensionFromContentTypeString(contentType) {
    if (contentType !== undefined) {
        const inferredFileExtension = mime.extension(contentType);
        if (inferredFileExtension !== false) {
            if (Object.values(types_1.FileExtension).includes(inferredFileExtension)) {
                return inferredFileExtension;
            }
        }
    }
    return undefined;
}
exports.inferFileExtensionFromContentTypeString = inferFileExtensionFromContentTypeString;


/***/ }),

/***/ "./libs/execution/src/lib/util/implements-static-decorator.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.implementsStatic = void 0;
/**
 * Decorator for classes, so they need to implement a given interface using static members / functions.
 *
 * @example
 * interface SampleInterface {
 *     a: string
 *     foo(): void;
 * }
 * @implementsStatic<SampleInterface>()
 * class SampleClass {
 *     static a: string = 'A';
 *     static foo(): void {
 *         return;
 *     }
 * }
 */
function implementsStatic() {
    return (constructor) => {
        constructor;
    };
}
exports.implementsStatic = implementsStatic;


/***/ }),

/***/ "./libs/execution/src/lib/util/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/util/implements-static-decorator.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/util/file-util.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/execution/src/lib/util/string-util.ts"), exports);


/***/ }),

/***/ "./libs/execution/src/lib/util/string-util.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.splitLines = void 0;
function splitLines(textContent, lineBreak) {
    const lines = textContent.split(lineBreak);
    // There may be an additional empty line due to the previous splitting
    if (lines[lines.length - 1] === '') {
        lines.splice(lines.length - 1, 1);
    }
    return lines;
}
exports.splitLines = splitLines;


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RdbmsExecExtension = void 0;
const lib_1 = __webpack_require__("./libs/extensions/rdbms/exec/src/lib/index.ts");
class RdbmsExecExtension {
    getBlockExecutors() {
        return [lib_1.PostgresLoaderExecutor, lib_1.SQLiteLoaderExecutor];
    }
}
exports.RdbmsExecExtension = RdbmsExecExtension;


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/exec/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/exec/src/lib/postgres-loader-executor.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/extensions/rdbms/exec/src/lib/sqlite-loader-executor.ts"), exports);


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/lib/postgres-loader-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PostgresLoaderExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const pg_1 = __webpack_require__("pg");
let PostgresLoaderExecutor = class PostgresLoaderExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.TABLE, jayvee_language_server_1.IOType.NONE);
    }
    doExecute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const host = context.getPropertyValue('host', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const port = context.getPropertyValue('port', jayvee_language_server_1.PrimitiveValuetypes.Integer);
            const user = context.getPropertyValue('username', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const password = context.getPropertyValue('password', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const database = context.getPropertyValue('database', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const table = context.getPropertyValue('table', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const client = new pg_1.Client({
                host,
                port,
                user,
                password,
                database,
            });
            try {
                context.logger.logDebug(`Connecting to database`);
                yield client.connect();
                context.logger.logDebug(`Dropping previous table "${table}" if it exists`);
                yield client.query(jayvee_execution_1.Table.generateDropTableStatement(table));
                context.logger.logDebug(`Creating table "${table}"`);
                yield client.query(input.generateCreateTableStatement(table));
                context.logger.logDebug(`Inserting ${input.getNumberOfRows()} row(s) into table "${table}"`);
                yield client.query(input.generateInsertValuesStatement(table));
                context.logger.logDebug(`The data was successfully loaded into the database`);
                return R.ok(jayvee_execution_1.NONE);
            }
            catch (err) {
                return R.err({
                    message: `Could not write to postgres database: ${err instanceof Error ? err.message : JSON.stringify(err)}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
            finally {
                yield client.end();
            }
        });
    }
};
PostgresLoaderExecutor.type = 'PostgresLoader';
PostgresLoaderExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], PostgresLoaderExecutor);
exports.PostgresLoaderExecutor = PostgresLoaderExecutor;


/***/ }),

/***/ "./libs/extensions/rdbms/exec/src/lib/sqlite-loader-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SQLiteLoaderExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const sqlite3 = __webpack_require__("sqlite3");
let SQLiteLoaderExecutor = class SQLiteLoaderExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.TABLE, jayvee_language_server_1.IOType.NONE);
    }
    doExecute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const file = context.getPropertyValue('file', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const table = context.getPropertyValue('table', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const dropTable = context.getPropertyValue('dropTable', jayvee_language_server_1.PrimitiveValuetypes.Boolean);
            let db;
            try {
                context.logger.logDebug(`Opening database file ${file}`);
                db = new sqlite3.Database(file);
                if (dropTable) {
                    context.logger.logDebug(`Dropping previous table "${table}" if it exists`);
                    yield this.runQuery(db, jayvee_execution_1.Table.generateDropTableStatement(table));
                }
                context.logger.logDebug(`Creating table "${table}"`);
                yield this.runQuery(db, input.generateCreateTableStatement(table));
                context.logger.logDebug(`Inserting ${input.getNumberOfRows()} row(s) into table "${table}"`);
                yield this.runQuery(db, input.generateInsertValuesStatement(table));
                context.logger.logDebug(`The data was successfully loaded into the database`);
                return R.ok(jayvee_execution_1.NONE);
            }
            catch (err) {
                return R.err({
                    message: `Could not write to sqlite database: ${err instanceof Error ? err.message : JSON.stringify(err)}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
            finally {
                db === null || db === void 0 ? void 0 : db.close();
            }
        });
    }
    runQuery(db, query) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                db.run(query, (result, error) => error ? reject(error) : resolve(result));
            });
        });
    }
};
SQLiteLoaderExecutor.type = 'SQLiteLoader';
SQLiteLoaderExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], SQLiteLoaderExecutor);
exports.SQLiteLoaderExecutor = SQLiteLoaderExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/archive-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ArchiveInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const zlib = __webpack_require__("node:zlib");
const path = __webpack_require__("path");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const JSZip = __webpack_require__("jszip");
let ArchiveInterpreterExecutor = class ArchiveInterpreterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.FILE, jayvee_language_server_1.IOType.FILE_SYSTEM);
    }
    doExecute(archiveFile, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const archiveType = context.getPropertyValue('archiveType', jayvee_language_server_1.PrimitiveValuetypes.Text);
            let fs;
            if (archiveType === 'zip') {
                fs = yield this.loadZipFileToInMemoryFileSystem(archiveFile, context);
            }
            else if (archiveType === 'gz') {
                fs = this.loadGzFileToInMemoryFileSystem(archiveFile, context);
            }
            else {
                return R.err({
                    message: `Archive type is not supported`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
            if (R.isErr(fs)) {
                return fs;
            }
            return R.ok(fs.right);
        });
    }
    loadGzFileToInMemoryFileSystem(archiveFile, context) {
        context.logger.logDebug(`Loading gz file from binary content`);
        try {
            const fs = new jayvee_execution_1.InMemoryFileSystem();
            const archivedObject = zlib.gunzipSync(archiveFile.content);
            const extNameArchive = path.extname(archiveFile.name);
            const file = this.createFileFromArchive(archiveFile.name, archivedObject, extNameArchive);
            const addedFile = fs.putFile(jayvee_execution_1.InMemoryFileSystem.getPathSeparator() + file.name, file);
            (0, assert_1.strict)(addedFile != null);
            return R.ok(fs);
        }
        catch (error) {
            return R.err(this.generateErrorObject(context, error));
        }
    }
    loadZipFileToInMemoryFileSystem(archiveFile, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            context.logger.logDebug(`Loading zip file from binary content`);
            try {
                const jszip = JSZip();
                const fs = new jayvee_execution_1.InMemoryFileSystem();
                const archivedObjects = yield jszip.loadAsync(archiveFile.content);
                for (const [relPath, archivedObject] of Object.entries(archivedObjects.files)) {
                    if (!archivedObject.dir) {
                        const content = yield archivedObject.async('arraybuffer');
                        const file = this.createFileFromArchive(archivedObject.name, content);
                        const addedFile = fs.putFile(jayvee_execution_1.InMemoryFileSystem.getPathSeparator() + relPath, file);
                        (0, assert_1.strict)(addedFile != null);
                    }
                }
                return R.ok(fs);
            }
            catch (error) {
                return R.err(this.generateErrorObject(context, error));
            }
        });
    }
    createFileFromArchive(archiveFileName, content, extNameArchive) {
        const fileName = path.basename(archiveFileName, extNameArchive);
        const extName = path.extname(fileName);
        const mimeType = (0, jayvee_execution_1.inferMimeTypeFromFileExtensionString)(extName) ||
            jayvee_execution_1.MimeType.APPLICATION_OCTET_STREAM;
        const fileExtension = (0, jayvee_execution_1.inferFileExtensionFromFileExtensionString)(extName) || jayvee_execution_1.FileExtension.NONE;
        return new jayvee_execution_1.BinaryFile(fileName, fileExtension, mimeType, content);
    }
    generateErrorObject(context, error) {
        return {
            message: `Unexpected Error ${error instanceof Error ? error.message : JSON.stringify(jayvee_execution_1.err)} occured during processing`,
            diagnostic: { node: context.getCurrentNode(), property: 'name' },
        };
    }
};
ArchiveInterpreterExecutor.type = 'ArchiveInterpreter';
ArchiveInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], ArchiveInterpreterExecutor);
exports.ArchiveInterpreterExecutor = ArchiveInterpreterExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.StdExecExtension = void 0;
const exec_1 = __webpack_require__("./libs/extensions/rdbms/exec/src/index.ts");
const exec_2 = __webpack_require__("./libs/extensions/tabular/exec/src/index.ts");
const archive_interpreter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/archive-interpreter-executor.ts");
const file_picker_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/file-picker-executor.ts");
const gtfs_rt_interpreter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/gtfs-rt-interpreter-executor.ts");
const http_extractor_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/http-extractor-executor.ts");
const text_file_interpreter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/text-file-interpreter-executor.ts");
const text_line_deleter_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/text-line-deleter-executor.ts");
const text_range_selector_executor_1 = __webpack_require__("./libs/extensions/std/exec/src/text-range-selector-executor.ts");
class StdExecExtension {
    constructor() {
        this.wrappedExtensions = [
            new exec_2.TabularExecExtension(),
            new exec_1.RdbmsExecExtension(),
        ];
    }
    getBlockExecutors() {
        return [
            ...this.wrappedExtensions.map((x) => x.getBlockExecutors()).flat(),
            http_extractor_executor_1.HttpExtractorExecutor,
            text_file_interpreter_executor_1.TextFileInterpreterExecutor,
            text_range_selector_executor_1.TextRangeSelectorExecutor,
            text_line_deleter_executor_1.TextLineDeleterExecutor,
            archive_interpreter_executor_1.ArchiveInterpreterExecutor,
            file_picker_executor_1.FilePickerExecutor,
            gtfs_rt_interpreter_executor_1.GtfsRTInterpreterExecutor,
        ];
    }
}
exports.StdExecExtension = StdExecExtension;


/***/ }),

/***/ "./libs/extensions/std/exec/src/file-picker-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.FilePickerExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let FilePickerExecutor = class FilePickerExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.FILE_SYSTEM, jayvee_language_server_1.IOType.FILE);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(fileSystem, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const path = context.getPropertyValue('path', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const file = fileSystem.getFile(path);
            if (file == null) {
                return R.err({
                    message: `File '${path}' not found`,
                    diagnostic: { node: context.getCurrentNode() },
                });
            }
            (0, assert_1.strict)(file instanceof jayvee_execution_1.BinaryFile);
            return R.ok(file);
        });
    }
};
FilePickerExecutor.type = 'FilePicker';
FilePickerExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], FilePickerExecutor);
exports.FilePickerExecutor = FilePickerExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/gtfs-rt-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.GtfsRTInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const E = __webpack_require__("fp-ts/lib/Either");
const GtfsRealtimeBindings = __webpack_require__("gtfs-realtime-bindings");
let GtfsRTInterpreterExecutor = class GtfsRTInterpreterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.FILE, jayvee_language_server_1.IOType.SHEET);
    }
    doExecute(inputFile, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // Accessing attribute values by their name:
            const entity = context.getPropertyValue('entity', jayvee_language_server_1.PrimitiveValuetypes.Text);
            // https://github.com/MobilityData/gtfs-realtime-bindings/tree/master/nodejs
            let feedMessage;
            try {
                feedMessage = GtfsRealtimeBindings.transit_realtime.FeedMessage.decode(new Uint8Array(inputFile.content));
            }
            catch (e) {
                return Promise.resolve(R.err({
                    message: `Failed to decode gtfs file: ${this.getErrorMessage(e)}`,
                    diagnostic: { node: context.getCurrentNode() },
                }));
            }
            // Parse all possible feedentity to Sheet
            const sheet = yield this.parseFeedMessage(entity, feedMessage, context);
            if (E.isLeft(sheet)) {
                return Promise.resolve(R.err({
                    message: sheet.left.message,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                }));
            }
            return R.ok(sheet.right);
        });
    }
    getErrorMessage(e) {
        if (e instanceof Error) {
            return e.message;
        }
        return String(e);
    }
    parseFeedMessage(entityType, feedMessage, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            switch (entityType) {
                case 'trip_update':
                    // Extract the trip updates from thee feed message
                    return this.parseTripUpdates(feedMessage, context);
                case 'alert':
                    // Extract the alerts from thee feed message
                    return this.parseAlerts(feedMessage, context);
                case 'vehicle':
                    // Extract vehicle positions from thee feed message
                    return this.parseVehiclePositions(feedMessage, context);
                // Entity invalid
                default:
                    return E.left(new Error(`Entity ${entityType} not allowed for block GtfsRTInterpreterblock, expected "trip_update", "alert" or "vehicle".`));
            }
        });
    }
    parseTripUpdates(feedMessage, context) {
        return new Promise((resolve) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t;
            context.logger.logDebug(`Parsing raw gtfs-rt feed data as TripUpdate"`);
            const rows = [];
            // Add Header
            rows.push([...tripUpdateHeader]);
            for (const entity of feedMessage.entity) {
                // Case: No TripUpdates found -> return sheet just with header
                if (!entity.tripUpdate) {
                    context.logger.logDebug(`Parsing gtfs-rt feed data as TripUpdates: No Tripupdates found in feedmessage with timestamp "${String(feedMessage.header.timestamp)}"`);
                    break;
                }
                // Case: No stopTimeUpdate found -> continue with next TripUpdate
                if (!entity.tripUpdate.stopTimeUpdate) {
                    context.logger.logDebug(`Parsing gtfs-rt feed data as TripUpdates: StopTimeUpdate of TripUpdate "${String(entity.tripUpdate.trip.tripId)}" does not contain a single entry. Skipping this TripUpdate and continue with next one"`);
                    continue;
                }
                for (const stopTimeUpdate of entity.tripUpdate.stopTimeUpdate) {
                    const row = {
                        'header.gtfs_realtime_version': feedMessage.header.gtfsRealtimeVersion,
                        'header.timestamp': (_b = (_a = feedMessage.header.timestamp) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '',
                        'header.incrementality': (_d = (_c = feedMessage.header.incrementality) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : '',
                        'entity.id': entity.id,
                        'entity.trip_update.trip.trip_id': (_f = (_e = entity.tripUpdate.trip.tripId) === null || _e === void 0 ? void 0 : _e.toString()) !== null && _f !== void 0 ? _f : '',
                        'entity.trip_update.trip.route_id': (_h = (_g = entity.tripUpdate.trip.routeId) === null || _g === void 0 ? void 0 : _g.toString()) !== null && _h !== void 0 ? _h : '',
                        'entity.trip_update.stop_time_update.stop_sequence': (_k = (_j = stopTimeUpdate.stopSequence) === null || _j === void 0 ? void 0 : _j.toString()) !== null && _k !== void 0 ? _k : '',
                        'entity.trip_update.stop_time_update.stop_id': (_m = (_l = stopTimeUpdate.stopId) === null || _l === void 0 ? void 0 : _l.toString()) !== null && _m !== void 0 ? _m : '',
                        'entity.trip_update.stop_time_update.arrival.time': (_q = (_p = (_o = stopTimeUpdate.arrival) === null || _o === void 0 ? void 0 : _o.time) === null || _p === void 0 ? void 0 : _p.toString()) !== null && _q !== void 0 ? _q : '',
                        'entity.trip_update.stop_time_update.departure.time': (_t = (_s = (_r = stopTimeUpdate.departure) === null || _r === void 0 ? void 0 : _r.time) === null || _s === void 0 ? void 0 : _s.toString()) !== null && _t !== void 0 ? _t : '',
                    };
                    rows.push(Object.values(row));
                }
            }
            resolve(E.right(new jayvee_execution_1.Sheet(rows)));
        });
    }
    parseVehiclePositions(feedMessage, context) {
        return new Promise((resolve) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u;
            context.logger.logDebug(`Parsing raw gtfs-rt feed data as VehiclePosition"`);
            const rows = [];
            // Add header
            rows.push([...vehiclePositionHeader]);
            for (const entity of feedMessage.entity) {
                // Case: No VehiclePositions found -> return sheet just with header
                if (!entity.vehicle) {
                    context.logger.logDebug(`Parsing gtfs-rt feed data as VehiclePosition: No VehiclePositions found in feedmessage with timestamp "${String(feedMessage.header.timestamp)}"`);
                    break;
                }
                const row = {
                    'header.gtfs_realtime_version': feedMessage.header.gtfsRealtimeVersion,
                    'header.timestamp': (_b = (_a = feedMessage.header.timestamp) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '',
                    'header.incrementality': (_d = (_c = feedMessage.header.incrementality) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : '',
                    'entity.id': String(entity.id),
                    'entity.vehicle_position.vehicle_descriptor.id': (_g = (_f = (_e = entity.vehicle.vehicle) === null || _e === void 0 ? void 0 : _e.id) === null || _f === void 0 ? void 0 : _f.toString()) !== null && _g !== void 0 ? _g : '',
                    'entity.vehicle_position.trip.trip_id': (_k = (_j = (_h = entity.vehicle.trip) === null || _h === void 0 ? void 0 : _h.tripId) === null || _j === void 0 ? void 0 : _j.toString()) !== null && _k !== void 0 ? _k : '',
                    'entity.vehicle_position.trip.route_id': (_o = (_m = (_l = entity.vehicle.trip) === null || _l === void 0 ? void 0 : _l.routeId) === null || _m === void 0 ? void 0 : _m.toString()) !== null && _o !== void 0 ? _o : '',
                    'entity.vehicle_position.position.latitude': (_q = (_p = entity.vehicle.position) === null || _p === void 0 ? void 0 : _p.latitude.toString()) !== null && _q !== void 0 ? _q : '',
                    'entity.vehicle_position.position.longitude': (_s = (_r = entity.vehicle.position) === null || _r === void 0 ? void 0 : _r.longitude.toString()) !== null && _s !== void 0 ? _s : '',
                    'entity.vehicle_position.timestamp': (_u = (_t = entity.vehicle.timestamp) === null || _t === void 0 ? void 0 : _t.toString()) !== null && _u !== void 0 ? _u : '',
                };
                rows.push(Object.values(row));
            }
            resolve(E.right(new jayvee_execution_1.Sheet(rows)));
        });
    }
    parseAlerts(feedMessage, context) {
        return new Promise((resolve) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
            context.logger.logDebug(`Parsing raw gtfs-rt feed data as Alerts"`);
            const rows = [];
            // Add header
            rows.push([...alertHeader]);
            for (const entity of feedMessage.entity) {
                // Case: No Alerts found -> return sheet just with header
                if (!entity.alert) {
                    context.logger.logDebug(`Parsing gtfs-rt feed data as Alert: No Alerts found in feedmessage with timestamp "${String(feedMessage.header.timestamp)}"`);
                    break;
                }
                if (!entity.alert.informedEntity) {
                    context.logger.logDebug(`Parsing gtfs-rt feed data as Alert: InformedEntity of Alert does not contain a single entry. Skipping this Alert and continue with next one"`);
                    continue;
                }
                for (const informedEntity of entity.alert.informedEntity) {
                    const row = {
                        'header.gtfs_realtime_version': feedMessage.header.gtfsRealtimeVersion,
                        'header.timestamp': (_b = (_a = feedMessage.header.timestamp) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '',
                        'header.incrementality': (_d = (_c = feedMessage.header.incrementality) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : '',
                        'entity.id': entity.id.toString(),
                        'entity.alert.informed_entity.route_id': (_f = (_e = informedEntity.routeId) === null || _e === void 0 ? void 0 : _e.toString()) !== null && _f !== void 0 ? _f : '',
                        'entity.alert.header_text': (_k = (_j = (_h = (_g = entity.alert.headerText) === null || _g === void 0 ? void 0 : _g.translation) === null || _h === void 0 ? void 0 : _h.shift()) === null || _j === void 0 ? void 0 : _j.text.toString()) !== null && _k !== void 0 ? _k : '',
                        'entity.alert.description_text': (_p = (_o = (_m = (_l = entity.alert.descriptionText) === null || _l === void 0 ? void 0 : _l.translation) === null || _m === void 0 ? void 0 : _m.shift()) === null || _o === void 0 ? void 0 : _o.text.toString()) !== null && _p !== void 0 ? _p : '',
                    };
                    rows.push(Object.values(row));
                }
            }
            resolve(E.right(new jayvee_execution_1.Sheet(rows)));
        });
    }
};
GtfsRTInterpreterExecutor.type = 'GtfsRTInterpreter';
GtfsRTInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], GtfsRTInterpreterExecutor);
exports.GtfsRTInterpreterExecutor = GtfsRTInterpreterExecutor;
const tripUpdateHeader = [
    'header.gtfs_realtime_version',
    'header.timestamp',
    'header.incrementality',
    'entity.id',
    'entity.trip_update.trip.trip_id',
    'entity.trip_update.trip.route_id',
    'entity.trip_update.stop_time_update.stop_sequence',
    'entity.trip_update.stop_time_update.stop_id',
    'entity.trip_update.stop_time_update.arrival.time',
    'entity.trip_update.stop_time_update.departure.time',
];
const vehiclePositionHeader = [
    'header.gtfs_realtime_version',
    'header.timestamp',
    'header.incrementality',
    'entity.id',
    'entity.vehicle_position.vehicle_descriptor.id',
    'entity.vehicle_position.trip.trip_id',
    'entity.vehicle_position.trip.route_id',
    'entity.vehicle_position.position.latitude',
    'entity.vehicle_position.position.longitude',
    'entity.vehicle_position.timestamp',
];
const alertHeader = [
    'header.gtfs_realtime_version',
    'header.timestamp',
    'header.incrementality',
    'entity.id',
    'entity.alert.informed_entity.route_id',
    'entity.alert.header_text',
    'entity.alert.description_text',
];


/***/ }),

/***/ "./libs/extensions/std/exec/src/http-extractor-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.HttpExtractorExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const path = __webpack_require__("path");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const follow_redirects_1 = __webpack_require__("follow-redirects");
const backoff_strategy_1 = __webpack_require__("./libs/extensions/std/exec/src/util/backoff-strategy.ts");
let HttpExtractorExecutor = class HttpExtractorExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.NONE, jayvee_language_server_1.IOType.FILE);
    }
    doExecute(input, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const url = context.getPropertyValue('url', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const retries = context.getPropertyValue('retries', jayvee_language_server_1.PrimitiveValuetypes.Integer);
            (0, assert_1.strict)(retries >= 0); // loop executes at least once
            const retryBackoffMilliseconds = context.getPropertyValue('retryBackoffMilliseconds', jayvee_language_server_1.PrimitiveValuetypes.Integer);
            const retryBackoffStrategy = context.getPropertyValue('retryBackoffStrategy', jayvee_language_server_1.PrimitiveValuetypes.Text);
            (0, assert_1.strict)((0, backoff_strategy_1.isBackoffStrategyHandle)(retryBackoffStrategy));
            const backoffStrategy = (0, backoff_strategy_1.createBackoffStrategy)(retryBackoffStrategy, retryBackoffMilliseconds);
            let failure;
            for (let attempt = 0; attempt <= retries; ++attempt) {
                const isLastAttempt = attempt === retries;
                const file = yield this.fetchRawDataAsFile(url, context);
                if (R.isOk(file)) {
                    context.logger.logDebug(`Successfully fetched raw data`);
                    return R.ok(file.right);
                }
                failure = file.left;
                if (!isLastAttempt) {
                    context.logger.logDebug(failure.message);
                    const currentBackoff = backoffStrategy.getBackoffMilliseconds(attempt + 1);
                    context.logger.logDebug(`Waiting ${currentBackoff}ms before trying again...`);
                    yield new Promise((p) => setTimeout(p, currentBackoff));
                    continue;
                }
            }
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            return R.err(failure);
        });
    }
    fetchRawDataAsFile(url, context) {
        context.logger.logDebug(`Fetching raw data from ${url}`);
        let httpGetFunction;
        if (url.startsWith('https')) {
            httpGetFunction = follow_redirects_1.https.get.bind(follow_redirects_1.https);
        }
        else {
            httpGetFunction = follow_redirects_1.http.get.bind(follow_redirects_1.http);
        }
        const followRedirects = context.getPropertyValue('followRedirects', jayvee_language_server_1.PrimitiveValuetypes.Boolean);
        return new Promise((resolve) => {
            httpGetFunction(url, { followRedirects: followRedirects }, (response) => {
                const responseCode = response.statusCode;
                // Catch errors
                if (responseCode === undefined || responseCode >= 400) {
                    resolve(R.err({
                        message: `HTTP fetch failed with code ${responseCode !== null && responseCode !== void 0 ? responseCode : 'undefined'}. Please check your connection.`,
                        diagnostic: { node: context.getOrFailProperty('url') },
                    }));
                }
                else if (responseCode >= 301 && responseCode < 400) {
                    resolve(R.err({
                        message: `HTTP fetch was redirected with code ${responseCode}. Redirects are either disabled or maximum number of redirects was exeeded.`,
                        diagnostic: { node: context.getOrFailProperty('url') },
                    }));
                }
                // Get chunked data and store to ArrayBuffer
                let rawData = new Uint8Array(0);
                response.on('data', (chunk) => {
                    const tmp = new Uint8Array(rawData.length + chunk.length);
                    tmp.set(rawData, 0);
                    tmp.set(chunk, rawData.length);
                    rawData = tmp;
                });
                // When all data is downloaded, create file
                response.on('end', () => {
                    response.headers;
                    // Infer Mimetype from HTTP-Header, if not inferrable, then default to application/octet-stream
                    const mimeType = (0, jayvee_execution_1.inferMimeTypeFromFileExtensionString)(response.headers['content-type']) || jayvee_execution_1.MimeType.APPLICATION_OCTET_STREAM;
                    // Infer FileName and FileExtension from url, if not inferrable, then default to None
                    // Get last element of URL assuming this is a filename
                    const url = new URL(response.responseUrl);
                    let fileName = url.pathname.split('/').pop();
                    if (fileName === undefined) {
                        fileName = url.pathname.replace('/', '-');
                    }
                    const extName = path.extname(fileName);
                    let fileExtension = (0, jayvee_execution_1.inferFileExtensionFromFileExtensionString)(extName) ||
                        jayvee_execution_1.FileExtension.NONE;
                    // If FileExtension is not in url, try to infer extension from content-type, if not inferrable, then default to None
                    if (fileExtension === jayvee_execution_1.FileExtension.NONE) {
                        fileExtension =
                            (0, jayvee_execution_1.inferFileExtensionFromContentTypeString)(response.headers['content-type']) || jayvee_execution_1.FileExtension.NONE;
                    }
                    // Create file and return file
                    const file = new jayvee_execution_1.BinaryFile(fileName, fileExtension, mimeType, rawData.buffer);
                    resolve(R.ok(file));
                });
                response.on('error', (errorObj) => {
                    resolve(R.err({
                        message: errorObj.message,
                        diagnostic: { node: context.getCurrentNode(), property: 'name' },
                    }));
                });
            }).on('error', (e) => {
                resolve(R.err({
                    message: e.message,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                }));
            });
        });
    }
};
HttpExtractorExecutor.type = 'HttpExtractor';
HttpExtractorExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], HttpExtractorExecutor);
exports.HttpExtractorExecutor = HttpExtractorExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/std/exec/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/std/exec/src/text-file-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextFileInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const util_1 = __webpack_require__("util");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TextFileInterpreterExecutor = class TextFileInterpreterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.FILE, jayvee_language_server_1.IOType.TEXT_FILE);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const encoding = context.getPropertyValue('encoding', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const lineBreak = context.getPropertyValue('lineBreak', jayvee_language_server_1.PrimitiveValuetypes.Regex);
            const decoder = new util_1.TextDecoder(encoding);
            context.logger.logDebug(`Decoding file content using encoding "${encoding}"`);
            const textContent = decoder.decode(file.content);
            context.logger.logDebug(`Splitting lines using line break /${lineBreak.source}/`);
            const lines = (0, jayvee_execution_1.splitLines)(textContent, lineBreak);
            context.logger.logDebug(`Lines were split successfully, the resulting text file has ${lines.length} lines`);
            return R.ok(new jayvee_execution_1.TextFile(file.name, file.extension, file.mimeType, lines));
        });
    }
};
TextFileInterpreterExecutor.type = 'TextFileInterpreter';
TextFileInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], TextFileInterpreterExecutor);
exports.TextFileInterpreterExecutor = TextFileInterpreterExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/text-line-deleter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextLineDeleterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TextLineDeleterExecutor = class TextLineDeleterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.TEXT_FILE, jayvee_language_server_1.IOType.TEXT_FILE);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const lines = context.getPropertyValue('lines', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.Integer));
            const numberOfLines = file.content.length;
            let lineIndex = 0;
            for (const lineNumber of lines) {
                if (lineNumber > numberOfLines) {
                    return R.err({
                        message: `Line ${lineNumber} does not exist in the text file, only ${file.content.length} line(s) are present`,
                        diagnostic: {
                            node: context.getOrFailProperty('lines').value,
                            property: 'values',
                            index: lineIndex,
                        },
                    });
                }
                ++lineIndex;
            }
            const distinctLines = new Set(lines);
            const sortedLines = [...distinctLines].sort((a, b) => a - b);
            context.logger.logDebug(`Deleting line(s) ${sortedLines.join(', ')}`);
            const reversedLines = sortedLines.reverse();
            const newContent = [...file.content];
            for (const lineToDelete of reversedLines) {
                newContent.splice(lineToDelete - 1, 1);
            }
            return R.ok(new jayvee_execution_1.TextFile(file.name, file.extension, file.mimeType, newContent));
        });
    }
};
TextLineDeleterExecutor.type = 'TextLineDeleter';
TextLineDeleterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], TextLineDeleterExecutor);
exports.TextLineDeleterExecutor = TextLineDeleterExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/text-range-selector-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TextRangeSelectorExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TextRangeSelectorExecutor = class TextRangeSelectorExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.TEXT_FILE, jayvee_language_server_1.IOType.TEXT_FILE);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const lineFrom = context.getPropertyValue('lineFrom', jayvee_language_server_1.PrimitiveValuetypes.Integer);
            const lineTo = context.getPropertyValue('lineTo', jayvee_language_server_1.PrimitiveValuetypes.Integer);
            const numberOfLines = file.content.length;
            context.logger.logDebug(`Selecting lines from ${lineFrom} to ${lineTo === Number.MAX_SAFE_INTEGER || lineTo >= numberOfLines
                ? 'the end'
                : `${lineTo}`}`);
            const selectedLines = file.content.slice(lineFrom - 1, lineTo);
            return R.ok(new jayvee_execution_1.TextFile(file.name, file.extension, file.mimeType, selectedLines));
        });
    }
};
TextRangeSelectorExecutor.type = 'TextRangeSelector';
TextRangeSelectorExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], TextRangeSelectorExecutor);
exports.TextRangeSelectorExecutor = TextRangeSelectorExecutor;


/***/ }),

/***/ "./libs/extensions/std/exec/src/util/backoff-strategy.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LinearBackoffStrategy = exports.ExponentialBackoffStrategy = exports.createBackoffStrategy = exports.isBackoffStrategyHandle = void 0;
function isBackoffStrategyHandle(v) {
    return v === 'exponential' || v === 'linear';
}
exports.isBackoffStrategyHandle = isBackoffStrategyHandle;
function createBackoffStrategy(handle, backoffMilliseconds) {
    if (handle === 'linear') {
        return new LinearBackoffStrategy(backoffMilliseconds);
    }
    return new ExponentialBackoffStrategy(backoffMilliseconds);
}
exports.createBackoffStrategy = createBackoffStrategy;
/**
 * Strategy for exponential backoffs.
 * Uses seconds as unit for calculating the exponent.
 */
class ExponentialBackoffStrategy {
    constructor(initialBackoffMilliseconds) {
        this.initialBackoffMilliseconds = initialBackoffMilliseconds;
    }
    getBackoffMilliseconds(retry) {
        const initialBackoffSeconds = this.initialBackoffMilliseconds / 1000;
        return Math.pow(initialBackoffSeconds, retry) * 1000;
    }
}
exports.ExponentialBackoffStrategy = ExponentialBackoffStrategy;
/**
 * Strategy for linear backoffs.
 */
class LinearBackoffStrategy {
    constructor(backoffMilliseconds) {
        this.backoffMilliseconds = backoffMilliseconds;
    }
    getBackoffMilliseconds() {
        return this.backoffMilliseconds;
    }
}
exports.LinearBackoffStrategy = LinearBackoffStrategy;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/extension.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TabularExecExtension = void 0;
const cell_range_selector_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/cell-range-selector-executor.ts");
const cell_writer_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/cell-writer-executor.ts");
const column_deleter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/column-deleter-executor.ts");
const csv_interpreter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/csv-interpreter-executor.ts");
const row_deleter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/row-deleter-executor.ts");
const sheet_picker_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/sheet-picker-executor.ts");
const table_interpreter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/table-interpreter-executor.ts");
const table_transformer_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/table-transformer-executor.ts");
const xlsx_interpreter_executor_1 = __webpack_require__("./libs/extensions/tabular/exec/src/lib/xlsx-interpreter-executor.ts");
class TabularExecExtension {
    getBlockExecutors() {
        return [
            cell_writer_executor_1.CellWriterExecutor,
            column_deleter_executor_1.ColumnDeleterExecutor,
            row_deleter_executor_1.RowDeleterExecutor,
            cell_range_selector_executor_1.CellRangeSelectorExecutor,
            table_interpreter_executor_1.TableInterpreterExecutor,
            csv_interpreter_executor_1.CSVInterpreterExecutor,
            table_transformer_executor_1.TableTransformerExecutor,
            xlsx_interpreter_executor_1.XLSXInterpreterExecutor,
            sheet_picker_executor_1.SheetPickerExecutor,
        ];
    }
}
exports.TabularExecExtension = TabularExecExtension;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/extensions/tabular/exec/src/extension.ts"), exports);


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/cell-range-selector-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellRangeSelectorExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let CellRangeSelectorExecutor = class CellRangeSelectorExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeRange = context.getPropertyValue('select', jayvee_language_server_1.PrimitiveValuetypes.CellRange);
            const absoluteRange = inputSheet.resolveRelativeIndexes(relativeRange);
            if (!inputSheet.isInBounds(absoluteRange)) {
                return R.err({
                    message: 'The specified cell range does not fit the sheet',
                    diagnostic: { node: absoluteRange.astNode },
                });
            }
            context.logger.logDebug(`Selecting cell range ${absoluteRange.toString()}`);
            const resultingSheet = inputSheet.clone();
            resultingSheet.selectRange(absoluteRange);
            return R.ok(resultingSheet);
        });
    }
};
CellRangeSelectorExecutor.type = 'CellRangeSelector';
CellRangeSelectorExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], CellRangeSelectorExecutor);
exports.CellRangeSelectorExecutor = CellRangeSelectorExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/cell-writer-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CellWriterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let CellWriterExecutor = class CellWriterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeCellRange = context.getPropertyValue('at', jayvee_language_server_1.PrimitiveValuetypes.CellRange);
            const writeValues = context.getPropertyValue('write', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.Text));
            (0, assert_1.strict)(relativeCellRange.isOneDimensional());
            const absoluteCellRange = inputSheet.resolveRelativeIndexes(relativeCellRange);
            if (!inputSheet.isInBounds(absoluteCellRange)) {
                return R.err({
                    message: 'Some specified cells do not exist in the sheet',
                    diagnostic: { node: absoluteCellRange.astNode },
                });
            }
            const cellIndexesToWrite = inputSheet.enumerateCellIndexes(relativeCellRange);
            if (writeValues.length !== cellIndexesToWrite.length) {
                context.logger.logWarnDiagnostic(`The number of values to write (${writeValues.length}) does not match the number of cells (${cellIndexesToWrite.length})`, { node: relativeCellRange.astNode });
            }
            const resultingSheet = inputSheet.clone();
            for (let i = 0; i < Math.min(writeValues.length, cellIndexesToWrite.length); ++i) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const cellIndex = cellIndexesToWrite[i];
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                const writeValue = writeValues[i];
                context.logger.logDebug(`Writing "${writeValue}" at cell ${cellIndex.toString()}`);
                resultingSheet.writeCell(cellIndex, writeValue);
            }
            return R.ok(resultingSheet);
        });
    }
};
CellWriterExecutor.type = 'CellWriter';
CellWriterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], CellWriterExecutor);
exports.CellWriterExecutor = CellWriterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/column-deleter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ColumnDeleterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let ColumnDeleterExecutor = class ColumnDeleterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeColumns = context.getPropertyValue('delete', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.CellRange));
            (0, assert_1.strict)(relativeColumns.every(jayvee_language_server_1.isColumnWrapper));
            let absoluteColumns = relativeColumns.map((column) => inputSheet.resolveRelativeIndexes(column));
            for (const column of absoluteColumns) {
                if (!inputSheet.isInBounds(column)) {
                    const columnIndex = (0, jayvee_language_server_1.getColumnIndex)(column);
                    return R.err({
                        message: `The specified column ${(0, jayvee_language_server_1.columnIndexToString)(columnIndex)} does not exist in the sheet`,
                        diagnostic: { node: column.astNode },
                    });
                }
            }
            // Required for removing duplicates in the next step
            this.sortByColumnIndex(absoluteColumns);
            // That way, the upcoming deletion is only called once per individual column
            absoluteColumns = this.removeDuplicateColumns(absoluteColumns);
            context.logger.logDebug(`Deleting column(s) ${absoluteColumns
                .map(jayvee_language_server_1.getColumnIndex)
                .map(jayvee_language_server_1.columnIndexToString)
                .join(', ')}`);
            // By reversing the order, the column indexes stay stable during deletion
            absoluteColumns.reverse();
            const resultingSheet = inputSheet.clone();
            absoluteColumns.forEach((column) => {
                resultingSheet.deleteColumn(column);
            });
            return R.ok(resultingSheet);
        });
    }
    sortByColumnIndex(columns) {
        columns.sort((columnA, columnB) => (0, jayvee_language_server_1.getColumnIndex)(columnA) - (0, jayvee_language_server_1.getColumnIndex)(columnB));
    }
    removeDuplicateColumns(columns) {
        return columns.reduce((previous, column, index) => {
            const previousColumn = previous[index - 1];
            if (previousColumn !== undefined) {
                if ((0, jayvee_language_server_1.getColumnIndex)(previousColumn) === (0, jayvee_language_server_1.getColumnIndex)(column)) {
                    // The current column is a duplicate because it has the same index as the previous column
                    return previous;
                }
            }
            return [...previous, column];
        }, []);
    }
};
ColumnDeleterExecutor.type = 'ColumnDeleter';
ColumnDeleterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], ColumnDeleterExecutor);
exports.ColumnDeleterExecutor = ColumnDeleterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/csv-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CSVInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const parse_1 = __webpack_require__("@fast-csv/parse");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const E = __webpack_require__("fp-ts/lib/Either");
const Either_1 = __webpack_require__("fp-ts/lib/Either");
let CSVInterpreterExecutor = class CSVInterpreterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.TEXT_FILE, jayvee_language_server_1.IOType.SHEET);
    }
    doExecute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const delimiter = context.getPropertyValue('delimiter', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const enclosing = context.getPropertyValue('enclosing', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const enclosingEscape = context.getPropertyValue('enclosingEscape', jayvee_language_server_1.PrimitiveValuetypes.Text);
            context.logger.logDebug(`Parsing raw data as CSV using delimiter "${delimiter}"`);
            const parseOptions = {
                delimiter,
                quote: enclosing,
                escape: enclosingEscape,
            };
            const csvData = yield parseAsCsv(file.content, parseOptions);
            if ((0, Either_1.isLeft)(csvData)) {
                return Promise.resolve(R.err({
                    message: `CSV parse failed in line ${csvData.left.lineNumber}: ${csvData.left.error.message}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                }));
            }
            const sheet = new jayvee_execution_1.Sheet(csvData.right);
            context.logger.logDebug(`Parsing raw data as CSV-sheet successful`);
            return Promise.resolve(R.ok(sheet));
        });
    }
};
CSVInterpreterExecutor.type = 'CSVInterpreter';
CSVInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], CSVInterpreterExecutor);
exports.CSVInterpreterExecutor = CSVInterpreterExecutor;
function parseAsCsv(lines, parseOptions) {
    var lines_1, lines_1_1;
    var e_1, _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let lineNumber = 1;
        const rows = [];
        try {
            for (lines_1 = tslib_1.__asyncValues(lines); lines_1_1 = yield lines_1.next(), !lines_1_1.done;) {
                const line = lines_1_1.value;
                const rowParseResult = yield parseLineAsRow(line, parseOptions);
                if ((0, Either_1.isLeft)(rowParseResult)) {
                    return E.left({ error: rowParseResult.left, lineNumber });
                }
                rows.push(rowParseResult.right);
                ++lineNumber;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (lines_1_1 && !lines_1_1.done && (_a = lines_1.return)) yield _a.call(lines_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return E.right(rows);
    });
}
function parseLineAsRow(line, parseOptions) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => {
            let row;
            (0, parse_1.parseString)(line, parseOptions)
                .on('data', (data) => {
                row = data;
            })
                .on('error', (error) => {
                resolve(E.left(error));
            })
                .on('end', () => {
                resolve(E.right(row));
            });
        });
    });
}


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/row-deleter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RowDeleterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let RowDeleterExecutor = class RowDeleterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.SHEET);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const relativeRows = context.getPropertyValue('delete', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.CellRange));
            (0, assert_1.strict)(relativeRows.every(jayvee_language_server_1.isRowWrapper));
            let absoluteRows = relativeRows.map((row) => inputSheet.resolveRelativeIndexes(row));
            for (const row of absoluteRows) {
                if (!inputSheet.isInBounds(row)) {
                    const rowIndex = (0, jayvee_language_server_1.getRowIndex)(row);
                    return R.err({
                        message: `The specified row ${(0, jayvee_language_server_1.rowIndexToString)(rowIndex)} does not exist in the sheet`,
                        diagnostic: { node: row.astNode },
                    });
                }
            }
            // Required for removing duplicates in the next step
            this.sortByRowIndex(absoluteRows);
            // That way, the upcoming deletion is only called once per individual row
            absoluteRows = this.removeDuplicateRows(absoluteRows);
            context.logger.logDebug(`Deleting row(s) ${absoluteRows
                .map(jayvee_language_server_1.getRowIndex)
                .map(jayvee_language_server_1.rowIndexToString)
                .join(', ')}`);
            // By reversing the order, the row indexes stay stable during deletion
            absoluteRows.reverse();
            const resultingSheet = inputSheet.clone();
            absoluteRows.forEach((row) => {
                resultingSheet.deleteRow(row);
            });
            return R.ok(resultingSheet);
        });
    }
    sortByRowIndex(rows) {
        rows.sort((firstRow, secondRow) => (0, jayvee_language_server_1.getRowIndex)(firstRow) - (0, jayvee_language_server_1.getRowIndex)(secondRow));
    }
    removeDuplicateRows(rows) {
        return rows.reduce((previous, row, index) => {
            const previousRow = previous[index - 1];
            if (previousRow !== undefined) {
                if ((0, jayvee_language_server_1.getRowIndex)(previousRow) === (0, jayvee_language_server_1.getRowIndex)(row)) {
                    // The current row is a duplicate because it has the same index as the previous row
                    return previous;
                }
            }
            return [...previous, row];
        }, []);
    }
};
RowDeleterExecutor.type = 'RowDeleter';
RowDeleterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], RowDeleterExecutor);
exports.RowDeleterExecutor = RowDeleterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/sheet-picker-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SheetPickerExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let SheetPickerExecutor = class SheetPickerExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.WORKBOOK, jayvee_language_server_1.IOType.SHEET);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(workbook, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const sheetName = context.getPropertyValue('sheetName', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const sheet = workbook.getSheetByName(sheetName);
            if (sheet === undefined) {
                return R.err({
                    message: `Workbook does not contain a sheet named ${sheetName}`,
                    diagnostic: { node: context.getCurrentNode(), property: 'name' },
                });
            }
            return R.ok(sheet);
        });
    }
};
SheetPickerExecutor.type = 'SheetPicker';
SheetPickerExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], SheetPickerExecutor);
exports.SheetPickerExecutor = SheetPickerExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/table-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TableInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TableInterpreterExecutor = class TableInterpreterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.SHEET, jayvee_language_server_1.IOType.TABLE);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(inputSheet, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const header = context.getPropertyValue('header', jayvee_language_server_1.PrimitiveValuetypes.Boolean);
            const columnDefinitions = context.getPropertyValue('columns', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.ValuetypeAssignment));
            let columnEntries;
            if (header) {
                if (inputSheet.getNumberOfRows() < 1) {
                    return R.err({
                        message: 'The input sheet is empty and thus has no header',
                        diagnostic: {
                            node: context.getOrFailProperty('header'),
                        },
                    });
                }
                const headerRow = inputSheet.getHeaderRow();
                columnEntries = this.deriveColumnDefinitionEntriesFromHeader(columnDefinitions, headerRow, context);
            }
            else {
                if (inputSheet.getNumberOfColumns() < columnDefinitions.length) {
                    return R.err({
                        message: `There are ${columnDefinitions.length} column definitions but the input sheet only has ${inputSheet.getNumberOfColumns()} columns`,
                        diagnostic: {
                            node: context.getOrFailProperty('columns'),
                        },
                    });
                }
                columnEntries =
                    this.deriveColumnDefinitionEntriesWithoutHeader(columnDefinitions);
            }
            const numberOfTableRows = header
                ? inputSheet.getNumberOfRows() - 1
                : inputSheet.getNumberOfRows();
            context.logger.logDebug(`Validating ${numberOfTableRows} row(s) according to the column types`);
            const resultingTable = this.constructAndValidateTable(inputSheet, header, columnEntries, context);
            context.logger.logDebug(`Validation completed, the resulting table has ${resultingTable.getNumberOfRows()} row(s) and ${resultingTable.getNumberOfColumns()} column(s)`);
            return R.ok(resultingTable);
        });
    }
    constructAndValidateTable(sheet, header, columnEntries, context) {
        const table = new jayvee_execution_1.Table();
        // add columns
        columnEntries.forEach((columnEntry) => {
            table.addColumn(columnEntry.columnName, {
                values: [],
                valuetype: columnEntry.valuetype,
            });
        });
        // add rows
        sheet.iterateRows((sheetRow, sheetRowIndex) => {
            if (header && sheetRowIndex === 0) {
                return;
            }
            const tableRow = this.constructAndValidateTableRow(sheetRow, sheetRowIndex, columnEntries, context);
            if (tableRow === undefined) {
                context.logger.logDebug(`Omitting row ${(0, jayvee_language_server_1.rowIndexToString)(sheetRowIndex)}`);
                return;
            }
            table.addRow(tableRow);
        });
        return table;
    }
    constructAndValidateTableRow(sheetRow, sheetRowIndex, columnEntries, context) {
        let invalidRow = false;
        const tableRow = {};
        columnEntries.forEach((columnEntry) => {
            const sheetColumnIndex = columnEntry.sheetColumnIndex;
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const value = sheetRow[sheetColumnIndex];
            const valuetype = columnEntry.valuetype;
            const parsedValue = this.parseAndValidateValue(value, valuetype, context);
            if (parsedValue === undefined) {
                const currentCellIndex = new jayvee_language_server_1.CellIndex(sheetColumnIndex, sheetRowIndex);
                context.logger.logDebug(`Invalid value at cell ${currentCellIndex.toString()}: "${value}" does not match the type ${columnEntry.valuetype.getName()}`);
                invalidRow = true;
                return;
            }
            tableRow[columnEntry.columnName] = parsedValue;
        });
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (invalidRow) {
            return undefined;
        }
        (0, assert_1.strict)(Object.keys(tableRow).length === columnEntries.length);
        return tableRow;
    }
    parseAndValidateValue(value, valuetype, context) {
        const parsedValue = (0, jayvee_execution_1.parseValueToInternalRepresentation)(value, valuetype);
        if (parsedValue === undefined) {
            return undefined;
        }
        if (!(0, jayvee_execution_1.isValidValueRepresentation)(parsedValue, valuetype, context)) {
            return undefined;
        }
        return parsedValue;
    }
    deriveColumnDefinitionEntriesWithoutHeader(columnDefinitions) {
        return columnDefinitions.map((columnDefinition, columnDefinitionIndex) => {
            const columnValuetype = (0, jayvee_language_server_1.createValuetype)(columnDefinition.type);
            (0, assert_1.strict)(columnValuetype !== undefined);
            return {
                sheetColumnIndex: columnDefinitionIndex,
                columnName: columnDefinition.name,
                valuetype: columnValuetype,
                astNode: columnDefinition,
            };
        });
    }
    deriveColumnDefinitionEntriesFromHeader(columnDefinitions, headerRow, context) {
        context.logger.logDebug(`Matching header with provided column names`);
        const columnEntries = [];
        for (const columnDefinition of columnDefinitions) {
            const indexOfMatchingHeader = headerRow.findIndex((headerColumnName) => headerColumnName === columnDefinition.name);
            if (indexOfMatchingHeader === -1) {
                context.logger.logDebug(`Omitting column "${columnDefinition.name}" as the name was not found in the header`);
                continue;
            }
            const columnValuetype = (0, jayvee_language_server_1.createValuetype)(columnDefinition.type);
            (0, assert_1.strict)(columnValuetype !== undefined);
            columnEntries.push({
                sheetColumnIndex: indexOfMatchingHeader,
                columnName: columnDefinition.name,
                valuetype: columnValuetype,
                astNode: columnDefinition,
            });
        }
        return columnEntries;
    }
};
TableInterpreterExecutor.type = 'TableInterpreter';
TableInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], TableInterpreterExecutor);
exports.TableInterpreterExecutor = TableInterpreterExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/table-transformer-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TableTransformerExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
let TableTransformerExecutor = class TableTransformerExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.TABLE, jayvee_language_server_1.IOType.TABLE);
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    doExecute(inputTable, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const inputColumnNames = context.getPropertyValue('inputColumns', new jayvee_language_server_1.CollectionValuetype(jayvee_language_server_1.PrimitiveValuetypes.Text));
            const outputColumnName = context.getPropertyValue('outputColumn', jayvee_language_server_1.PrimitiveValuetypes.Text);
            const usedTransform = context.getPropertyValue('use', jayvee_language_server_1.PrimitiveValuetypes.Transform);
            const checkInputColumnsExistResult = this.checkInputColumnsExist(inputColumnNames, inputTable, context);
            if (R.isErr(checkInputColumnsExistResult)) {
                return checkInputColumnsExistResult;
            }
            const executor = new jayvee_execution_1.TransformExecutor(usedTransform);
            const transformInputDetailsList = executor.getInputDetails();
            const transformOutputDetails = executor.getOutputDetails();
            const checkInputColumnsMatchTransformInputTypesResult = this.checkInputColumnsMatchTransformInputTypes(inputColumnNames, inputTable, transformInputDetailsList, context);
            if (R.isErr(checkInputColumnsMatchTransformInputTypesResult)) {
                return checkInputColumnsMatchTransformInputTypesResult;
            }
            const variableToColumnMap = R.okData(checkInputColumnsMatchTransformInputTypesResult);
            this.logColumnOverwriteStatus(inputTable, outputColumnName, context, transformOutputDetails);
            const transformResult = executor.executeTransform(variableToColumnMap, inputTable.getNumberOfRows(), context);
            const outputTable = this.createOutputTable(inputTable, transformResult, outputColumnName);
            return R.ok(outputTable);
        });
    }
    checkInputColumnsMatchTransformInputTypes(inputColumnNames, inputTable, transformInputDetailsList, context) {
        const variableToColumnMap = new Map();
        for (let i = 0; i < inputColumnNames.length; ++i) {
            const inputColumnName = inputColumnNames[i];
            (0, assert_1.strict)(inputColumnName !== undefined);
            const inputColumn = inputTable.getColumn(inputColumnName);
            (0, assert_1.strict)(inputColumn !== undefined);
            const matchingInputDetails = transformInputDetailsList[i];
            (0, assert_1.strict)(matchingInputDetails !== undefined);
            if (!inputColumn.valuetype.isConvertibleTo(matchingInputDetails.valuetype)) {
                return R.err({
                    message: `Type ${inputColumn.valuetype.getName()} of column "${inputColumnName}" is not convertible to type ${matchingInputDetails.valuetype.getName()}`,
                    diagnostic: {
                        node: context.getOrFailProperty('use'),
                    },
                });
            }
            const variableName = matchingInputDetails.port.name;
            variableToColumnMap.set(variableName, inputColumn);
        }
        return R.ok(variableToColumnMap);
    }
    checkInputColumnsExist(inputColumnNames, inputTable, context) {
        // check input columns exist
        let i = 0;
        for (const inputColumnName of inputColumnNames) {
            const inputColumn = inputTable.getColumn(inputColumnName);
            if (inputColumn === undefined) {
                return R.err({
                    message: `The specified input column "${inputColumnName}" does not exist in the given table`,
                    diagnostic: {
                        node: context.getOrFailProperty('inputColumns').value,
                        property: 'values',
                        index: i,
                    },
                });
            }
            ++i;
        }
        return R.ok(undefined);
    }
    createOutputTable(inputTable, transformResult, outputColumnName) {
        const outputTable = inputTable.clone();
        outputTable.dropRows(transformResult.rowsToDelete);
        outputTable.addColumn(outputColumnName, transformResult.resultingColumn);
        return outputTable;
    }
    logColumnOverwriteStatus(inputTable, outputColumnName, context, transformOutputDetails) {
        const outputColumn = inputTable.getColumn(outputColumnName);
        if (outputColumn !== undefined) {
            context.logger.logInfo(`Column "${outputColumnName}" will be overwritten`);
            // log if output column type changes
            if (!outputColumn.valuetype.equals(transformOutputDetails.valuetype)) {
                context.logger.logInfo(`Column "${outputColumnName}" will change its type from ${outputColumn.valuetype.getName()} to ${transformOutputDetails.valuetype.getName()}`);
            }
        }
    }
};
TableTransformerExecutor.type = 'TableTransformer';
TableTransformerExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], TableTransformerExecutor);
exports.TableTransformerExecutor = TableTransformerExecutor;


/***/ }),

/***/ "./libs/extensions/tabular/exec/src/lib/xlsx-interpreter-executor.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.XLSXInterpreterExecutor = void 0;
const tslib_1 = __webpack_require__("tslib");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const exceljs = __webpack_require__("exceljs");
let XLSXInterpreterExecutor = class XLSXInterpreterExecutor extends jayvee_execution_1.AbstractBlockExecutor {
    constructor() {
        super(jayvee_language_server_1.IOType.FILE, jayvee_language_server_1.IOType.WORKBOOK);
    }
    doExecute(file, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            context.logger.logDebug(`Reading from XLSX file`);
            const workBookFromFile = new exceljs.Workbook();
            yield workBookFromFile.xlsx.load(file.content);
            const workbook = new jayvee_execution_1.Workbook();
            workBookFromFile.eachSheet((workSheet) => {
                const workSheetDataArray = [];
                workSheet.eachRow((row, rowNumber) => {
                    const cellValues = [];
                    // ExcelJS Rows and Columns are indexed from 1
                    // We reduce their index to match Sheets being zero indexed
                    row.eachCell({ includeEmpty: true }, (cell, colNumber) => {
                        cellValues[colNumber - 1] = cell.text;
                    });
                    workSheetDataArray[rowNumber - 1] = cellValues;
                });
                workbook.addSheet(workSheetDataArray, workSheet.name);
            });
            return Promise.resolve(R.ok(workbook));
        });
    }
};
XLSXInterpreterExecutor.type = 'XLSXInterpreter';
XLSXInterpreterExecutor = tslib_1.__decorate([
    (0, jayvee_execution_1.implementsStatic)(),
    tslib_1.__metadata("design:paramtypes", [])
], XLSXInterpreterExecutor);
exports.XLSXInterpreterExecutor = XLSXInterpreterExecutor;


/***/ }),

/***/ "./libs/interpreter-lib/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/interpreter-lib/src/interpreter.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/interpreter-lib/src/parsing-util.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/interpreter-lib/src/logging/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/interpreter-lib/src/validation-checks/index.ts"), exports);


/***/ }),

/***/ "./libs/interpreter-lib/src/interpreter.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.logPipelineOverview = exports.useStdExtension = exports.interpretModel = exports.parseModel = exports.interpretString = void 0;
const tslib_1 = __webpack_require__("tslib");
const assert_1 = __webpack_require__("assert");
const R = __webpack_require__("./libs/execution/src/index.ts");
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const exec_1 = __webpack_require__("./libs/extensions/std/exec/src/index.ts");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const chalk = __webpack_require__("chalk");
const node_1 = __webpack_require__("langium/node");
const logging_1 = __webpack_require__("./libs/interpreter-lib/src/logging/index.ts");
const parsing_util_1 = __webpack_require__("./libs/interpreter-lib/src/parsing-util.ts");
const validation_checks_1 = __webpack_require__("./libs/interpreter-lib/src/validation-checks/index.ts");
function interpretString(modelString, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const extractAstNodeFn = (services, loggerFactory) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield (0, parsing_util_1.extractAstNodeFromString)(modelString, services, loggerFactory.createLogger());
        });
        return yield interpretModel(extractAstNodeFn, options);
    });
}
exports.interpretString = interpretString;
/**
 * Parses a model without executing it.
 * Also sets up the environment so that the model can be properly executed.
 *
 * @returns non-null model, services and loggerFactory on success.
 */
function parseModel(extractAstNodeFn, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let services = null;
        let model = null;
        const loggerFactory = new logging_1.LoggerFactory(options.debug);
        if (!(0, jayvee_execution_1.isDebugGranularity)(options.debugGranularity)) {
            loggerFactory
                .createLogger()
                .logErr(`Unknown value "${options.debugGranularity}" for debug granularity option: -dg --debug-granularity.\n` +
                `Please use one of the following values: ${R.DebugGranularityValues.join(', ')}.`);
            return { model, services, loggerFactory };
        }
        useStdExtension();
        (0, jayvee_execution_1.registerDefaultConstraintExecutors)();
        services = (0, jayvee_language_server_1.createJayveeServices)(node_1.NodeFileSystem).Jayvee;
        yield (0, jayvee_language_server_1.initializeWorkspace)(services);
        setupJayveeServices(services, options.env);
        try {
            model = yield extractAstNodeFn(services, loggerFactory);
            return { model, services, loggerFactory };
        }
        catch (e) {
            loggerFactory
                .createLogger()
                .logErr('Could not extract the AST node of the given model.');
            return { model, services, loggerFactory };
        }
    });
}
exports.parseModel = parseModel;
function interpretModel(extractAstNodeFn, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { model, services, loggerFactory } = yield parseModel(extractAstNodeFn, options);
        if (model == null || services == null) {
            return parsing_util_1.ExitCode.FAILURE;
        }
        const debugTargets = getDebugTargets(options.debugTarget);
        const interpretationExitCode = yield interpretJayveeModel(model, services.RuntimeParameterProvider, loggerFactory, {
            debug: options.debug,
            // type of options.debugGranularity is asserted in parseModel
            debugGranularity: options.debugGranularity,
            debugTargets: debugTargets,
        });
        return interpretationExitCode;
    });
}
exports.interpretModel = interpretModel;
function setupJayveeServices(services, rawRuntimeParameters) {
    setupRuntimeParameterProvider(services.RuntimeParameterProvider, rawRuntimeParameters);
    services.validation.ValidationRegistry.registerJayveeValidationChecks({
        RuntimeParameterLiteral: validation_checks_1.validateRuntimeParameterLiteral,
    });
}
function setupRuntimeParameterProvider(runtimeParameterProvider, rawRuntimeParameters) {
    runtimeParameterProvider.setValueParser(jayvee_execution_1.parseValueToInternalRepresentation);
    for (const [key, value] of rawRuntimeParameters.entries()) {
        runtimeParameterProvider.setValue(key, value);
    }
}
function useStdExtension() {
    (0, jayvee_execution_1.useExtension)(new exec_1.StdExecExtension());
}
exports.useStdExtension = useStdExtension;
function interpretJayveeModel(model, runtimeParameterProvider, loggerFactory, runOptions) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const pipelineRuns = model.pipelines.map((pipeline) => {
            return runPipeline(pipeline, runtimeParameterProvider, loggerFactory, runOptions);
        });
        const exitCodes = yield Promise.all(pipelineRuns);
        if (exitCodes.includes(parsing_util_1.ExitCode.FAILURE)) {
            return parsing_util_1.ExitCode.FAILURE;
        }
        return parsing_util_1.ExitCode.SUCCESS;
    });
}
function runPipeline(pipeline, runtimeParameterProvider, loggerFactory, runOptions) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const executionContext = new jayvee_execution_1.ExecutionContext(pipeline, loggerFactory.createLogger(), {
            isDebugMode: runOptions.debug,
            debugGranularity: runOptions.debugGranularity,
            debugTargets: runOptions.debugTargets,
        }, new jayvee_language_server_1.EvaluationContext(runtimeParameterProvider));
        logPipelineOverview(pipeline, runtimeParameterProvider, executionContext.logger);
        const startTime = new Date();
        const executionOrder = (0, jayvee_language_server_1.getBlocksInTopologicalSorting)(pipeline).map((block) => {
            return { block: block, value: jayvee_execution_1.NONE };
        });
        const executionResult = yield (0, jayvee_execution_1.executeBlocks)(executionContext, executionOrder);
        if (R.isErr(executionResult)) {
            const diagnosticError = executionResult.left;
            executionContext.logger.logErrDiagnostic(diagnosticError.message, diagnosticError.diagnostic);
            (0, jayvee_execution_1.logExecutionDuration)(startTime, executionContext.logger);
            return parsing_util_1.ExitCode.FAILURE;
        }
        (0, jayvee_execution_1.logExecutionDuration)(startTime, executionContext.logger);
        return parsing_util_1.ExitCode.SUCCESS;
    });
}
function logPipelineOverview(pipeline, runtimeParameterProvider, logger) {
    const toString = (block, depth = 0) => {
        var _a;
        const blockTypeName = (_a = block.type.ref) === null || _a === void 0 ? void 0 : _a.name;
        (0, assert_1.strict)(blockTypeName !== undefined);
        const blockString = `${'\t'.repeat(depth)} -> ${block.name} (${blockTypeName})`;
        const childString = (0, jayvee_language_server_1.collectChildren)(block)
            .map((child) => toString(child, depth + 1))
            .join('\n');
        return blockString + '\n' + childString;
    };
    const linesBuffer = [];
    linesBuffer.push(chalk.underline('Overview:'));
    const runtimeParameters = runtimeParameterProvider.getReadonlyMap();
    if (runtimeParameters.size > 0) {
        linesBuffer.push(`\tRuntime Parameters (${runtimeParameters.size}):`);
        for (const [key, value] of runtimeParameters.entries()) {
            linesBuffer.push(`\t\t${key}: ${value.toString()}`);
        }
    }
    linesBuffer.push(`\tBlocks (${pipeline.blocks.length} blocks with ${pipeline.pipes.length} pipes):`);
    for (const block of (0, jayvee_language_server_1.collectStartingBlocks)(pipeline)) {
        linesBuffer.push(toString(block, 1));
    }
    logger.logInfo(linesBuffer.join('\n'));
}
exports.logPipelineOverview = logPipelineOverview;
function getDebugTargets(debugTargetsString) {
    const areAllBlocksTargeted = debugTargetsString === undefined;
    if (areAllBlocksTargeted) {
        return R.DefaultDebugTargetsValue;
    }
    return debugTargetsString.split(',').map((target) => target.trim());
}


/***/ }),

/***/ "./libs/interpreter-lib/src/logging/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/interpreter-lib/src/logging/logger-factory.ts"), exports);


/***/ }),

/***/ "./libs/interpreter-lib/src/logging/logger-factory.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LoggerFactory = void 0;
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
class LoggerFactory {
    constructor(enableDebugLogging) {
        this.enableDebugLogging = enableDebugLogging;
    }
    createLogger(loggingContext) {
        return new jayvee_execution_1.DefaultLogger(this.enableDebugLogging, loggingContext);
    }
}
exports.LoggerFactory = LoggerFactory;


/***/ }),

/***/ "./libs/interpreter-lib/src/parsing-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.extractAstNodeFromString = exports.extractAstNodeFromFile = exports.validateDocument = exports.extractDocumentFromString = exports.extractDocumentFromFile = exports.ExitCode = void 0;
const tslib_1 = __webpack_require__("tslib");
const fs = __webpack_require__("fs");
const path = __webpack_require__("path");
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
const vscode_languageserver_protocol_1 = __webpack_require__("vscode-languageserver-protocol");
const vscode_uri_1 = __webpack_require__("vscode-uri");
var ExitCode;
(function (ExitCode) {
    ExitCode[ExitCode["SUCCESS"] = 0] = "SUCCESS";
    ExitCode[ExitCode["FAILURE"] = 1] = "FAILURE";
})(ExitCode = exports.ExitCode || (exports.ExitCode = {}));
function extractDocumentFromFile(fileName, services, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const extensions = services.LanguageMetaData.fileExtensions;
        if (!extensions.includes(path.extname(fileName))) {
            const errorMessage = `Please choose a file with ${extensions.length === 1 ? 'this extension' : 'one of these extensions'}: ${extensions.map((extension) => `"${extension}"`).join(',')}`;
            logger.logErr(errorMessage);
            return Promise.reject(ExitCode.FAILURE);
        }
        if (!fs.existsSync(fileName)) {
            logger.logErr(`File ${fileName} does not exist.`);
            return Promise.reject(ExitCode.FAILURE);
        }
        const document = services.shared.workspace.LangiumDocuments.getOrCreateDocument(vscode_uri_1.URI.file(path.resolve(fileName)));
        yield (0, jayvee_language_server_1.initializeWorkspace)(services);
        return yield validateDocument(document, services, logger);
    });
}
exports.extractDocumentFromFile = extractDocumentFromFile;
function extractDocumentFromString(modelString, services, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const document = services.shared.workspace.LangiumDocumentFactory.fromString(modelString, vscode_uri_1.URI.parse('memory://jayvee.document'));
        yield (0, jayvee_language_server_1.initializeWorkspace)(services);
        return yield validateDocument(document, services, logger);
    });
}
exports.extractDocumentFromString = extractDocumentFromString;
function validateDocument(document, services, logger) {
    var _a;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield services.shared.workspace.DocumentBuilder.build([document], {
            validationChecks: 'all',
        });
        const diagnostics = (_a = document.diagnostics) !== null && _a !== void 0 ? _a : [];
        const errDiagnostics = diagnostics.filter((diagnostic) => diagnostic.severity === vscode_languageserver_protocol_1.DiagnosticSeverity.Error);
        if (errDiagnostics.length > 0) {
            for (const errDiagnostic of errDiagnostics) {
                logger.logLanguageServerDiagnostic(errDiagnostic, document);
            }
            return Promise.reject(ExitCode.FAILURE);
        }
        const nonErrDiagnostics = diagnostics.filter((diagnostic) => diagnostic.severity !== vscode_languageserver_protocol_1.DiagnosticSeverity.Error);
        for (const nonErrDiagnostic of nonErrDiagnostics) {
            logger.logLanguageServerDiagnostic(nonErrDiagnostic, document);
        }
        return document;
    });
}
exports.validateDocument = validateDocument;
function extractAstNodeFromFile(fileName, services, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return (yield extractDocumentFromFile(fileName, services, logger)).parseResult
            .value;
    });
}
exports.extractAstNodeFromFile = extractAstNodeFromFile;
function extractAstNodeFromString(modelString, services, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return (yield extractDocumentFromString(modelString, services, logger))
            .parseResult.value;
    });
}
exports.extractAstNodeFromString = extractAstNodeFromString;


/***/ }),

/***/ "./libs/interpreter-lib/src/validation-checks/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/interpreter-lib/src/validation-checks/runtime-parameter-literal.ts"), exports);


/***/ }),

/***/ "./libs/interpreter-lib/src/validation-checks/runtime-parameter-literal.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateRuntimeParameterLiteral = void 0;
const jayvee_language_server_1 = __webpack_require__("./libs/language-server/src/index.ts");
function validateRuntimeParameterLiteral(runtimeParameter, validationContext, evaluationContext) {
    checkRuntimeParameterValuePresence(runtimeParameter, validationContext, evaluationContext);
    if (validationContext.hasErrorOccurred()) {
        return;
    }
    checkRuntimeParameterValueParsing(runtimeParameter, validationContext, evaluationContext);
}
exports.validateRuntimeParameterLiteral = validateRuntimeParameterLiteral;
function checkRuntimeParameterValuePresence(runtimeParameter, validationContext, evaluationContext) {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const runtimeParameterName = runtimeParameter === null || runtimeParameter === void 0 ? void 0 : runtimeParameter.name;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (runtimeParameterName === undefined) {
        return;
    }
    if (!evaluationContext.hasValueForRuntimeParameter(runtimeParameterName)) {
        validationContext.accept('error', `A value needs to be provided by adding "-e ${runtimeParameterName}=<value>" to the command.`, { node: runtimeParameter });
    }
}
function checkRuntimeParameterValueParsing(runtimeParameter, validationContext, evaluationContext) {
    var _a, _b;
    const enclosingPropertyBody = getEnclosingPropertyBody(runtimeParameter);
    const type = 
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    (_a = enclosingPropertyBody.$container) === null || _a === void 0 ? void 0 : _a.type;
    const wrapper = (0, jayvee_language_server_1.getTypedObjectWrapper)(type);
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const propertyName = (_b = runtimeParameter.$container) === null || _b === void 0 ? void 0 : _b.name;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (propertyName === undefined) {
        return;
    }
    const propertySpec = wrapper === null || wrapper === void 0 ? void 0 : wrapper.getPropertySpecification(propertyName);
    if (propertySpec === undefined) {
        return;
    }
    const valuetype = propertySpec.type;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const runtimeParameterName = runtimeParameter === null || runtimeParameter === void 0 ? void 0 : runtimeParameter.name;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (runtimeParameterName === undefined) {
        return;
    }
    const runtimeParameterValue = evaluationContext.getValueForRuntimeParameter(runtimeParameterName, valuetype);
    if (runtimeParameterValue === undefined) {
        const rawValue = evaluationContext.runtimeParameterProvider.getRawValue(runtimeParameterName);
        validationContext.accept('error', `Unable to parse the value "${rawValue !== null && rawValue !== void 0 ? rawValue : ''}" as ${valuetype.getName()}.`, { node: runtimeParameter });
    }
}
function getEnclosingPropertyBody(runtimeParameter) {
    return runtimeParameter.$container.$container;
}


/***/ }),

/***/ "./libs/language-server/src/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/index.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluation.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.evaluateExpression = exports.evaluatePropertyValue = exports.EvaluationContext = exports.EvaluationStrategy = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const cell_range_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts");
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
// eslint-disable-next-line import/no-cycle
const operator_registry_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-registry.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
var EvaluationStrategy;
(function (EvaluationStrategy) {
    EvaluationStrategy[EvaluationStrategy["EXHAUSTIVE"] = 0] = "EXHAUSTIVE";
    EvaluationStrategy[EvaluationStrategy["LAZY"] = 1] = "LAZY";
})(EvaluationStrategy = exports.EvaluationStrategy || (exports.EvaluationStrategy = {}));
class EvaluationContext {
    constructor(runtimeParameterProvider) {
        this.runtimeParameterProvider = runtimeParameterProvider;
        this.variableValues = new Map();
        this.valueKeywordValue = undefined;
    }
    getValueFor(literal) {
        if ((0, ast_1.isReferenceLiteral)(literal)) {
            return this.getValueForReference(literal);
        }
        else if ((0, ast_1.isValueKeywordLiteral)(literal)) {
            return this.getValueForValueKeyword(literal);
        }
        (0, langium_1.assertUnreachable)(literal);
    }
    setValueForReference(refText, value) {
        this.variableValues.set(refText, value);
    }
    deleteValueForReference(refText) {
        this.variableValues.delete(refText);
    }
    getValueForReference(referenceLiteral) {
        var _a;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        const dereferenced = (_a = referenceLiteral === null || referenceLiteral === void 0 ? void 0 : referenceLiteral.value) === null || _a === void 0 ? void 0 : _a.ref;
        if (dereferenced === undefined) {
            return undefined;
        }
        if ((0, ast_1.isConstraintDefinition)(dereferenced)) {
            return dereferenced;
        }
        if ((0, ast_1.isTransformDefinition)(dereferenced)) {
            return dereferenced;
        }
        if ((0, ast_1.isTransformPortDefinition)(dereferenced)) {
            return this.variableValues.get(dereferenced.name);
        }
        if ((0, ast_1.isBlocktypeProperty)(dereferenced)) {
            return this.variableValues.get(dereferenced.name);
        }
        (0, langium_1.assertUnreachable)(dereferenced);
    }
    hasValueForRuntimeParameter(key) {
        return this.runtimeParameterProvider.hasValue(key);
    }
    getValueForRuntimeParameter(key, valuetype) {
        return this.runtimeParameterProvider.getParsedValue(key, valuetype);
    }
    setValueForValueKeyword(value) {
        this.valueKeywordValue = value;
    }
    deleteValueForValueKeyword() {
        this.valueKeywordValue = undefined;
    }
    getValueForValueKeyword(literal) {
        if (this.valueKeywordValue === undefined) {
            return undefined;
        }
        if (literal.lengthAccess) {
            (0, assert_1.strict)(primitive_valuetypes_1.PrimitiveValuetypes.Text.isInternalValueRepresentation(this.valueKeywordValue));
            return this.valueKeywordValue.length;
        }
        return this.valueKeywordValue;
    }
}
exports.EvaluationContext = EvaluationContext;
function evaluatePropertyValue(property, evaluationContext, valuetype) {
    var _a;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const propertyValue = property === null || property === void 0 ? void 0 : property.value;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    (0, assert_1.strict)(propertyValue !== undefined);
    if ((0, ast_1.isBlocktypeProperty)(propertyValue)) {
        // Properties of blocktypes are always undefined
        // because they are set in the block that instantiates the block type
        return undefined;
    }
    let result;
    if ((0, ast_1.isRuntimeParameterLiteral)(propertyValue)) {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        const runtimeParameterName = propertyValue === null || propertyValue === void 0 ? void 0 : propertyValue.name;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (runtimeParameterName === undefined) {
            result = undefined;
        }
        else {
            result = evaluationContext.getValueForRuntimeParameter(runtimeParameterName, valuetype);
        }
    }
    else if ((0, ast_1.isExpression)(propertyValue)) {
        result = evaluateExpression(propertyValue, evaluationContext);
    }
    else {
        (0, langium_1.assertUnreachable)(propertyValue);
    }
    (0, assert_1.strict)(result === undefined || valuetype.isInternalValueRepresentation(result), `Evaluation result ${(_a = result === null || result === void 0 ? void 0 : result.toString()) !== null && _a !== void 0 ? _a : 'undefined'} is not valid: Neither undefined, nor of type ${valuetype.getName()}`);
    return result;
}
exports.evaluatePropertyValue = evaluatePropertyValue;
function evaluateExpression(expression, evaluationContext, context = undefined, strategy = EvaluationStrategy.LAZY) {
    if (expression === undefined) {
        return undefined;
    }
    if ((0, ast_1.isExpressionLiteral)(expression)) {
        if ((0, ast_1.isFreeVariableLiteral)(expression)) {
            return evaluationContext.getValueFor(expression);
        }
        else if ((0, ast_1.isValueLiteral)(expression)) {
            return evaluateValueLiteral(expression, evaluationContext, context, strategy);
        }
        (0, langium_1.assertUnreachable)(expression);
    }
    if ((0, ast_1.isUnaryExpression)(expression)) {
        const operator = expression.operator;
        const evaluator = operator_registry_1.unaryOperatorRegistry[operator].evaluation;
        return evaluator.evaluate(expression, evaluationContext, strategy, context);
    }
    if ((0, ast_1.isBinaryExpression)(expression)) {
        const operator = expression.operator;
        const evaluator = operator_registry_1.binaryOperatorRegistry[operator].evaluation;
        return evaluator.evaluate(expression, evaluationContext, strategy, context);
    }
    (0, langium_1.assertUnreachable)(expression);
}
exports.evaluateExpression = evaluateExpression;
function evaluateValueLiteral(expression, evaluationContext, context = undefined, strategy = EvaluationStrategy.LAZY) {
    if ((0, ast_1.isCollectionLiteral)(expression)) {
        const evaluatedCollection = expression.values.map((v) => evaluateExpression(v, evaluationContext, context, strategy));
        if (!(0, typeguards_1.isEveryValueDefined)(evaluatedCollection)) {
            return undefined;
        }
        return evaluatedCollection;
    }
    if ((0, ast_1.isCellRangeLiteral)(expression)) {
        if (!cell_range_wrapper_1.CellRangeWrapper.canBeWrapped(expression)) {
            return undefined;
        }
        return new cell_range_wrapper_1.CellRangeWrapper(expression);
    }
    if ((0, ast_1.isRegexLiteral)(expression)) {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if ((expression === null || expression === void 0 ? void 0 : expression.value) === undefined) {
            return undefined;
        }
        return new RegExp(expression.value);
    }
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    return expression === null || expression === void 0 ? void 0 : expression.value;
}


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/addition-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AdditionOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class AdditionOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('+', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue + rightValue;
    }
}
exports.AdditionOperatorEvaluator = AdditionOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/and-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AndOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
class AndOperatorEvaluator extends operator_evaluator_1.BooleanShortCircuitOperatorEvaluator {
    constructor() {
        super('and');
    }
    canSkipRightOperandEvaluation(leftValue) {
        return leftValue === false;
    }
    getShortCircuitValue() {
        return false;
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue && rightValue;
    }
}
exports.AndOperatorEvaluator = AndOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/ceil-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CeilOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class CeilOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('ceil', typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(operandValue) {
        return Math.ceil(operandValue);
    }
}
exports.CeilOperatorEvaluator = CeilOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/division-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DivisionOperatorEvaluator = void 0;
const assert_1 = __webpack_require__("assert");
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class DivisionOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('/', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue, expression, context) {
        const resultingValue = leftValue / rightValue;
        if (!isFinite(resultingValue)) {
            (0, assert_1.strict)(rightValue === 0);
            context === null || context === void 0 ? void 0 : context.accept('error', 'Arithmetic error: division by zero', {
                node: expression,
            });
            return undefined;
        }
        return resultingValue;
    }
}
exports.DivisionOperatorEvaluator = DivisionOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/equality-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.EqualityOperatorEvaluator = void 0;
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class EqualityOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('==', typeguards_1.INTERNAL_VALUE_REPRESENTATION_TYPEGUARD, typeguards_1.INTERNAL_VALUE_REPRESENTATION_TYPEGUARD);
    }
    doEvaluate(left, right) {
        return left === right;
    }
}
exports.EqualityOperatorEvaluator = EqualityOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/floor-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.FloorOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class FloorOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('floor', typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(operandValue) {
        return Math.floor(operandValue);
    }
}
exports.FloorOperatorEvaluator = FloorOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/greater-equal-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.GreaterEqualOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class GreaterEqualOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('>=', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue >= rightValue;
    }
}
exports.GreaterEqualOperatorEvaluator = GreaterEqualOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/greater-than-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.GreaterThanOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class GreaterThanOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('>', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue > rightValue;
    }
}
exports.GreaterThanOperatorEvaluator = GreaterThanOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/in-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.InOperatorEvaluator = void 0;
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class InOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('in', isLeftOperandMatchingValueRepresentationTypeguard, isRightOperandMatchingValueRepresentationTypeguard);
    }
    doEvaluate(left, right) {
        return right.includes(left);
    }
}
exports.InOperatorEvaluator = InOperatorEvaluator;
const isLeftOperandMatchingValueRepresentationTypeguard = (value) => {
    return (0, typeguards_1.STRING_TYPEGUARD)(value) || (0, typeguards_1.NUMBER_TYPEGUARD)(value);
};
const isRightOperandMatchingValueRepresentationTypeguard = (value) => {
    return (Array.isArray(value) &&
        value.every(isLeftOperandMatchingValueRepresentationTypeguard));
};


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/inequality-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.InequalityOperatorEvaluator = void 0;
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class InequalityOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('!=', typeguards_1.INTERNAL_VALUE_REPRESENTATION_TYPEGUARD, typeguards_1.INTERNAL_VALUE_REPRESENTATION_TYPEGUARD);
    }
    doEvaluate(left, right) {
        return left !== right;
    }
}
exports.InequalityOperatorEvaluator = InequalityOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/less-equal-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LessEqualOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class LessEqualOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('<=', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue <= rightValue;
    }
}
exports.LessEqualOperatorEvaluator = LessEqualOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/less-than-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LessThanOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class LessThanOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('<', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue < rightValue;
    }
}
exports.LessThanOperatorEvaluator = LessThanOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/matches-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MatchesOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class MatchesOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('matches', typeguards_1.STRING_TYPEGUARD, typeguards_1.REGEXP_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return rightValue.test(leftValue);
    }
}
exports.MatchesOperatorEvaluator = MatchesOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/minus-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MinusOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class MinusOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('-', typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(operandValue) {
        return -operandValue;
    }
}
exports.MinusOperatorEvaluator = MinusOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/modulo-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ModuloOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class ModuloOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('%', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue, expression, context) {
        const resultingValue = leftValue % rightValue;
        if (!isFinite(resultingValue)) {
            if (rightValue === 0) {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Arithmetic error: modulo by zero', {
                    node: expression,
                });
            }
            else {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Unknown arithmetic error', {
                    node: expression,
                });
            }
            return undefined;
        }
        return resultingValue;
    }
}
exports.ModuloOperatorEvaluator = ModuloOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/multiplication-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MultiplicationOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class MultiplicationOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('*', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue * rightValue;
    }
}
exports.MultiplicationOperatorEvaluator = MultiplicationOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/not-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.NotOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class NotOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('not', typeguards_1.BOOLEAN_TYPEGUARD);
    }
    doEvaluate(operandValue) {
        return !operandValue;
    }
}
exports.NotOperatorEvaluator = NotOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/or-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.OrOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
class OrOperatorEvaluator extends operator_evaluator_1.BooleanShortCircuitOperatorEvaluator {
    constructor() {
        super('or');
    }
    canSkipRightOperandEvaluation(leftValue) {
        return leftValue === true;
    }
    getShortCircuitValue() {
        return true;
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue || rightValue;
    }
}
exports.OrOperatorEvaluator = OrOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/plus-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PlusOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class PlusOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('+', typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(operandValue) {
        return operandValue;
    }
}
exports.PlusOperatorEvaluator = PlusOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/pow-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PowOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class PowOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('pow', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue, expression, context) {
        const resultingValue = Math.pow(leftValue, rightValue);
        if (!isFinite(resultingValue)) {
            if (leftValue === 0 && rightValue < 0) {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Arithmetic error: zero raised to a negative number', { node: expression });
            }
            else {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Unknown arithmetic error', {
                    node: expression,
                });
            }
            return undefined;
        }
        return resultingValue;
    }
}
exports.PowOperatorEvaluator = PowOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/root-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RootOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class RootOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('root', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue, expression, context) {
        const resultingValue = Math.pow(leftValue, (1 / rightValue));
        if (!isFinite(resultingValue)) {
            if (leftValue === 0 && rightValue < 0) {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Arithmetic error: root of zero with negative degree', { node: expression });
            }
            else if (rightValue === 0) {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Arithmetic error: root of degree zero', {
                    node: expression,
                });
            }
            else {
                context === null || context === void 0 ? void 0 : context.accept('error', 'Unknown arithmetic error', {
                    node: expression,
                });
            }
            return undefined;
        }
        return resultingValue;
    }
}
exports.RootOperatorEvaluator = RootOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/round-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RoundOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class RoundOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('round', typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(operandValue) {
        return Math.round(operandValue);
    }
}
exports.RoundOperatorEvaluator = RoundOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/sqrt-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SqrtOperatorEvaluator = void 0;
const assert_1 = __webpack_require__("assert");
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class SqrtOperatorEvaluator extends operator_evaluator_1.DefaultUnaryOperatorEvaluator {
    constructor() {
        super('sqrt', typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(operandValue, expression, context) {
        const resultingValue = Math.sqrt(operandValue);
        if (!isFinite(resultingValue)) {
            (0, assert_1.strict)(operandValue < 0);
            context === null || context === void 0 ? void 0 : context.accept('error', 'Arithmetic error: square root of negative number', { node: expression });
            return undefined;
        }
        return resultingValue;
    }
}
exports.SqrtOperatorEvaluator = SqrtOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/subtraction-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SubtractionOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class SubtractionOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('-', typeguards_1.NUMBER_TYPEGUARD, typeguards_1.NUMBER_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return leftValue - rightValue;
    }
}
exports.SubtractionOperatorEvaluator = SubtractionOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/evaluators/xor-operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.XorOperatorEvaluator = void 0;
// eslint-disable-next-line import/no-cycle
const operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
class XorOperatorEvaluator extends operator_evaluator_1.DefaultBinaryOperatorEvaluator {
    constructor() {
        super('xor', typeguards_1.BOOLEAN_TYPEGUARD, typeguards_1.BOOLEAN_TYPEGUARD);
    }
    doEvaluate(leftValue, rightValue) {
        return (leftValue && !rightValue) || (!leftValue && rightValue);
    }
}
exports.XorOperatorEvaluator = XorOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
// eslint-disable-next-line import/no-cycle
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluation.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/expressions/internal-value-representation.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-registry.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/expressions/type-inference.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/internal-value-representation.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.internalValueToString = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
// eslint-disable-next-line import/no-cycle
const wrappers_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/index.ts");
function internalValueToString(valueRepresentation) {
    if (Array.isArray(valueRepresentation)) {
        return ('[ ' +
            valueRepresentation
                .map((item) => internalValueToString(item))
                .join(', ') +
            ' ]');
    }
    if (typeof valueRepresentation === 'boolean') {
        return String(valueRepresentation);
    }
    if (typeof valueRepresentation === 'number') {
        if (valueRepresentation === Number.POSITIVE_INFINITY) {
            return Number.MAX_VALUE.toLocaleString('fullwide', {
                useGrouping: false,
            });
        }
        if (valueRepresentation === Number.NEGATIVE_INFINITY) {
            return Number.MIN_VALUE.toLocaleString('fullwide', {
                useGrouping: false,
            });
        }
        return `${valueRepresentation}`;
    }
    if (typeof valueRepresentation === 'string') {
        return `"${valueRepresentation}"`;
    }
    if (valueRepresentation instanceof RegExp) {
        return valueRepresentation.source;
    }
    if (valueRepresentation instanceof wrappers_1.CellRangeWrapper) {
        return valueRepresentation.toString();
    }
    if ((0, ast_1.isConstraintDefinition)(valueRepresentation)) {
        return valueRepresentation.name;
    }
    if ((0, ast_1.isValuetypeAssignment)(valueRepresentation)) {
        return valueRepresentation.name;
    }
    if ((0, ast_1.isTransformDefinition)(valueRepresentation)) {
        return valueRepresentation.name;
    }
    if ((0, ast_1.isBlocktypeProperty)(valueRepresentation)) {
        return valueRepresentation.name;
    }
    throw new Error('Convert of this InternalValueRepresentation is not implemented');
}
exports.internalValueToString = internalValueToString;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/operator-evaluator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BooleanShortCircuitOperatorEvaluator = exports.DefaultBinaryOperatorEvaluator = exports.DefaultUnaryOperatorEvaluator = void 0;
const assert_1 = __webpack_require__("assert");
const evaluation_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluation.ts");
class DefaultUnaryOperatorEvaluator {
    constructor(operator, operandValueTypeguard) {
        this.operator = operator;
        this.operandValueTypeguard = operandValueTypeguard;
    }
    evaluate(expression, evaluationContext, strategy, validationContext) {
        (0, assert_1.strict)(expression.operator === this.operator);
        const operandValue = (0, evaluation_1.evaluateExpression)(expression.expression, evaluationContext, validationContext, strategy);
        if (operandValue === undefined) {
            return undefined;
        }
        (0, assert_1.strict)(this.operandValueTypeguard(operandValue));
        return this.doEvaluate(operandValue, expression, validationContext);
    }
}
exports.DefaultUnaryOperatorEvaluator = DefaultUnaryOperatorEvaluator;
class DefaultBinaryOperatorEvaluator {
    constructor(operator, leftValueTypeguard, rightValueTypeguard) {
        this.operator = operator;
        this.leftValueTypeguard = leftValueTypeguard;
        this.rightValueTypeguard = rightValueTypeguard;
    }
    evaluate(expression, evaluationContext, strategy, validationContext) {
        (0, assert_1.strict)(expression.operator === this.operator);
        const leftValue = (0, evaluation_1.evaluateExpression)(expression.left, evaluationContext, validationContext, strategy);
        if (strategy === evaluation_1.EvaluationStrategy.LAZY && leftValue === undefined) {
            return undefined;
        }
        const rightValue = (0, evaluation_1.evaluateExpression)(expression.right, evaluationContext, validationContext, strategy);
        if (leftValue === undefined || rightValue === undefined) {
            return undefined;
        }
        (0, assert_1.strict)(this.leftValueTypeguard(leftValue));
        (0, assert_1.strict)(this.rightValueTypeguard(rightValue));
        return this.doEvaluate(leftValue, rightValue, expression, validationContext);
    }
}
exports.DefaultBinaryOperatorEvaluator = DefaultBinaryOperatorEvaluator;
/**
 * This class serves as a base for boolean operators that support short-circuit evaluation.
 * Short-circuit evaluation means that the right operand is not evaluated
 * if the resulting value can be determined by solely evaluating the left operand.
 */
class BooleanShortCircuitOperatorEvaluator {
    constructor(operator) {
        this.operator = operator;
    }
    evaluate(expression, evaluationContext, strategy, validationContext) {
        (0, assert_1.strict)(expression.operator === this.operator);
        const leftValue = (0, evaluation_1.evaluateExpression)(expression.left, evaluationContext, validationContext, strategy);
        (0, assert_1.strict)(leftValue === undefined || typeof leftValue === 'boolean');
        if (strategy === evaluation_1.EvaluationStrategy.LAZY) {
            if (leftValue === undefined) {
                return undefined;
            }
            if (this.canSkipRightOperandEvaluation(leftValue)) {
                return this.getShortCircuitValue();
            }
        }
        const rightValue = (0, evaluation_1.evaluateExpression)(expression.right, evaluationContext, validationContext, strategy);
        if (leftValue === undefined || rightValue === undefined) {
            return undefined;
        }
        (0, assert_1.strict)(typeof rightValue === 'boolean');
        return this.doEvaluate(leftValue, rightValue);
    }
}
exports.BooleanShortCircuitOperatorEvaluator = BooleanShortCircuitOperatorEvaluator;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/operator-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.binaryOperatorRegistry = exports.unaryOperatorRegistry = void 0;
const addition_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/addition-operator-evaluator.ts");
const and_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/and-operator-evaluator.ts");
const ceil_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/ceil-operator-evaluator.ts");
const division_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/division-operator-evaluator.ts");
const equality_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/equality-operator-evaluator.ts");
const floor_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/floor-operator-evaluator.ts");
const greater_equal_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/greater-equal-operator-evaluator.ts");
const greater_than_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/greater-than-operator-evaluator.ts");
const in_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/in-operator-evaluator.ts");
const inequality_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/inequality-operator-evaluator.ts");
const less_equal_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/less-equal-operator-evaluator.ts");
const less_than_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/less-than-operator-evaluator.ts");
const matches_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/matches-operator-evaluator.ts");
const minus_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/minus-operator-evaluator.ts");
const modulo_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/modulo-operator-evaluator.ts");
const multiplication_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/multiplication-operator-evaluator.ts");
const not_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/not-operator-evaluator.ts");
const or_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/or-operator-evaluator.ts");
const plus_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/plus-operator-evaluator.ts");
const pow_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/pow-operator-evaluator.ts");
const root_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/root-operator-evaluator.ts");
const round_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/round-operator-evaluator.ts");
const sqrt_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/sqrt-operator-evaluator.ts");
const subtraction_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/subtraction-operator-evaluator.ts");
const xor_operator_evaluator_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluators/xor-operator-evaluator.ts");
const basic_arithmetic_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/basic-arithmetic-operator-type-computer.ts");
const division_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/division-operator-type-computer.ts");
const equality_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/equality-operator-type-computer.ts");
const exponential_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/exponential-operator-type-computer.ts");
const in_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/in-operator-type-computer.ts");
const integer_conversion_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/integer-conversion-operator-type-computer.ts");
const logical_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/logical-operator-type-computer.ts");
const matches_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/matches-operator-type-computer.ts");
const not_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/not-operator-type-computer.ts");
const relational_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/relational-operator-type-computer.ts");
const sign_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/sign-operator-type-computer.ts");
const sqrt_operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-computers/sqrt-operator-type-computer.ts");
exports.unaryOperatorRegistry = {
    not: {
        typeInference: new not_operator_type_computer_1.NotOperatorTypeComputer(),
        evaluation: new not_operator_evaluator_1.NotOperatorEvaluator(),
    },
    '+': {
        typeInference: new sign_operator_type_computer_1.SignOperatorTypeComputer(),
        evaluation: new plus_operator_evaluator_1.PlusOperatorEvaluator(),
    },
    '-': {
        typeInference: new sign_operator_type_computer_1.SignOperatorTypeComputer(),
        evaluation: new minus_operator_evaluator_1.MinusOperatorEvaluator(),
    },
    sqrt: {
        typeInference: new sqrt_operator_type_computer_1.SqrtOperatorTypeComputer(),
        evaluation: new sqrt_operator_evaluator_1.SqrtOperatorEvaluator(),
    },
    floor: {
        typeInference: new integer_conversion_operator_type_computer_1.IntegerConversionOperatorTypeComputer(),
        evaluation: new floor_operator_evaluator_1.FloorOperatorEvaluator(),
    },
    ceil: {
        typeInference: new integer_conversion_operator_type_computer_1.IntegerConversionOperatorTypeComputer(),
        evaluation: new ceil_operator_evaluator_1.CeilOperatorEvaluator(),
    },
    round: {
        typeInference: new integer_conversion_operator_type_computer_1.IntegerConversionOperatorTypeComputer(),
        evaluation: new round_operator_evaluator_1.RoundOperatorEvaluator(),
    },
};
exports.binaryOperatorRegistry = {
    pow: {
        typeInference: new exponential_operator_type_computer_1.ExponentialOperatorTypeComputer(),
        evaluation: new pow_operator_evaluator_1.PowOperatorEvaluator(),
    },
    root: {
        typeInference: new exponential_operator_type_computer_1.ExponentialOperatorTypeComputer(),
        evaluation: new root_operator_evaluator_1.RootOperatorEvaluator(),
    },
    '*': {
        typeInference: new basic_arithmetic_operator_type_computer_1.BasicArithmeticOperatorTypeComputer(),
        evaluation: new multiplication_operator_evaluator_1.MultiplicationOperatorEvaluator(),
    },
    '/': {
        typeInference: new division_operator_type_computer_1.DivisionOperatorTypeComputer(),
        evaluation: new division_operator_evaluator_1.DivisionOperatorEvaluator(),
    },
    '%': {
        typeInference: new division_operator_type_computer_1.DivisionOperatorTypeComputer(),
        evaluation: new modulo_operator_evaluator_1.ModuloOperatorEvaluator(),
    },
    '+': {
        typeInference: new basic_arithmetic_operator_type_computer_1.BasicArithmeticOperatorTypeComputer(),
        evaluation: new addition_operator_evaluator_1.AdditionOperatorEvaluator(),
    },
    '-': {
        typeInference: new basic_arithmetic_operator_type_computer_1.BasicArithmeticOperatorTypeComputer(),
        evaluation: new subtraction_operator_evaluator_1.SubtractionOperatorEvaluator(),
    },
    matches: {
        typeInference: new matches_operator_type_computer_1.MatchesOperatorTypeComputer(),
        evaluation: new matches_operator_evaluator_1.MatchesOperatorEvaluator(),
    },
    in: {
        typeInference: new in_operator_type_computer_1.InOperatorTypeComputer(),
        evaluation: new in_operator_evaluator_1.InOperatorEvaluator(),
    },
    '<': {
        typeInference: new relational_operator_type_computer_1.RelationalOperatorTypeComputer(),
        evaluation: new less_than_operator_evaluator_1.LessThanOperatorEvaluator(),
    },
    '<=': {
        typeInference: new relational_operator_type_computer_1.RelationalOperatorTypeComputer(),
        evaluation: new less_equal_operator_evaluator_1.LessEqualOperatorEvaluator(),
    },
    '>': {
        typeInference: new relational_operator_type_computer_1.RelationalOperatorTypeComputer(),
        evaluation: new greater_than_operator_evaluator_1.GreaterThanOperatorEvaluator(),
    },
    '>=': {
        typeInference: new relational_operator_type_computer_1.RelationalOperatorTypeComputer(),
        evaluation: new greater_equal_operator_evaluator_1.GreaterEqualOperatorEvaluator(),
    },
    '==': {
        typeInference: new equality_operator_type_computer_1.EqualityOperatorTypeComputer(),
        evaluation: new equality_operator_evaluator_1.EqualityOperatorEvaluator(),
    },
    '!=': {
        typeInference: new equality_operator_type_computer_1.EqualityOperatorTypeComputer(),
        evaluation: new inequality_operator_evaluator_1.InequalityOperatorEvaluator(),
    },
    xor: {
        typeInference: new logical_operator_type_computer_1.LogicalOperatorTypeComputer(),
        evaluation: new xor_operator_evaluator_1.XorOperatorEvaluator(),
    },
    and: {
        typeInference: new logical_operator_type_computer_1.LogicalOperatorTypeComputer(),
        evaluation: new and_operator_evaluator_1.AndOperatorEvaluator(),
    },
    or: {
        typeInference: new logical_operator_type_computer_1.LogicalOperatorTypeComputer(),
        evaluation: new or_operator_evaluator_1.OrOperatorEvaluator(),
    },
};


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.generateUnexpectedTypeMessage = exports.DefaultBinaryOperatorTypeComputer = exports.DefaultUnaryOperatorTypeComputer = void 0;
class DefaultUnaryOperatorTypeComputer {
    constructor(expectedOperandType) {
        this.expectedOperandType = expectedOperandType;
    }
    computeType(operandType, expression, context) {
        if (!operandType.isConvertibleTo(this.expectedOperandType)) {
            context === null || context === void 0 ? void 0 : context.accept('error', generateUnexpectedTypeMessage(this.expectedOperandType, operandType), {
                node: expression.expression,
            });
            return undefined;
        }
        return this.doComputeType(operandType);
    }
}
exports.DefaultUnaryOperatorTypeComputer = DefaultUnaryOperatorTypeComputer;
class DefaultBinaryOperatorTypeComputer {
    constructor(expectedLeftOperandType, expectedRightOperandType) {
        this.expectedLeftOperandType = expectedLeftOperandType;
        this.expectedRightOperandType = expectedRightOperandType;
    }
    computeType(leftOperandType, rightOperandType, expression, context) {
        let typeErrorOccurred = false;
        if (!leftOperandType.isConvertibleTo(this.expectedLeftOperandType)) {
            context === null || context === void 0 ? void 0 : context.accept('error', generateUnexpectedTypeMessage(this.expectedLeftOperandType, leftOperandType), {
                node: expression.left,
            });
            typeErrorOccurred = true;
        }
        if (!rightOperandType.isConvertibleTo(this.expectedRightOperandType)) {
            context === null || context === void 0 ? void 0 : context.accept('error', generateUnexpectedTypeMessage(this.expectedRightOperandType, rightOperandType), {
                node: expression.right,
            });
            typeErrorOccurred = true;
        }
        if (typeErrorOccurred) {
            return undefined;
        }
        return this.doComputeType(leftOperandType, rightOperandType);
    }
}
exports.DefaultBinaryOperatorTypeComputer = DefaultBinaryOperatorTypeComputer;
function generateUnexpectedTypeMessage(expectedType, actualType) {
    return `The operand needs to be of type ${expectedType.getName()} but is of type ${actualType.getName()}`;
}
exports.generateUnexpectedTypeMessage = generateUnexpectedTypeMessage;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/basic-arithmetic-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BasicArithmeticOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class BasicArithmeticOperatorTypeComputer extends operator_type_computer_1.DefaultBinaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal, primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType(leftOperandType, rightOperandType) {
        if (leftOperandType === primitive_valuetypes_1.PrimitiveValuetypes.Integer &&
            rightOperandType === primitive_valuetypes_1.PrimitiveValuetypes.Integer) {
            return primitive_valuetypes_1.PrimitiveValuetypes.Integer;
        }
        return primitive_valuetypes_1.PrimitiveValuetypes.Decimal;
    }
}
exports.BasicArithmeticOperatorTypeComputer = BasicArithmeticOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/division-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.DivisionOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class DivisionOperatorTypeComputer extends operator_type_computer_1.DefaultBinaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal, primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Decimal;
    }
}
exports.DivisionOperatorTypeComputer = DivisionOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/equality-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.EqualityOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
class EqualityOperatorTypeComputer {
    constructor() {
        this.ALLOWED_OPERAND_TYPES = [
            primitive_valuetypes_1.PrimitiveValuetypes.Boolean,
            primitive_valuetypes_1.PrimitiveValuetypes.Text,
            primitive_valuetypes_1.PrimitiveValuetypes.Integer,
            primitive_valuetypes_1.PrimitiveValuetypes.Decimal,
        ];
    }
    computeType(leftOperandType, rightOperandType, expression, context) {
        const isLeftOperandTypeValid = this.ALLOWED_OPERAND_TYPES.includes(leftOperandType);
        const isRightOperandTypeValid = this.ALLOWED_OPERAND_TYPES.includes(rightOperandType);
        if (!isLeftOperandTypeValid || !isRightOperandTypeValid) {
            if (!isLeftOperandTypeValid) {
                context === null || context === void 0 ? void 0 : context.accept('error', `Operator does not support type ${leftOperandType.getName()}`, {
                    node: expression.left,
                });
            }
            if (!isRightOperandTypeValid) {
                context === null || context === void 0 ? void 0 : context.accept('error', `Operator does not support type ${rightOperandType.getName()}`, {
                    node: expression.right,
                });
            }
            return undefined;
        }
        if (!leftOperandType.isConvertibleTo(rightOperandType) &&
            !rightOperandType.isConvertibleTo(leftOperandType)) {
            context === null || context === void 0 ? void 0 : context.accept('error', `The types of the operands need to be equal but they differ (left: ${leftOperandType.getName()}, right: ${rightOperandType.getName()})`, { node: expression });
            return undefined;
        }
        return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    }
}
exports.EqualityOperatorTypeComputer = EqualityOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/exponential-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ExponentialOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class ExponentialOperatorTypeComputer extends operator_type_computer_1.DefaultBinaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal, primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Decimal;
    }
}
exports.ExponentialOperatorTypeComputer = ExponentialOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/in-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.InOperatorTypeComputer = void 0;
const assert_1 = __webpack_require__("assert");
// eslint-disable-next-line import/no-cycle
const value_type_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/index.ts");
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
class InOperatorTypeComputer {
    constructor() {
        this.ALLOWED_LEFT_OPERAND_TYPES = [
            primitive_valuetypes_1.PrimitiveValuetypes.Text,
            primitive_valuetypes_1.PrimitiveValuetypes.Integer,
            primitive_valuetypes_1.PrimitiveValuetypes.Decimal,
        ];
        this.ALLOWED_RIGHT_OPERAND_TYPES = this.ALLOWED_LEFT_OPERAND_TYPES.map((v) => new value_type_1.CollectionValuetype(v));
    }
    computeType(leftOperandType, rightOperandType, expression, context) {
        const isLeftOperandTypeValid = this.ALLOWED_LEFT_OPERAND_TYPES.includes(leftOperandType);
        const isRightOperandTypeValid = this.ALLOWED_RIGHT_OPERAND_TYPES.some((v) => v.equals(rightOperandType));
        if (!isLeftOperandTypeValid || !isRightOperandTypeValid) {
            if (!isLeftOperandTypeValid) {
                context === null || context === void 0 ? void 0 : context.accept('error', `Operator does not support type ${leftOperandType.getName()}`, {
                    node: expression.left,
                });
            }
            if (!isRightOperandTypeValid) {
                context === null || context === void 0 ? void 0 : context.accept('error', `Operator does not support type ${rightOperandType.getName()}`, {
                    node: expression.right,
                });
            }
            return undefined;
        }
        (0, assert_1.strict)((0, value_type_1.isCollectionValuetype)(rightOperandType, primitive_valuetypes_1.PrimitiveValuetypes.Decimal) ||
            (0, value_type_1.isCollectionValuetype)(rightOperandType, primitive_valuetypes_1.PrimitiveValuetypes.Integer) ||
            (0, value_type_1.isCollectionValuetype)(rightOperandType, primitive_valuetypes_1.PrimitiveValuetypes.Text));
        // allow 3 in [3.5, 5.3]
        const isLeftConvertibleToRight = leftOperandType.isConvertibleTo(rightOperandType.elementType);
        // allow 3.5 in [3.0, 2.0]
        const isRightConvertibleToLeft = rightOperandType.elementType.isConvertibleTo(leftOperandType);
        if (!isLeftConvertibleToRight && !isRightConvertibleToLeft) {
            context === null || context === void 0 ? void 0 : context.accept('error', `The type of the left operand needs to be compatible to the collection element type of the right operand but they differ (left: ${leftOperandType.getName()}, right: ${rightOperandType.elementType.getName()})`, { node: expression });
            return undefined;
        }
        return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    }
}
exports.InOperatorTypeComputer = InOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/integer-conversion-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.IntegerConversionOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class IntegerConversionOperatorTypeComputer extends operator_type_computer_1.DefaultUnaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Integer;
    }
}
exports.IntegerConversionOperatorTypeComputer = IntegerConversionOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/logical-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LogicalOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class LogicalOperatorTypeComputer extends operator_type_computer_1.DefaultBinaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Boolean, primitive_valuetypes_1.PrimitiveValuetypes.Boolean);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    }
}
exports.LogicalOperatorTypeComputer = LogicalOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/matches-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MatchesOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class MatchesOperatorTypeComputer extends operator_type_computer_1.DefaultBinaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Text, primitive_valuetypes_1.PrimitiveValuetypes.Regex);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    }
}
exports.MatchesOperatorTypeComputer = MatchesOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/not-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.NotOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class NotOperatorTypeComputer extends operator_type_computer_1.DefaultUnaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Boolean);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    }
}
exports.NotOperatorTypeComputer = NotOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/relational-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RelationalOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class RelationalOperatorTypeComputer extends operator_type_computer_1.DefaultBinaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal, primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    }
}
exports.RelationalOperatorTypeComputer = RelationalOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/sign-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SignOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class SignOperatorTypeComputer extends operator_type_computer_1.DefaultUnaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType(operandType) {
        return operandType;
    }
}
exports.SignOperatorTypeComputer = SignOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-computers/sqrt-operator-type-computer.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.SqrtOperatorTypeComputer = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const operator_type_computer_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-type-computer.ts");
class SqrtOperatorTypeComputer extends operator_type_computer_1.DefaultUnaryOperatorTypeComputer {
    constructor() {
        super(primitive_valuetypes_1.PrimitiveValuetypes.Decimal);
    }
    doComputeType() {
        return primitive_valuetypes_1.PrimitiveValuetypes.Decimal;
    }
}
exports.SqrtOperatorTypeComputer = SqrtOperatorTypeComputer;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/type-inference.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.inferExpressionType = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
// eslint-disable-next-line import/no-cycle
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const wrappers_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/index.ts");
const empty_collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/empty-collection-valuetype.ts");
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const valuetype_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype-util.ts");
const operator_registry_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/operator-registry.ts");
const typeguards_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/typeguards.ts");
function inferExpressionType(expression, context) {
    if (expression === undefined) {
        return undefined;
    }
    if ((0, ast_1.isExpressionLiteral)(expression)) {
        return inferTypeFromExpressionLiteral(expression, context);
    }
    if ((0, ast_1.isUnaryExpression)(expression)) {
        const innerType = inferExpressionType(expression.expression, context);
        if (innerType === undefined) {
            return undefined;
        }
        const operator = expression.operator;
        const typeComputer = operator_registry_1.unaryOperatorRegistry[operator].typeInference;
        return typeComputer.computeType(innerType, expression, context);
    }
    if ((0, ast_1.isBinaryExpression)(expression)) {
        const leftType = inferExpressionType(expression.left, context);
        const rightType = inferExpressionType(expression.right, context);
        if (leftType === undefined || rightType === undefined) {
            return undefined;
        }
        const operator = expression.operator;
        const typeComputer = operator_registry_1.binaryOperatorRegistry[operator].typeInference;
        return typeComputer.computeType(leftType, rightType, expression, context);
    }
    (0, langium_1.assertUnreachable)(expression);
}
exports.inferExpressionType = inferExpressionType;
/**
 * @returns the resolved valuetype or undefined (e.g. if a reference is not resolved)
 */
function inferTypeFromExpressionLiteral(expression, context) {
    if ((0, ast_1.isValueLiteral)(expression)) {
        if ((0, ast_1.isTextLiteral)(expression)) {
            return primitive_valuetypes_1.PrimitiveValuetypes.Text;
        }
        else if ((0, ast_1.isBooleanLiteral)(expression)) {
            return primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
        }
        else if ((0, ast_1.isNumericLiteral)(expression)) {
            return inferNumericType(expression);
        }
        else if ((0, ast_1.isCellRangeLiteral)(expression)) {
            return primitive_valuetypes_1.PrimitiveValuetypes.CellRange;
        }
        else if ((0, ast_1.isRegexLiteral)(expression)) {
            return primitive_valuetypes_1.PrimitiveValuetypes.Regex;
        }
        else if ((0, ast_1.isValuetypeAssignmentLiteral)(expression)) {
            return primitive_valuetypes_1.PrimitiveValuetypes.ValuetypeAssignment;
        }
        else if ((0, ast_1.isCollectionLiteral)(expression)) {
            return inferCollectionType(expression, context);
        }
        (0, langium_1.assertUnreachable)(expression);
    }
    else if ((0, ast_1.isFreeVariableLiteral)(expression)) {
        if ((0, ast_1.isValueKeywordLiteral)(expression)) {
            return inferTypeFromValueKeyword(expression, context);
        }
        else if ((0, ast_1.isReferenceLiteral)(expression)) {
            return inferTypeFromReferenceLiteral(expression);
        }
        (0, langium_1.assertUnreachable)(expression);
    }
    (0, langium_1.assertUnreachable)(expression);
}
/**
 * Infers the numeric type dependent on the value parsed to TypeScript.
 * Thus, the inferred type might differ from the literal type.
 * E.g., 3.0 is currently interpreted as integer but is a DecimalLiteral.
 */
function inferNumericType(expression) {
    if (Number.isInteger(expression.value)) {
        return primitive_valuetypes_1.PrimitiveValuetypes.Integer;
    }
    return primitive_valuetypes_1.PrimitiveValuetypes.Decimal;
}
function inferCollectionType(collection, context) {
    const elementValuetypes = inferCollectionElementTypes(collection, context);
    if (elementValuetypes === undefined) {
        return undefined;
    }
    const stacks = elementValuetypes.map(getValuetypeHierarchyStack);
    if (stacks.length === 0) {
        return empty_collection_valuetype_1.EmptyCollection;
    }
    if (stacks.length === 1) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const stack = stacks[0];
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const resultingInnerType = stack[stack.length - 1];
        return new wrappers_1.CollectionValuetype(resultingInnerType);
    }
    const primitiveValuetypes = stacks.map((stack) => stack[0]);
    const commonPrimitiveValuetype = pickCommonPrimitiveValuetype(primitiveValuetypes);
    if (commonPrimitiveValuetype === undefined) {
        context === null || context === void 0 ? void 0 : context.accept('error', 'The type of the collection cannot be inferred from its elements', {
            node: collection,
        });
        return undefined;
    }
    const commonAtomicValuetype = pickCommonAtomicValuetype(stacks);
    if (commonAtomicValuetype === undefined) {
        return new wrappers_1.CollectionValuetype(commonPrimitiveValuetype);
    }
    return new wrappers_1.CollectionValuetype(commonAtomicValuetype);
}
function inferCollectionElementTypes(collection, context) {
    const elementValuetypes = collection.values.map((value) => inferExpressionType(value, context));
    if (!(0, typeguards_1.isEveryValueDefined)(elementValuetypes)) {
        return undefined;
    }
    return elementValuetypes;
}
function getValuetypeHierarchyStack(valuetype) {
    if ((0, wrappers_1.isPrimitiveValuetype)(valuetype)) {
        return [valuetype];
    }
    else if ((0, wrappers_1.isAtomicValuetype)(valuetype)) {
        const supertype = valuetype.getSupertype();
        (0, assert_1.strict)(supertype !== undefined);
        return [...getValuetypeHierarchyStack(supertype), valuetype];
    }
    throw new Error('Should be unreachable, encountered an unknown kind of valuetype');
}
function pickCommonPrimitiveValuetype(primitiveValuetypes) {
    (0, assert_1.strict)(primitiveValuetypes.length > 0);
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    let resultingType = primitiveValuetypes[0];
    for (let i = 1; i < primitiveValuetypes.length; ++i) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const currentType = primitiveValuetypes[i];
        if (currentType.isConvertibleTo(resultingType)) {
            continue;
        }
        if (resultingType.isConvertibleTo(currentType)) {
            // Pick the more general type as a result
            resultingType = currentType;
            continue;
        }
        // Unable to convert the valuetypes into each other, so there is no common primitive valuetype
        return undefined;
    }
    return resultingType;
}
function pickCommonAtomicValuetype(stacks) {
    const minimumStackLength = Math.min(...stacks.map((stack) => stack.length));
    let resultingType = undefined;
    for (let stackLevel = 1; stackLevel < minimumStackLength; ++stackLevel) {
        const typesOfCurrentLevel = stacks.map(
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        (stack) => stack[stackLevel]);
        if (!areAllTypesEqual(typesOfCurrentLevel)) {
            // Return the common valuetype of the previous level
            return resultingType;
        }
        // Pick any type of the current level since they are all equal
        resultingType = typesOfCurrentLevel[0];
    }
    return resultingType;
}
function areAllTypesEqual(types) {
    for (let i = 1; i < types.length; i++) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        if (!types[i - 1].equals(types[i])) {
            return false;
        }
    }
    return true;
}
function inferTypeFromValueKeyword(expression, context) {
    const expressionConstraintContainer = (0, model_util_1.getNextAstNodeContainer)(expression, ast_1.isExpressionConstraintDefinition);
    if (expressionConstraintContainer === undefined) {
        context === null || context === void 0 ? void 0 : context.accept('error', 'The value keyword is not allowed in this context', {
            node: expression,
        });
        return undefined;
    }
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const valuetype = (0, valuetype_util_1.createValuetype)(expressionConstraintContainer === null || expressionConstraintContainer === void 0 ? void 0 : expressionConstraintContainer.valuetype);
    if (valuetype === undefined) {
        return undefined;
    }
    if (expression.lengthAccess) {
        if (!valuetype.isConvertibleTo(primitive_valuetypes_1.PrimitiveValuetypes.Text)) {
            context === null || context === void 0 ? void 0 : context.accept('error', 'The length can only be accessed from text values ', {
                node: expression,
                keyword: 'length',
            });
            return undefined;
        }
        return primitive_valuetypes_1.PrimitiveValuetypes.Integer;
    }
    return valuetype;
}
function inferTypeFromReferenceLiteral(expression) {
    var _a;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const referenced = (_a = expression === null || expression === void 0 ? void 0 : expression.value) === null || _a === void 0 ? void 0 : _a.ref;
    if (referenced === undefined) {
        return undefined;
    }
    if ((0, ast_1.isConstraintDefinition)(referenced)) {
        return primitive_valuetypes_1.PrimitiveValuetypes.Constraint;
    }
    if ((0, ast_1.isTransformDefinition)(referenced)) {
        return primitive_valuetypes_1.PrimitiveValuetypes.Transform;
    }
    if ((0, ast_1.isTransformPortDefinition)(referenced) ||
        (0, ast_1.isBlocktypeProperty)(referenced)) {
        const valueType = referenced.valueType;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (valueType === undefined) {
            return undefined;
        }
        return (0, valuetype_util_1.createValuetype)(valueType);
    }
    (0, langium_1.assertUnreachable)(referenced);
}


/***/ }),

/***/ "./libs/language-server/src/lib/ast/expressions/typeguards.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isEveryValueDefined = exports.REGEXP_TYPEGUARD = exports.STRING_TYPEGUARD = exports.BOOLEAN_TYPEGUARD = exports.NUMBER_TYPEGUARD = exports.INTERNAL_VALUE_REPRESENTATION_TYPEGUARD = void 0;
const INTERNAL_VALUE_REPRESENTATION_TYPEGUARD = (value) => {
    return true;
};
exports.INTERNAL_VALUE_REPRESENTATION_TYPEGUARD = INTERNAL_VALUE_REPRESENTATION_TYPEGUARD;
const NUMBER_TYPEGUARD = (value) => {
    return typeof value === 'number';
};
exports.NUMBER_TYPEGUARD = NUMBER_TYPEGUARD;
const BOOLEAN_TYPEGUARD = (value) => {
    return typeof value === 'boolean';
};
exports.BOOLEAN_TYPEGUARD = BOOLEAN_TYPEGUARD;
const STRING_TYPEGUARD = (value) => {
    return typeof value === 'string';
};
exports.STRING_TYPEGUARD = STRING_TYPEGUARD;
const REGEXP_TYPEGUARD = (value) => {
    return value instanceof RegExp;
};
exports.REGEXP_TYPEGUARD = REGEXP_TYPEGUARD;
function isEveryValueDefined(array) {
    return array.every((value) => value !== undefined);
}
exports.isEveryValueDefined = isEveryValueDefined;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/generated/ast.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/******************************************************************************
 * This file was generated by langium-cli 1.2.0.
 * DO NOT EDIT MANUALLY!
 ******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isExpressionConstraintDefinition = exports.ExpressionConstraintDefinition = exports.isConstrainttypeProperty = exports.ConstrainttypeProperty = exports.isCompositeBlocktypeDefinition = exports.CompositeBlocktypeDefinition = exports.isColumnLiteral = exports.ColumnLiteral = exports.isColumnId = exports.ColumnId = exports.isCollectionLiteral = exports.CollectionLiteral = exports.isCellReference = exports.CellReference = exports.isCellLiteral = exports.CellLiteral = exports.isBuiltinConstrainttypeDefinition = exports.BuiltinConstrainttypeDefinition = exports.isBuiltinBlocktypeDefinition = exports.BuiltinBlocktypeDefinition = exports.isBooleanLiteral = exports.BooleanLiteral = exports.isBlocktypeProperty = exports.BlocktypeProperty = exports.isBlocktypePipeline = exports.BlocktypePipeline = exports.isBlocktypeOutput = exports.BlocktypeOutput = exports.isBlocktypeInput = exports.BlocktypeInput = exports.isBlockDefinition = exports.BlockDefinition = exports.isBinaryExpression = exports.BinaryExpression = exports.isValueLiteral = exports.ValueLiteral = exports.isReferenceableBlocktypeDefinition = exports.ReferenceableBlocktypeDefinition = exports.isReferencable = exports.Referencable = exports.isFreeVariableLiteral = exports.FreeVariableLiteral = exports.isExpressionLiteral = exports.ExpressionLiteral = exports.isExpression = exports.Expression = exports.isConstraintDefinition = exports.ConstraintDefinition = exports.isCellRangeLiteral = exports.CellRangeLiteral = void 0;
exports.isValuetypeGenericDefinition = exports.ValuetypeGenericDefinition = exports.isValuetypeDefinition = exports.ValuetypeDefinition = exports.isValuetypeAssignmentLiteral = exports.ValuetypeAssignmentLiteral = exports.isValuetypeAssignment = exports.ValuetypeAssignment = exports.isValueKeywordLiteral = exports.ValueKeywordLiteral = exports.isUnaryExpression = exports.UnaryExpression = exports.isTypedConstraintDefinition = exports.TypedConstraintDefinition = exports.isTransformPortDefinition = exports.TransformPortDefinition = exports.isTransformOutputAssignment = exports.TransformOutputAssignment = exports.isTransformDefinition = exports.TransformDefinition = exports.isTransformBody = exports.TransformBody = exports.isTextLiteral = exports.TextLiteral = exports.isRuntimeParameterLiteral = exports.RuntimeParameterLiteral = exports.isRowLiteral = exports.RowLiteral = exports.isRowId = exports.RowId = exports.isRegexLiteral = exports.RegexLiteral = exports.isReferenceLiteral = exports.ReferenceLiteral = exports.isRangeLiteral = exports.RangeLiteral = exports.isPropertyBody = exports.PropertyBody = exports.isPropertyAssignment = exports.PropertyAssignment = exports.isPipelineDefinition = exports.PipelineDefinition = exports.isPipeDefinition = exports.PipeDefinition = exports.isNumericLiteral = exports.NumericLiteral = exports.isJayveeModel = exports.JayveeModel = exports.isIotypeDefinition = exports.IotypeDefinition = void 0;
exports.reflection = exports.JayveeAstReflection = exports.isValuetypeReference = exports.ValuetypeReference = exports.isValuetypeGenericsDefinition = exports.ValuetypeGenericsDefinition = void 0;
/* eslint-disable */
const langium_1 = __webpack_require__("langium");
exports.CellRangeLiteral = 'CellRangeLiteral';
function isCellRangeLiteral(item) {
    return exports.reflection.isInstance(item, exports.CellRangeLiteral);
}
exports.isCellRangeLiteral = isCellRangeLiteral;
exports.ConstraintDefinition = 'ConstraintDefinition';
function isConstraintDefinition(item) {
    return exports.reflection.isInstance(item, exports.ConstraintDefinition);
}
exports.isConstraintDefinition = isConstraintDefinition;
exports.Expression = 'Expression';
function isExpression(item) {
    return exports.reflection.isInstance(item, exports.Expression);
}
exports.isExpression = isExpression;
exports.ExpressionLiteral = 'ExpressionLiteral';
function isExpressionLiteral(item) {
    return exports.reflection.isInstance(item, exports.ExpressionLiteral);
}
exports.isExpressionLiteral = isExpressionLiteral;
exports.FreeVariableLiteral = 'FreeVariableLiteral';
function isFreeVariableLiteral(item) {
    return exports.reflection.isInstance(item, exports.FreeVariableLiteral);
}
exports.isFreeVariableLiteral = isFreeVariableLiteral;
exports.Referencable = 'Referencable';
function isReferencable(item) {
    return exports.reflection.isInstance(item, exports.Referencable);
}
exports.isReferencable = isReferencable;
exports.ReferenceableBlocktypeDefinition = 'ReferenceableBlocktypeDefinition';
function isReferenceableBlocktypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.ReferenceableBlocktypeDefinition);
}
exports.isReferenceableBlocktypeDefinition = isReferenceableBlocktypeDefinition;
exports.ValueLiteral = 'ValueLiteral';
function isValueLiteral(item) {
    return exports.reflection.isInstance(item, exports.ValueLiteral);
}
exports.isValueLiteral = isValueLiteral;
exports.BinaryExpression = 'BinaryExpression';
function isBinaryExpression(item) {
    return exports.reflection.isInstance(item, exports.BinaryExpression);
}
exports.isBinaryExpression = isBinaryExpression;
exports.BlockDefinition = 'BlockDefinition';
function isBlockDefinition(item) {
    return exports.reflection.isInstance(item, exports.BlockDefinition);
}
exports.isBlockDefinition = isBlockDefinition;
exports.BlocktypeInput = 'BlocktypeInput';
function isBlocktypeInput(item) {
    return exports.reflection.isInstance(item, exports.BlocktypeInput);
}
exports.isBlocktypeInput = isBlocktypeInput;
exports.BlocktypeOutput = 'BlocktypeOutput';
function isBlocktypeOutput(item) {
    return exports.reflection.isInstance(item, exports.BlocktypeOutput);
}
exports.isBlocktypeOutput = isBlocktypeOutput;
exports.BlocktypePipeline = 'BlocktypePipeline';
function isBlocktypePipeline(item) {
    return exports.reflection.isInstance(item, exports.BlocktypePipeline);
}
exports.isBlocktypePipeline = isBlocktypePipeline;
exports.BlocktypeProperty = 'BlocktypeProperty';
function isBlocktypeProperty(item) {
    return exports.reflection.isInstance(item, exports.BlocktypeProperty);
}
exports.isBlocktypeProperty = isBlocktypeProperty;
exports.BooleanLiteral = 'BooleanLiteral';
function isBooleanLiteral(item) {
    return exports.reflection.isInstance(item, exports.BooleanLiteral);
}
exports.isBooleanLiteral = isBooleanLiteral;
exports.BuiltinBlocktypeDefinition = 'BuiltinBlocktypeDefinition';
function isBuiltinBlocktypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.BuiltinBlocktypeDefinition);
}
exports.isBuiltinBlocktypeDefinition = isBuiltinBlocktypeDefinition;
exports.BuiltinConstrainttypeDefinition = 'BuiltinConstrainttypeDefinition';
function isBuiltinConstrainttypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.BuiltinConstrainttypeDefinition);
}
exports.isBuiltinConstrainttypeDefinition = isBuiltinConstrainttypeDefinition;
exports.CellLiteral = 'CellLiteral';
function isCellLiteral(item) {
    return exports.reflection.isInstance(item, exports.CellLiteral);
}
exports.isCellLiteral = isCellLiteral;
exports.CellReference = 'CellReference';
function isCellReference(item) {
    return exports.reflection.isInstance(item, exports.CellReference);
}
exports.isCellReference = isCellReference;
exports.CollectionLiteral = 'CollectionLiteral';
function isCollectionLiteral(item) {
    return exports.reflection.isInstance(item, exports.CollectionLiteral);
}
exports.isCollectionLiteral = isCollectionLiteral;
exports.ColumnId = 'ColumnId';
function isColumnId(item) {
    return exports.reflection.isInstance(item, exports.ColumnId);
}
exports.isColumnId = isColumnId;
exports.ColumnLiteral = 'ColumnLiteral';
function isColumnLiteral(item) {
    return exports.reflection.isInstance(item, exports.ColumnLiteral);
}
exports.isColumnLiteral = isColumnLiteral;
exports.CompositeBlocktypeDefinition = 'CompositeBlocktypeDefinition';
function isCompositeBlocktypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.CompositeBlocktypeDefinition);
}
exports.isCompositeBlocktypeDefinition = isCompositeBlocktypeDefinition;
exports.ConstrainttypeProperty = 'ConstrainttypeProperty';
function isConstrainttypeProperty(item) {
    return exports.reflection.isInstance(item, exports.ConstrainttypeProperty);
}
exports.isConstrainttypeProperty = isConstrainttypeProperty;
exports.ExpressionConstraintDefinition = 'ExpressionConstraintDefinition';
function isExpressionConstraintDefinition(item) {
    return exports.reflection.isInstance(item, exports.ExpressionConstraintDefinition);
}
exports.isExpressionConstraintDefinition = isExpressionConstraintDefinition;
exports.IotypeDefinition = 'IotypeDefinition';
function isIotypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.IotypeDefinition);
}
exports.isIotypeDefinition = isIotypeDefinition;
exports.JayveeModel = 'JayveeModel';
function isJayveeModel(item) {
    return exports.reflection.isInstance(item, exports.JayveeModel);
}
exports.isJayveeModel = isJayveeModel;
exports.NumericLiteral = 'NumericLiteral';
function isNumericLiteral(item) {
    return exports.reflection.isInstance(item, exports.NumericLiteral);
}
exports.isNumericLiteral = isNumericLiteral;
exports.PipeDefinition = 'PipeDefinition';
function isPipeDefinition(item) {
    return exports.reflection.isInstance(item, exports.PipeDefinition);
}
exports.isPipeDefinition = isPipeDefinition;
exports.PipelineDefinition = 'PipelineDefinition';
function isPipelineDefinition(item) {
    return exports.reflection.isInstance(item, exports.PipelineDefinition);
}
exports.isPipelineDefinition = isPipelineDefinition;
exports.PropertyAssignment = 'PropertyAssignment';
function isPropertyAssignment(item) {
    return exports.reflection.isInstance(item, exports.PropertyAssignment);
}
exports.isPropertyAssignment = isPropertyAssignment;
exports.PropertyBody = 'PropertyBody';
function isPropertyBody(item) {
    return exports.reflection.isInstance(item, exports.PropertyBody);
}
exports.isPropertyBody = isPropertyBody;
exports.RangeLiteral = 'RangeLiteral';
function isRangeLiteral(item) {
    return exports.reflection.isInstance(item, exports.RangeLiteral);
}
exports.isRangeLiteral = isRangeLiteral;
exports.ReferenceLiteral = 'ReferenceLiteral';
function isReferenceLiteral(item) {
    return exports.reflection.isInstance(item, exports.ReferenceLiteral);
}
exports.isReferenceLiteral = isReferenceLiteral;
exports.RegexLiteral = 'RegexLiteral';
function isRegexLiteral(item) {
    return exports.reflection.isInstance(item, exports.RegexLiteral);
}
exports.isRegexLiteral = isRegexLiteral;
exports.RowId = 'RowId';
function isRowId(item) {
    return exports.reflection.isInstance(item, exports.RowId);
}
exports.isRowId = isRowId;
exports.RowLiteral = 'RowLiteral';
function isRowLiteral(item) {
    return exports.reflection.isInstance(item, exports.RowLiteral);
}
exports.isRowLiteral = isRowLiteral;
exports.RuntimeParameterLiteral = 'RuntimeParameterLiteral';
function isRuntimeParameterLiteral(item) {
    return exports.reflection.isInstance(item, exports.RuntimeParameterLiteral);
}
exports.isRuntimeParameterLiteral = isRuntimeParameterLiteral;
exports.TextLiteral = 'TextLiteral';
function isTextLiteral(item) {
    return exports.reflection.isInstance(item, exports.TextLiteral);
}
exports.isTextLiteral = isTextLiteral;
exports.TransformBody = 'TransformBody';
function isTransformBody(item) {
    return exports.reflection.isInstance(item, exports.TransformBody);
}
exports.isTransformBody = isTransformBody;
exports.TransformDefinition = 'TransformDefinition';
function isTransformDefinition(item) {
    return exports.reflection.isInstance(item, exports.TransformDefinition);
}
exports.isTransformDefinition = isTransformDefinition;
exports.TransformOutputAssignment = 'TransformOutputAssignment';
function isTransformOutputAssignment(item) {
    return exports.reflection.isInstance(item, exports.TransformOutputAssignment);
}
exports.isTransformOutputAssignment = isTransformOutputAssignment;
exports.TransformPortDefinition = 'TransformPortDefinition';
function isTransformPortDefinition(item) {
    return exports.reflection.isInstance(item, exports.TransformPortDefinition);
}
exports.isTransformPortDefinition = isTransformPortDefinition;
exports.TypedConstraintDefinition = 'TypedConstraintDefinition';
function isTypedConstraintDefinition(item) {
    return exports.reflection.isInstance(item, exports.TypedConstraintDefinition);
}
exports.isTypedConstraintDefinition = isTypedConstraintDefinition;
exports.UnaryExpression = 'UnaryExpression';
function isUnaryExpression(item) {
    return exports.reflection.isInstance(item, exports.UnaryExpression);
}
exports.isUnaryExpression = isUnaryExpression;
exports.ValueKeywordLiteral = 'ValueKeywordLiteral';
function isValueKeywordLiteral(item) {
    return exports.reflection.isInstance(item, exports.ValueKeywordLiteral);
}
exports.isValueKeywordLiteral = isValueKeywordLiteral;
exports.ValuetypeAssignment = 'ValuetypeAssignment';
function isValuetypeAssignment(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeAssignment);
}
exports.isValuetypeAssignment = isValuetypeAssignment;
exports.ValuetypeAssignmentLiteral = 'ValuetypeAssignmentLiteral';
function isValuetypeAssignmentLiteral(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeAssignmentLiteral);
}
exports.isValuetypeAssignmentLiteral = isValuetypeAssignmentLiteral;
exports.ValuetypeDefinition = 'ValuetypeDefinition';
function isValuetypeDefinition(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeDefinition);
}
exports.isValuetypeDefinition = isValuetypeDefinition;
exports.ValuetypeGenericDefinition = 'ValuetypeGenericDefinition';
function isValuetypeGenericDefinition(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeGenericDefinition);
}
exports.isValuetypeGenericDefinition = isValuetypeGenericDefinition;
exports.ValuetypeGenericsDefinition = 'ValuetypeGenericsDefinition';
function isValuetypeGenericsDefinition(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeGenericsDefinition);
}
exports.isValuetypeGenericsDefinition = isValuetypeGenericsDefinition;
exports.ValuetypeReference = 'ValuetypeReference';
function isValuetypeReference(item) {
    return exports.reflection.isInstance(item, exports.ValuetypeReference);
}
exports.isValuetypeReference = isValuetypeReference;
class JayveeAstReflection extends langium_1.AbstractAstReflection {
    getAllTypes() {
        return ['BinaryExpression', 'BlockDefinition', 'BlocktypeInput', 'BlocktypeOutput', 'BlocktypePipeline', 'BlocktypeProperty', 'BooleanLiteral', 'BuiltinBlocktypeDefinition', 'BuiltinConstrainttypeDefinition', 'CellLiteral', 'CellRangeLiteral', 'CellReference', 'CollectionLiteral', 'ColumnId', 'ColumnLiteral', 'CompositeBlocktypeDefinition', 'ConstraintDefinition', 'ConstrainttypeProperty', 'Expression', 'ExpressionConstraintDefinition', 'ExpressionLiteral', 'FreeVariableLiteral', 'IotypeDefinition', 'JayveeModel', 'NumericLiteral', 'PipeDefinition', 'PipelineDefinition', 'PropertyAssignment', 'PropertyBody', 'RangeLiteral', 'Referencable', 'ReferenceLiteral', 'ReferenceableBlocktypeDefinition', 'RegexLiteral', 'RowId', 'RowLiteral', 'RuntimeParameterLiteral', 'TextLiteral', 'TransformBody', 'TransformDefinition', 'TransformOutputAssignment', 'TransformPortDefinition', 'TypedConstraintDefinition', 'UnaryExpression', 'ValueKeywordLiteral', 'ValueLiteral', 'ValuetypeAssignment', 'ValuetypeAssignmentLiteral', 'ValuetypeDefinition', 'ValuetypeGenericDefinition', 'ValuetypeGenericsDefinition', 'ValuetypeReference'];
    }
    computeIsSubtype(subtype, supertype) {
        switch (subtype) {
            case exports.BinaryExpression:
            case exports.ExpressionLiteral:
            case exports.UnaryExpression: {
                return this.isSubtype(exports.Expression, supertype);
            }
            case exports.BlocktypeProperty:
            case exports.ConstraintDefinition:
            case exports.TransformDefinition:
            case exports.TransformPortDefinition: {
                return this.isSubtype(exports.Referencable, supertype);
            }
            case exports.BooleanLiteral:
            case exports.CellRangeLiteral:
            case exports.CollectionLiteral:
            case exports.NumericLiteral:
            case exports.RegexLiteral:
            case exports.TextLiteral:
            case exports.ValuetypeAssignmentLiteral: {
                return this.isSubtype(exports.ValueLiteral, supertype);
            }
            case exports.BuiltinBlocktypeDefinition:
            case exports.CompositeBlocktypeDefinition: {
                return this.isSubtype(exports.ReferenceableBlocktypeDefinition, supertype);
            }
            case exports.CellLiteral:
            case exports.ColumnLiteral:
            case exports.RangeLiteral:
            case exports.RowLiteral: {
                return this.isSubtype(exports.CellRangeLiteral, supertype);
            }
            case exports.ExpressionConstraintDefinition:
            case exports.TypedConstraintDefinition: {
                return this.isSubtype(exports.ConstraintDefinition, supertype);
            }
            case exports.FreeVariableLiteral:
            case exports.ValueLiteral: {
                return this.isSubtype(exports.ExpressionLiteral, supertype);
            }
            case exports.ReferenceLiteral:
            case exports.ValueKeywordLiteral: {
                return this.isSubtype(exports.FreeVariableLiteral, supertype);
            }
            default: {
                return false;
            }
        }
    }
    getReferenceType(refInfo) {
        const referenceId = `${refInfo.container.$type}:${refInfo.property}`;
        switch (referenceId) {
            case 'BlockDefinition:type': {
                return exports.ReferenceableBlocktypeDefinition;
            }
            case 'BlocktypeInput:iotype':
            case 'BlocktypeOutput:iotype': {
                return exports.IotypeDefinition;
            }
            case 'BlocktypePipeline:blocks':
            case 'PipeDefinition:blocks': {
                return exports.BlockDefinition;
            }
            case 'BlocktypePipeline:input': {
                return exports.BlocktypeInput;
            }
            case 'BlocktypePipeline:output': {
                return exports.BlocktypeOutput;
            }
            case 'ReferenceLiteral:value': {
                return exports.Referencable;
            }
            case 'TransformOutputAssignment:outPortName': {
                return exports.TransformPortDefinition;
            }
            case 'TypedConstraintDefinition:type': {
                return exports.BuiltinConstrainttypeDefinition;
            }
            case 'ValuetypeReference:genericRefs':
            case 'ValuetypeReference:reference': {
                return exports.ValuetypeDefinition;
            }
            default: {
                throw new Error(`${referenceId} is not a valid reference id.`);
            }
        }
    }
    getTypeMetaData(type) {
        switch (type) {
            case 'BlocktypePipeline': {
                return {
                    name: 'BlocktypePipeline',
                    mandatory: [
                        { name: 'blocks', type: 'array' }
                    ]
                };
            }
            case 'BooleanLiteral': {
                return {
                    name: 'BooleanLiteral',
                    mandatory: [
                        { name: 'value', type: 'boolean' }
                    ]
                };
            }
            case 'BuiltinBlocktypeDefinition': {
                return {
                    name: 'BuiltinBlocktypeDefinition',
                    mandatory: [
                        { name: 'inputs', type: 'array' },
                        { name: 'outputs', type: 'array' },
                        { name: 'properties', type: 'array' }
                    ]
                };
            }
            case 'BuiltinConstrainttypeDefinition': {
                return {
                    name: 'BuiltinConstrainttypeDefinition',
                    mandatory: [
                        { name: 'properties', type: 'array' }
                    ]
                };
            }
            case 'CollectionLiteral': {
                return {
                    name: 'CollectionLiteral',
                    mandatory: [
                        { name: 'values', type: 'array' }
                    ]
                };
            }
            case 'CompositeBlocktypeDefinition': {
                return {
                    name: 'CompositeBlocktypeDefinition',
                    mandatory: [
                        { name: 'blocks', type: 'array' },
                        { name: 'inputs', type: 'array' },
                        { name: 'outputs', type: 'array' },
                        { name: 'pipes', type: 'array' },
                        { name: 'properties', type: 'array' },
                        { name: 'transforms', type: 'array' }
                    ]
                };
            }
            case 'JayveeModel': {
                return {
                    name: 'JayveeModel',
                    mandatory: [
                        { name: 'blocktypes', type: 'array' },
                        { name: 'constraints', type: 'array' },
                        { name: 'constrainttypes', type: 'array' },
                        { name: 'iotypes', type: 'array' },
                        { name: 'pipelines', type: 'array' },
                        { name: 'transforms', type: 'array' },
                        { name: 'valuetypes', type: 'array' }
                    ]
                };
            }
            case 'PipeDefinition': {
                return {
                    name: 'PipeDefinition',
                    mandatory: [
                        { name: 'blocks', type: 'array' }
                    ]
                };
            }
            case 'PipelineDefinition': {
                return {
                    name: 'PipelineDefinition',
                    mandatory: [
                        { name: 'blocks', type: 'array' },
                        { name: 'constraints', type: 'array' },
                        { name: 'pipes', type: 'array' },
                        { name: 'transforms', type: 'array' },
                        { name: 'valuetypes', type: 'array' }
                    ]
                };
            }
            case 'PropertyBody': {
                return {
                    name: 'PropertyBody',
                    mandatory: [
                        { name: 'properties', type: 'array' }
                    ]
                };
            }
            case 'TransformBody': {
                return {
                    name: 'TransformBody',
                    mandatory: [
                        { name: 'outputAssignments', type: 'array' },
                        { name: 'ports', type: 'array' }
                    ]
                };
            }
            case 'ValueKeywordLiteral': {
                return {
                    name: 'ValueKeywordLiteral',
                    mandatory: [
                        { name: 'lengthAccess', type: 'boolean' }
                    ]
                };
            }
            case 'ValuetypeDefinition': {
                return {
                    name: 'ValuetypeDefinition',
                    mandatory: [
                        { name: 'isBuiltin', type: 'boolean' }
                    ]
                };
            }
            case 'ValuetypeGenericsDefinition': {
                return {
                    name: 'ValuetypeGenericsDefinition',
                    mandatory: [
                        { name: 'generics', type: 'array' }
                    ]
                };
            }
            case 'ValuetypeReference': {
                return {
                    name: 'ValuetypeReference',
                    mandatory: [
                        { name: 'genericRefs', type: 'array' }
                    ]
                };
            }
            default: {
                return {
                    name: type,
                    mandatory: []
                };
            }
        }
    }
}
exports.JayveeAstReflection = JayveeAstReflection;
exports.reflection = new JayveeAstReflection();


/***/ }),

/***/ "./libs/language-server/src/lib/ast/generated/grammar.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/******************************************************************************
 * This file was generated by langium-cli 1.2.0.
 * DO NOT EDIT MANUALLY!
 ******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeGrammar = void 0;
const langium_1 = __webpack_require__("langium");
let loadedJayveeGrammar;
const JayveeGrammar = () => loadedJayveeGrammar !== null && loadedJayveeGrammar !== void 0 ? loadedJayveeGrammar : (loadedJayveeGrammar = (0, langium_1.loadGrammarFromJson)(`{
  "$type": "Grammar",
  "isDeclared": true,
  "name": "Jayvee",
  "imports": [],
  "rules": [
    {
      "$type": "ParserRule",
      "name": "JayveeModel",
      "entry": true,
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "pipelines",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@70"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "valuetypes",
            "operator": "+=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@58"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@57"
                  },
                  "arguments": []
                }
              ]
            }
          },
          {
            "$type": "Assignment",
            "feature": "constraints",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@49"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "transforms",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@64"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "blocktypes",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@1"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "constrainttypes",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@52"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "iotypes",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@68"
              },
              "arguments": []
            }
          }
        ],
        "cardinality": "*"
      },
      "definesHiddenTokens": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ReferenceableBlocktypeDefinition",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@2"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@3"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BuiltinBlocktypeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "builtin"
          },
          {
            "$type": "Keyword",
            "value": "blocktype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Alternatives",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "inputs",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@4"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "outputs",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@5"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "properties",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@6"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CompositeBlocktypeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "composite"
          },
          {
            "$type": "Keyword",
            "value": "blocktype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Alternatives",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "inputs",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@4"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "outputs",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@5"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "properties",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@6"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "blocks",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@69"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "pipes",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@7"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "transforms",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@64"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlocktypeInput",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "input"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "iotype",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@68"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlocktypeOutput",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "output"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "iotype",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@68"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlocktypeProperty",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "property"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "valueType",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": ":"
              },
              {
                "$type": "Assignment",
                "feature": "defaultValue",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@8"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlocktypePipeline",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "input",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@4"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": "->"
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "blocks",
                "operator": "+=",
                "terminal": {
                  "$type": "CrossReference",
                  "type": {
                    "$ref": "#/rules@69"
                  },
                  "deprecatedSyntax": false
                }
              },
              {
                "$type": "Keyword",
                "value": "->"
              }
            ],
            "cardinality": "+"
          },
          {
            "$type": "Assignment",
            "feature": "output",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@5"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "Expression",
      "definition": {
        "$type": "RuleCall",
        "rule": {
          "$ref": "#/rules@9"
        },
        "arguments": []
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "OrExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@10"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Keyword",
                  "value": "or"
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@10"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "AndExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@11"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Keyword",
                  "value": "and"
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@11"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "XorExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@12"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Keyword",
                  "value": "xor"
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@12"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "EqualityExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@13"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Alternatives",
                  "elements": [
                    {
                      "$type": "Keyword",
                      "value": "=="
                    },
                    {
                      "$type": "Keyword",
                      "value": "!="
                    }
                  ]
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@13"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "MatchesExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@14"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Keyword",
                  "value": "matches"
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@14"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "InExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@15"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Keyword",
                  "value": "in"
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@15"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RelationalExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@16"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Alternatives",
                  "elements": [
                    {
                      "$type": "Keyword",
                      "value": "<"
                    },
                    {
                      "$type": "Keyword",
                      "value": "<="
                    },
                    {
                      "$type": "Keyword",
                      "value": ">"
                    },
                    {
                      "$type": "Keyword",
                      "value": ">="
                    }
                  ]
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@16"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "AdditiveExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@17"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Alternatives",
                  "elements": [
                    {
                      "$type": "Keyword",
                      "value": "+"
                    },
                    {
                      "$type": "Keyword",
                      "value": "-"
                    }
                  ]
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@17"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "MultiplicativeExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@18"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Alternatives",
                  "elements": [
                    {
                      "$type": "Keyword",
                      "value": "*"
                    },
                    {
                      "$type": "Keyword",
                      "value": "/"
                    },
                    {
                      "$type": "Keyword",
                      "value": "%"
                    }
                  ]
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@18"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ExponentialExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@19"
            },
            "arguments": []
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Action",
                "inferredType": {
                  "$type": "InferredType",
                  "name": "BinaryExpression"
                },
                "feature": "left",
                "operator": "="
              },
              {
                "$type": "Assignment",
                "feature": "operator",
                "operator": "=",
                "terminal": {
                  "$type": "Alternatives",
                  "elements": [
                    {
                      "$type": "Keyword",
                      "value": "pow"
                    },
                    {
                      "$type": "Keyword",
                      "value": "root"
                    }
                  ]
                }
              },
              {
                "$type": "Assignment",
                "feature": "right",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@19"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PrimaryExpression",
      "inferredType": {
        "$type": "InferredType",
        "name": "Expression"
      },
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": "("
              },
              {
                "$type": "RuleCall",
                "rule": {
                  "$ref": "#/rules@8"
                },
                "arguments": []
              },
              {
                "$type": "Keyword",
                "value": ")"
              }
            ]
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@20"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@21"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "UnaryExpression",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "operator",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "Keyword",
                  "value": "not"
                },
                {
                  "$type": "Keyword",
                  "value": "+"
                },
                {
                  "$type": "Keyword",
                  "value": "-"
                },
                {
                  "$type": "Keyword",
                  "value": "sqrt"
                },
                {
                  "$type": "Keyword",
                  "value": "floor"
                },
                {
                  "$type": "Keyword",
                  "value": "ceil"
                },
                {
                  "$type": "Keyword",
                  "value": "round"
                }
              ]
            }
          },
          {
            "$type": "Assignment",
            "feature": "expression",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@19"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ExpressionLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@22"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@28"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValueLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@23"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@24"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@25"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@26"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@32"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@59"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@27"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TextLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@44"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "NumericLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "value",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@43"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "value",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@42"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BooleanLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "value",
            "operator": "?=",
            "terminal": {
              "$type": "Keyword",
              "value": "true"
            }
          },
          {
            "$type": "Keyword",
            "value": "false"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RegexLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@47"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CollectionLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "["
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "values",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@8"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Group",
                "elements": [
                  {
                    "$type": "Keyword",
                    "value": ","
                  },
                  {
                    "$type": "Assignment",
                    "feature": "values",
                    "operator": "+=",
                    "terminal": {
                      "$type": "RuleCall",
                      "rule": {
                        "$ref": "#/rules@8"
                      },
                      "arguments": []
                    }
                  }
                ],
                "cardinality": "*"
              }
            ],
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": ",",
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": "]"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "FreeVariableLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@29"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@30"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValueKeywordLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "value"
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": "."
              },
              {
                "$type": "Assignment",
                "feature": "lengthAccess",
                "operator": "?=",
                "terminal": {
                  "$type": "Keyword",
                  "value": "length"
                }
              }
            ],
            "cardinality": "?"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ReferenceLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "CrossReference",
          "type": {
            "$ref": "#/rules@31"
          },
          "deprecatedSyntax": false
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "Referencable",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@49"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@6"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@64"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@66"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CellRangeLiteral",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@33"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@34"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@35"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@36"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RangeLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "range"
          },
          {
            "$type": "Assignment",
            "feature": "cellFrom",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@40"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@37"
                  },
                  "arguments": []
                }
              ]
            }
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "cellTo",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@40"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@37"
                  },
                  "arguments": []
                }
              ]
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ColumnLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "column"
          },
          {
            "$type": "Assignment",
            "feature": "columnId",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@38"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RowLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "row"
          },
          {
            "$type": "Assignment",
            "feature": "rowId",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@39"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CellLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "cell"
          },
          {
            "$type": "Assignment",
            "feature": "cellId",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@40"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@37"
                  },
                  "arguments": []
                }
              ]
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CellReference",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "columnId",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@38"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "rowId",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@39"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ColumnId",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "Alternatives",
          "elements": [
            {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            },
            {
              "$type": "Keyword",
              "value": "*"
            }
          ]
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RowId",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "Alternatives",
          "elements": [
            {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@43"
              },
              "arguments": []
            },
            {
              "$type": "Keyword",
              "value": "*"
            }
          ]
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "TerminalRule",
      "name": "CELL_REFERENCE",
      "definition": {
        "$type": "TerminalGroup",
        "elements": [
          {
            "$type": "RegexToken",
            "regex": "[A-Z]+"
          },
          {
            "$type": "TerminalRuleCall",
            "rule": {
              "$ref": "#/rules@43"
            }
          }
        ]
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "ID",
      "definition": {
        "$type": "RegexToken",
        "regex": "[_a-zA-Z][\\\\w_]*"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "DECIMAL",
      "type": {
        "$type": "ReturnType",
        "name": "number"
      },
      "definition": {
        "$type": "TerminalGroup",
        "elements": [
          {
            "$type": "TerminalRuleCall",
            "rule": {
              "$ref": "#/rules@43"
            }
          },
          {
            "$type": "CharacterRange",
            "left": {
              "$type": "Keyword",
              "value": "."
            }
          },
          {
            "$type": "TerminalRuleCall",
            "rule": {
              "$ref": "#/rules@43"
            }
          }
        ]
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "INTEGER",
      "type": {
        "$type": "ReturnType",
        "name": "number"
      },
      "definition": {
        "$type": "RegexToken",
        "regex": "[0-9]+"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "name": "STRING",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\"[^\\"]*\\"|'[^']*'"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "hidden": true,
      "name": "SL_COMMENT",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\/\\\\/[^\\\\n\\\\r]*"
      },
      "fragment": false
    },
    {
      "$type": "TerminalRule",
      "hidden": true,
      "name": "ML_COMMENT",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\/\\\\*[\\\\s\\\\S]*?\\\\*\\\\/"
      },
      "fragment": false
    },
    {
      "$type": "TerminalRule",
      "name": "REGEX",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\/.+\\\\/"
      },
      "fragment": false,
      "hidden": false
    },
    {
      "$type": "TerminalRule",
      "hidden": true,
      "name": "WHITESPACE",
      "definition": {
        "$type": "RegexToken",
        "regex": "\\\\s+"
      },
      "fragment": false
    },
    {
      "$type": "ParserRule",
      "name": "ConstraintDefinition",
      "definition": {
        "$type": "Alternatives",
        "elements": [
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@50"
            },
            "arguments": []
          },
          {
            "$type": "RuleCall",
            "rule": {
              "$ref": "#/rules@51"
            },
            "arguments": []
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TypedConstraintDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "constraint"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@52"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Assignment",
            "feature": "body",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@54"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ExpressionConstraintDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "constraint"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "on"
          },
          {
            "$type": "Assignment",
            "feature": "valuetype",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "expression",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@8"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BuiltinConstrainttypeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "builtin"
          },
          {
            "$type": "Keyword",
            "value": "constrainttype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "on"
          },
          {
            "$type": "Assignment",
            "feature": "valuetype",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Assignment",
            "feature": "properties",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@53"
              },
              "arguments": []
            },
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ConstrainttypeProperty",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "property"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "valueType",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": ":"
              },
              {
                "$type": "Assignment",
                "feature": "defaultValue",
                "operator": "=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@8"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PropertyBody",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Assignment",
            "feature": "properties",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@55"
              },
              "arguments": []
            },
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PropertyAssignment",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "value",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@8"
                  },
                  "arguments": []
                },
                {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@56"
                  },
                  "arguments": []
                }
              ]
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "RuntimeParameterLiteral",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "requires"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BuiltinValuetypeDefinition",
      "inferredType": {
        "$type": "InferredType",
        "name": "ValuetypeDefinition"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "isBuiltin",
            "operator": "?=",
            "terminal": {
              "$type": "Keyword",
              "value": "builtin"
            }
          },
          {
            "$type": "Keyword",
            "value": "valuetype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "genericDefinition",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@62"
              },
              "arguments": []
            },
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "CustomValuetypeDefinition",
      "inferredType": {
        "$type": "InferredType",
        "name": "ValuetypeDefinition"
      },
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "valuetype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "genericDefinition",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@62"
              },
              "arguments": []
            },
            "cardinality": "?"
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Keyword",
            "value": "constraints"
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "constraints",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@27"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeAssignmentLiteral",
      "definition": {
        "$type": "Assignment",
        "feature": "value",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@60"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeAssignment",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@44"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeReference",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "reference",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@57/inferredType"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": "<"
              },
              {
                "$type": "Assignment",
                "feature": "genericRefs",
                "operator": "+=",
                "terminal": {
                  "$type": "CrossReference",
                  "type": {
                    "$ref": "#/rules@57/inferredType"
                  },
                  "deprecatedSyntax": false
                }
              },
              {
                "$type": "Group",
                "elements": [
                  {
                    "$type": "Keyword",
                    "value": ","
                  },
                  {
                    "$type": "Assignment",
                    "feature": "genericRefs",
                    "operator": "+=",
                    "terminal": {
                      "$type": "CrossReference",
                      "type": {
                        "$ref": "#/rules@57/inferredType"
                      },
                      "deprecatedSyntax": false
                    }
                  }
                ],
                "cardinality": "*"
              },
              {
                "$type": "Keyword",
                "value": ">"
              }
            ],
            "cardinality": "?"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeGenericsDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "<"
          },
          {
            "$type": "Assignment",
            "feature": "generics",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@63"
              },
              "arguments": []
            }
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": ","
              },
              {
                "$type": "Assignment",
                "feature": "generics",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@63"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": ">"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "ValuetypeGenericDefinition",
      "definition": {
        "$type": "Assignment",
        "feature": "name",
        "operator": "=",
        "terminal": {
          "$type": "RuleCall",
          "rule": {
            "$ref": "#/rules@41"
          },
          "arguments": []
        }
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TransformDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "transform"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Assignment",
            "feature": "body",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@65"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TransformBody",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Assignment",
            "feature": "ports",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@66"
              },
              "arguments": []
            },
            "cardinality": "*"
          },
          {
            "$type": "Assignment",
            "feature": "outputAssignments",
            "operator": "+=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@67"
              },
              "arguments": []
            },
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TransformPortDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "kind",
            "operator": "=",
            "terminal": {
              "$type": "Alternatives",
              "elements": [
                {
                  "$type": "Keyword",
                  "value": "from"
                },
                {
                  "$type": "Keyword",
                  "value": "to"
                }
              ]
            }
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "valueType",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@61"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "TransformOutputAssignment",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "outPortName",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@66"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Keyword",
            "value": ":"
          },
          {
            "$type": "Assignment",
            "feature": "expression",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@8"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "IotypeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "builtin"
          },
          {
            "$type": "Keyword",
            "value": "iotype"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "BlockDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "block"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "oftype"
          },
          {
            "$type": "Assignment",
            "feature": "type",
            "operator": "=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@1"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Assignment",
            "feature": "body",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@54"
              },
              "arguments": []
            }
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PipelineDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Keyword",
            "value": "pipeline"
          },
          {
            "$type": "Assignment",
            "feature": "name",
            "operator": "=",
            "terminal": {
              "$type": "RuleCall",
              "rule": {
                "$ref": "#/rules@41"
              },
              "arguments": []
            }
          },
          {
            "$type": "Keyword",
            "value": "{"
          },
          {
            "$type": "Alternatives",
            "elements": [
              {
                "$type": "Assignment",
                "feature": "blocks",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@69"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "pipes",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@71"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "valuetypes",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@58"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "constraints",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@49"
                  },
                  "arguments": []
                }
              },
              {
                "$type": "Assignment",
                "feature": "transforms",
                "operator": "+=",
                "terminal": {
                  "$type": "RuleCall",
                  "rule": {
                    "$ref": "#/rules@64"
                  },
                  "arguments": []
                }
              }
            ],
            "cardinality": "*"
          },
          {
            "$type": "Keyword",
            "value": "}"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    },
    {
      "$type": "ParserRule",
      "name": "PipeDefinition",
      "definition": {
        "$type": "Group",
        "elements": [
          {
            "$type": "Assignment",
            "feature": "blocks",
            "operator": "+=",
            "terminal": {
              "$type": "CrossReference",
              "type": {
                "$ref": "#/rules@69"
              },
              "deprecatedSyntax": false
            }
          },
          {
            "$type": "Group",
            "elements": [
              {
                "$type": "Keyword",
                "value": "->"
              },
              {
                "$type": "Assignment",
                "feature": "blocks",
                "operator": "+=",
                "terminal": {
                  "$type": "CrossReference",
                  "type": {
                    "$ref": "#/rules@69"
                  },
                  "deprecatedSyntax": false
                }
              }
            ],
            "cardinality": "+"
          },
          {
            "$type": "Keyword",
            "value": ";"
          }
        ]
      },
      "definesHiddenTokens": false,
      "entry": false,
      "fragment": false,
      "hiddenTokens": [],
      "parameters": [],
      "wildcard": false
    }
  ],
  "definesHiddenTokens": false,
  "hiddenTokens": [],
  "interfaces": [],
  "types": [],
  "usedGrammars": []
}`));
exports.JayveeGrammar = JayveeGrammar;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/generated/module.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/******************************************************************************
 * This file was generated by langium-cli 1.2.0.
 * DO NOT EDIT MANUALLY!
 ******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeGeneratedModule = exports.JayveeGeneratedSharedModule = exports.JayveeLanguageMetaData = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const grammar_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/grammar.ts");
exports.JayveeLanguageMetaData = {
    languageId: 'jayvee',
    fileExtensions: ['.jv'],
    caseInsensitive: false
};
exports.JayveeGeneratedSharedModule = {
    AstReflection: () => new ast_1.JayveeAstReflection()
};
exports.JayveeGeneratedModule = {
    Grammar: () => (0, grammar_1.JayveeGrammar)(),
    LanguageMetaData: () => exports.JayveeLanguageMetaData,
    parser: {}
};


/***/ }),

/***/ "./libs/language-server/src/lib/ast/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/expressions/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/generated/grammar.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/generated/module.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/model-util.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/io-type.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/io-type.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getIOType = exports.IOType = void 0;
const assert_1 = __webpack_require__("assert");
var IOType;
(function (IOType) {
    IOType["NONE"] = "None";
    IOType["FILE"] = "File";
    IOType["TEXT_FILE"] = "TextFile";
    IOType["FILE_SYSTEM"] = "FileSystem";
    IOType["SHEET"] = "Sheet";
    IOType["TABLE"] = "Table";
    IOType["WORKBOOK"] = "Workbook";
})(IOType = exports.IOType || (exports.IOType = {}));
function getIOType(blockIO) {
    var _a;
    const ioTypeName = (_a = blockIO.iotype.ref) === null || _a === void 0 ? void 0 : _a.name;
    (0, assert_1.strict)(ioTypeName !== undefined, `Unknown IOType name for block input/output ${blockIO.name}.`);
    (0, assert_1.strict)(Object.values(IOType).some((type) => type === ioTypeName), `IOType ${ioTypeName} does not exist.`);
    return ioTypeName;
}
exports.getIOType = getIOType;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/model-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getAllBuiltinConstraintTypes = exports.getAllBuiltinBlocktypes = exports.getNextAstNodeContainer = exports.getBlocksInTopologicalSorting = exports.collectAllPipes = exports.collectIngoingPipes = exports.collectOutgoingPipes = exports.collectParents = exports.collectChildren = exports.collectStartingBlocks = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
// eslint-disable-next-line import/no-cycle
const wrappers_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/index.ts");
const pipe_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/pipe-wrapper.ts");
function collectStartingBlocks(container) {
    var _a;
    // For composite blocks the first blocks of all pipelines are starting blocks as they have inputs
    if ((0, ast_1.isCompositeBlocktypeDefinition)(container)) {
        const startingBlocks = container.pipes
            .map((pipe) => pipe.blocks[0])
            .map((blockRef) => {
            if ((blockRef === null || blockRef === void 0 ? void 0 : blockRef.ref) !== undefined &&
                wrappers_1.BlockTypeWrapper.canBeWrapped(blockRef.ref.type)) {
                return blockRef.ref;
            }
            return undefined;
        })
            .filter((x) => x !== undefined);
        return startingBlocks;
    }
    const result = [];
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const blocks = (_a = container === null || container === void 0 ? void 0 : container.blocks) !== null && _a !== void 0 ? _a : [];
    for (const block of blocks) {
        if (!wrappers_1.BlockTypeWrapper.canBeWrapped(block.type)) {
            continue;
        }
        const blockType = new wrappers_1.BlockTypeWrapper(block.type);
        if (!blockType.hasInput()) {
            result.push(block);
        }
    }
    return result;
}
exports.collectStartingBlocks = collectStartingBlocks;
function collectChildren(block) {
    const outgoingPipes = collectOutgoingPipes(block);
    return outgoingPipes.map((pipe) => pipe.to);
}
exports.collectChildren = collectChildren;
function collectParents(block) {
    const ingoingPipes = collectIngoingPipes(block);
    return ingoingPipes.map((pipe) => pipe.from);
}
exports.collectParents = collectParents;
function collectOutgoingPipes(block) {
    return collectPipes(block, 'outgoing');
}
exports.collectOutgoingPipes = collectOutgoingPipes;
function collectIngoingPipes(block) {
    return collectPipes(block, 'ingoing');
}
exports.collectIngoingPipes = collectIngoingPipes;
function collectPipes(block, kind) {
    const pipeline = block.$container;
    const allPipes = collectAllPipes(pipeline);
    return allPipes.filter((pipeWrapper) => {
        switch (kind) {
            case 'outgoing':
                return pipeWrapper.from === block;
            case 'ingoing':
                return pipeWrapper.to === block;
            case undefined:
                return false;
        }
        return (0, langium_1.assertUnreachable)(kind);
    });
}
function collectAllPipes(container) {
    const result = [];
    for (const pipe of container.pipes) {
        result.push(...(0, pipe_wrapper_1.createWrappersFromPipeChain)(pipe));
    }
    return result;
}
exports.collectAllPipes = collectAllPipes;
/**
 * Returns blocks in a pipeline in topological order, based on
 * Kahn's algorithm.
 *
 * Considers a pipeline as a directed, acyclical graph where
 * blocks are nodes and pipes are edges. A list in topological
 * order has the property that parent nodes are always listed
 * before their children.
 *
 * "[...] a list in topological order is such that no element
 * appears in it until after all elements appearing on all paths
 * leading to the particular element have been listed."
 *
 * Kahn, A. B. (1962). Topological sorting of large networks. Communications of the ACM, 5(11), 558–562.
 */
function getBlocksInTopologicalSorting(pipeline) {
    const sortedNodes = [];
    const currentNodes = [...collectStartingBlocks(pipeline)];
    let unvisitedEdges = [...collectAllPipes(pipeline)];
    while (currentNodes.length > 0) {
        const node = currentNodes.pop();
        (0, assert_1.strict)(node !== undefined);
        sortedNodes.push(node);
        for (const childNode of collectChildren(node)) {
            // Mark edges between parent and child as visited
            collectIngoingPipes(childNode)
                .filter((e) => e.from === node)
                .forEach((e) => {
                unvisitedEdges = unvisitedEdges.filter((edge) => !edge.equals(e));
            });
            // If all edges to the child have been visited
            const notRemovedEdges = collectIngoingPipes(childNode).filter((e) => unvisitedEdges.some((edge) => edge.equals(e)));
            if (notRemovedEdges.length === 0) {
                // Insert it into currentBlocks
                currentNodes.push(childNode);
            }
        }
    }
    // If the graph still contains unvisited edges it is not a DAG
    (0, assert_1.strict)(unvisitedEdges.length === 0, `The pipeline ${pipeline.name} is expected to have no cycles`);
    return sortedNodes;
}
exports.getBlocksInTopologicalSorting = getBlocksInTopologicalSorting;
/**
 * Recursively goes upwards through the AST until it finds an AST node that satisfies the given type guard.
 * The entered AST node itself cannot be the result.
 * @param node The current AST node to start the search from.
 * @param guard The type guard function to check if a container matches the desired type.
 * @returns The desired container node that satisfies the type guard, or undefined if not found.
 */
function getNextAstNodeContainer(node, guard) {
    if (node.$container === undefined) {
        return undefined;
    }
    if (guard(node.$container)) {
        return node.$container;
    }
    return getNextAstNodeContainer(node.$container, guard);
}
exports.getNextAstNodeContainer = getNextAstNodeContainer;
/**
 * Utility function that gets all builtin blocktypes.
 * Duplicates are only added once.
 * Make sure to call @see initializeWorkspace first so that the file system is initialized.
 */
function getAllBuiltinBlocktypes(documentService) {
    const allBuiltinBlocktypes = [];
    const visitedBuiltinBlocktypeDefinitions = new Set();
    documentService.all
        .map((document) => document.parseResult.value)
        .forEach((parsedDocument) => {
        if (!(0, ast_1.isJayveeModel)(parsedDocument)) {
            throw new Error('Expected parsed document to be a JayveeModel');
        }
        parsedDocument.blocktypes.forEach((blocktypeDefinition) => {
            if (!(0, ast_1.isBuiltinBlocktypeDefinition)(blocktypeDefinition)) {
                return;
            }
            const wasAlreadyVisited = visitedBuiltinBlocktypeDefinitions.has(blocktypeDefinition);
            if (wasAlreadyVisited) {
                return;
            }
            if (wrappers_1.BlockTypeWrapper.canBeWrapped(blocktypeDefinition)) {
                allBuiltinBlocktypes.push(new wrappers_1.BlockTypeWrapper(blocktypeDefinition));
                visitedBuiltinBlocktypeDefinitions.add(blocktypeDefinition);
            }
        });
    });
    return allBuiltinBlocktypes;
}
exports.getAllBuiltinBlocktypes = getAllBuiltinBlocktypes;
/**
 * Utility function that gets all builtin constraint types.
 * Duplicates are only added once.
 * Make sure to call @see initializeWorkspace first so that the file system is initialized.
 */
function getAllBuiltinConstraintTypes(documentService) {
    const allBuiltinConstraintTypes = [];
    const visitedBuiltinConstraintTypeDefinitions = new Set();
    documentService.all
        .map((document) => document.parseResult.value)
        .forEach((parsedDocument) => {
        if (!(0, ast_1.isJayveeModel)(parsedDocument)) {
            throw new Error('Expected parsed document to be a JayveeModel');
        }
        parsedDocument.constrainttypes.forEach((constraintTypeDefinition) => {
            const wasAlreadyVisited = visitedBuiltinConstraintTypeDefinitions.has(constraintTypeDefinition);
            if (wasAlreadyVisited) {
                return;
            }
            if (wrappers_1.ConstraintTypeWrapper.canBeWrapped(constraintTypeDefinition)) {
                allBuiltinConstraintTypes.push(new wrappers_1.ConstraintTypeWrapper(constraintTypeDefinition));
                visitedBuiltinConstraintTypeDefinitions.add(constraintTypeDefinition);
            }
        });
    });
    return allBuiltinConstraintTypes;
}
exports.getAllBuiltinConstraintTypes = getAllBuiltinConstraintTypes;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/ast-node-wrapper.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getCellIndex = exports.isCellWrapper = exports.getRowIndex = exports.isRowWrapper = exports.getColumnIndex = exports.isColumnWrapper = exports.CellRangeWrapper = exports.rowIndexToString = exports.columnIndexToString = exports.CellIndex = exports.LAST_INDEX = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const column_id_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/util/column-id-util.ts");
exports.LAST_INDEX = Number.MAX_SAFE_INTEGER;
class CellIndex {
    constructor(columnIndex, rowIndex) {
        this.columnIndex = columnIndex;
        this.rowIndex = rowIndex;
    }
    toString() {
        return `${columnIndexToString(this.columnIndex)}${rowIndexToString(this.rowIndex)}`;
    }
    hasRelativeIndexes() {
        return this.columnIndex === exports.LAST_INDEX || this.rowIndex === exports.LAST_INDEX;
    }
    resolveRelativeIndexes(bounds) {
        let columnIndex = this.columnIndex;
        if (columnIndex === exports.LAST_INDEX) {
            columnIndex = bounds.lastColumnIndex;
        }
        let rowIndex = this.rowIndex;
        if (rowIndex === exports.LAST_INDEX) {
            rowIndex = bounds.lastRowIndex;
        }
        return new CellIndex(columnIndex, rowIndex);
    }
    isInBounds(bounds) {
        const columnInBounds = this.isIndexInBounds(this.columnIndex, bounds.lastColumnIndex);
        const rowInBounds = this.isIndexInBounds(this.rowIndex, bounds.lastRowIndex);
        return columnInBounds && rowInBounds;
    }
    isIndexInBounds(index, max) {
        if (max < 0) {
            return false;
        }
        return index === exports.LAST_INDEX || index <= max;
    }
}
exports.CellIndex = CellIndex;
function columnIndexToString(columnIndex) {
    if (columnIndex === exports.LAST_INDEX) {
        return '*';
    }
    return (0, column_id_util_1.columnIndexAsCharacters)(columnIndex);
}
exports.columnIndexToString = columnIndexToString;
function rowIndexToString(rowIndex) {
    if (rowIndex === exports.LAST_INDEX) {
        return '*';
    }
    return `${rowIndex + 1}`;
}
exports.rowIndexToString = rowIndexToString;
class CellRangeWrapper {
    constructor(cellRange, indexes) {
        this.astNode = cellRange;
        if (indexes !== undefined) {
            this.from = indexes.from;
            this.to = indexes.to;
        }
        else if ((0, ast_1.isCellLiteral)(cellRange)) {
            const cellIndex = parseCellReference(cellRange.cellId);
            this.from = cellIndex;
            this.to = cellIndex;
        }
        else if ((0, ast_1.isColumnLiteral)(cellRange)) {
            const columnIndex = parseColumnId(cellRange.columnId.value);
            this.from = new CellIndex(columnIndex, 0);
            this.to = new CellIndex(columnIndex, exports.LAST_INDEX);
        }
        else if ((0, ast_1.isRowLiteral)(cellRange)) {
            const rowIndex = parseRowId(cellRange.rowId.value);
            this.from = new CellIndex(0, rowIndex);
            this.to = new CellIndex(exports.LAST_INDEX, rowIndex);
        }
        else {
            this.from = parseCellReference(cellRange.cellFrom);
            this.to = parseCellReference(cellRange.cellTo);
        }
    }
    static canBeWrapped(cellRange) {
        if ((0, ast_1.isCellLiteral)(cellRange)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return isCompleteCellReference(cellRange === null || cellRange === void 0 ? void 0 : cellRange.cellId);
        }
        else if ((0, ast_1.isColumnLiteral)(cellRange)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return isCompleteColumnId(cellRange === null || cellRange === void 0 ? void 0 : cellRange.columnId);
        }
        else if ((0, ast_1.isRowLiteral)(cellRange)) {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            return isCompleteRowId(cellRange === null || cellRange === void 0 ? void 0 : cellRange.rowId);
        }
        else if ((0, ast_1.isRangeLiteral)(cellRange)) {
            return (
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            isCompleteCellReference(cellRange === null || cellRange === void 0 ? void 0 : cellRange.cellFrom) &&
                // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
                isCompleteCellReference(cellRange === null || cellRange === void 0 ? void 0 : cellRange.cellTo));
        }
        (0, langium_1.assertUnreachable)(cellRange);
    }
    isOneDimensional() {
        return (this.from.columnIndex === this.to.columnIndex ||
            this.from.rowIndex === this.to.rowIndex);
    }
    numberOfCells() {
        const numberOfRows = this.getNumberOfRows();
        const numberOfColumns = this.getNumberOfColumns();
        return numberOfRows * numberOfColumns;
    }
    getNumberOfRows() {
        if (this.from.rowIndex === exports.LAST_INDEX && this.to.rowIndex === exports.LAST_INDEX) {
            return 1;
        }
        return this.to.rowIndex - this.from.rowIndex + 1;
    }
    getNumberOfColumns() {
        if (this.from.columnIndex === exports.LAST_INDEX &&
            this.to.columnIndex === exports.LAST_INDEX) {
            return 1;
        }
        return this.to.columnIndex - this.from.columnIndex + 1;
    }
    isInBounds(bounds) {
        return this.from.isInBounds(bounds) && this.to.isInBounds(bounds);
    }
    resolveRelativeIndexes(bounds) {
        const boundFrom = this.from.resolveRelativeIndexes(bounds);
        const boundTo = this.to.resolveRelativeIndexes(bounds);
        return new CellRangeWrapper(this.astNode, {
            from: boundFrom,
            to: boundTo,
        });
    }
    hasRelativeIndexes() {
        return this.from.hasRelativeIndexes() || this.to.hasRelativeIndexes();
    }
    toString() {
        return `${this.from.toString()}:${this.to.toString()}`;
    }
}
exports.CellRangeWrapper = CellRangeWrapper;
function isColumnWrapper(cellRange) {
    if ((0, ast_1.isColumnLiteral)(cellRange.astNode)) {
        return true;
    }
    if ((0, ast_1.isRangeLiteral)(cellRange.astNode)) {
        return (cellRange.from.columnIndex === cellRange.to.columnIndex &&
            cellRange.from.rowIndex === 0 &&
            cellRange.to.rowIndex === exports.LAST_INDEX);
    }
    return false;
}
exports.isColumnWrapper = isColumnWrapper;
function getColumnIndex(column) {
    (0, assert_1.strict)(isColumnWrapper(column));
    return column.from.columnIndex;
}
exports.getColumnIndex = getColumnIndex;
function isRowWrapper(cellRange) {
    if ((0, ast_1.isRowLiteral)(cellRange.astNode)) {
        return true;
    }
    if ((0, ast_1.isRangeLiteral)(cellRange.astNode)) {
        return (cellRange.from.rowIndex === cellRange.to.rowIndex &&
            cellRange.from.columnIndex === 0 &&
            cellRange.to.columnIndex === exports.LAST_INDEX);
    }
    return false;
}
exports.isRowWrapper = isRowWrapper;
function getRowIndex(row) {
    (0, assert_1.strict)(isRowWrapper(row));
    return row.from.rowIndex;
}
exports.getRowIndex = getRowIndex;
function isCellWrapper(cellRange) {
    if ((0, ast_1.isCellLiteral)(cellRange.astNode)) {
        return true;
    }
    if ((0, ast_1.isRangeLiteral)(cellRange.astNode)) {
        return (cellRange.from.columnIndex === cellRange.to.columnIndex &&
            cellRange.from.rowIndex === cellRange.to.rowIndex);
    }
    return false;
}
exports.isCellWrapper = isCellWrapper;
function getCellIndex(cell) {
    (0, assert_1.strict)(isCellWrapper(cell));
    return cell.from;
}
exports.getCellIndex = getCellIndex;
function isCompleteCellReference(cellReference) {
    if ((0, ast_1.isCellReference)(cellReference)) {
        return (
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        isCompleteColumnId(cellReference === null || cellReference === void 0 ? void 0 : cellReference.columnId) &&
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            isCompleteRowId(cellReference === null || cellReference === void 0 ? void 0 : cellReference.rowId));
    }
    return cellReference !== undefined;
}
function isCompleteColumnId(columnId) {
    return (columnId === null || columnId === void 0 ? void 0 : columnId.value) !== undefined;
}
function isCompleteRowId(rowId) {
    return (rowId === null || rowId === void 0 ? void 0 : rowId.value) !== undefined;
}
function parseCellReference(cellReference) {
    let columnId;
    let rowId;
    if (typeof cellReference === 'string') {
        const parseResult = parseStringCellReference(cellReference);
        columnId = parseResult.columnId;
        rowId = parseResult.rowId;
    }
    else {
        columnId = cellReference.columnId.value;
        rowId = cellReference.rowId.value;
    }
    const columnIndex = parseColumnId(columnId);
    const rowIndex = parseRowId(rowId);
    return new CellIndex(columnIndex, rowIndex);
}
const CELL_REFERENCE_REGEX = /([A-Z]+)([0-9]+)/;
function parseStringCellReference(cellReference) {
    const matches = CELL_REFERENCE_REGEX.exec(cellReference) || [];
    const columnIdMatch = matches[1];
    const rowIdMatch = matches[2];
    (0, assert_1.strict)(columnIdMatch !== undefined && rowIdMatch !== undefined, `Cell IDs are expected to match the regular expression ${CELL_REFERENCE_REGEX.toString()}`);
    return {
        columnId: columnIdMatch,
        rowId: Number.parseInt(rowIdMatch, 10),
    };
}
function parseColumnId(columnId) {
    if (columnId === '*') {
        return exports.LAST_INDEX;
    }
    return (0, column_id_util_1.columnCharactersAsIndex)(columnId);
}
function parseRowId(rowId) {
    if (rowId === '*') {
        return exports.LAST_INDEX;
    }
    return rowId - 1;
}


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
// eslint-disable-next-line import/no-cycle
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/util/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/ast-node-wrapper.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/index.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/pipe-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createWrappersFromPipeChain = exports.PipeWrapper = void 0;
const assert_1 = __webpack_require__("assert");
class PipeWrapper {
    constructor(pipe, chainIndex) {
        var _a, _b;
        this.astNode = pipe;
        (0, assert_1.strict)(0 <= chainIndex && chainIndex + 1 < pipe.blocks.length);
        (0, assert_1.strict)(((_a = pipe.blocks[chainIndex]) === null || _a === void 0 ? void 0 : _a.ref) !== undefined);
        (0, assert_1.strict)(((_b = pipe.blocks[chainIndex + 1]) === null || _b === void 0 ? void 0 : _b.ref) !== undefined);
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        this.from = pipe.blocks[chainIndex].ref;
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        this.to = pipe.blocks[chainIndex + 1].ref;
        this.chainIndex = chainIndex;
    }
    getFromDiagnostic() {
        return {
            node: this.astNode,
            property: 'blocks',
            index: this.chainIndex,
        };
    }
    getToDiagnostic() {
        return {
            node: this.astNode,
            property: 'blocks',
            index: this.chainIndex + 1,
        };
    }
    equals(pipe) {
        return this.from === pipe.from && this.to === pipe.to;
    }
    static canBeWrapped(pipe, chainIndex) {
        var _a, _b;
        return (0 <= chainIndex &&
            chainIndex + 1 < pipe.blocks.length &&
            ((_a = pipe.blocks[chainIndex]) === null || _a === void 0 ? void 0 : _a.ref) !== undefined &&
            ((_b = pipe.blocks[chainIndex + 1]) === null || _b === void 0 ? void 0 : _b.ref) !== undefined);
    }
}
exports.PipeWrapper = PipeWrapper;
function createWrappersFromPipeChain(pipe) {
    const result = [];
    for (let chainIndex = 0; chainIndex < pipe.blocks.length - 1; ++chainIndex) {
        if (!PipeWrapper.canBeWrapped(pipe, chainIndex)) {
            continue;
        }
        const pipeWrapper = new PipeWrapper(pipe, chainIndex);
        result.push(pipeWrapper);
    }
    return result;
}
exports.createWrappersFromPipeChain = createWrappersFromPipeChain;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/typed-object/blocktype-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.BlockTypeWrapper = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const services_1 = __webpack_require__("./libs/language-server/src/lib/services/index.ts");
// eslint-disable-next-line import/no-cycle
const expressions_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/index.ts");
const io_type_1 = __webpack_require__("./libs/language-server/src/lib/ast/io-type.ts");
const value_type_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/index.ts");
const typed_object_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/typed-object-wrapper.ts");
class BlockTypeWrapper extends typed_object_wrapper_1.TypedObjectWrapper {
    constructor(toBeWrapped) {
        const blocktypeDefinition = (0, langium_1.isReference)(toBeWrapped)
            ? toBeWrapped.ref
            : toBeWrapped;
        (0, assert_1.strict)(blocktypeDefinition !== undefined);
        const blocktypeName = blocktypeDefinition.name;
        const properties = {};
        for (const property of blocktypeDefinition.properties) {
            const valuetype = (0, value_type_1.createValuetype)(property.valueType);
            (0, assert_1.strict)(valuetype !== undefined);
            properties[property.name] = {
                type: valuetype,
            };
            const defaultValue = (0, expressions_1.evaluateExpression)(property.defaultValue, new expressions_1.EvaluationContext(new services_1.RuntimeParameterProvider()));
            if (defaultValue !== undefined) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                properties[property.name].defaultValue = defaultValue;
            }
        }
        super(blocktypeDefinition, blocktypeName, properties, undefined);
        this.docs = {};
        const inputPort = blocktypeDefinition.inputs[0];
        (0, assert_1.strict)(inputPort !== undefined);
        this.inputType = (0, io_type_1.getIOType)(inputPort);
        const outputPort = blocktypeDefinition.outputs[0];
        (0, assert_1.strict)(outputPort !== undefined);
        this.outputType = (0, io_type_1.getIOType)(outputPort);
    }
    static canBeWrapped(toBeWrapped) {
        const blocktypeDefinition = (0, langium_1.isReference)(toBeWrapped)
            ? toBeWrapped.ref
            : toBeWrapped;
        if (blocktypeDefinition === undefined) {
            return false;
        }
        if (
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        blocktypeDefinition.properties === undefined ||
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            blocktypeDefinition.name === undefined ||
            blocktypeDefinition.inputs[0] === undefined ||
            blocktypeDefinition.outputs[0] === undefined) {
            return false;
        }
        if (blocktypeDefinition.properties.some((property) => {
            return property.valueType.reference.ref === undefined;
        })) {
            return false;
        }
        return true;
    }
    canBeConnectedTo(blockAfter) {
        return this.outputType === blockAfter.inputType;
    }
    hasInput() {
        return this.inputType !== io_type_1.IOType.NONE;
    }
    hasOutput() {
        return this.outputType !== io_type_1.IOType.NONE;
    }
}
exports.BlockTypeWrapper = BlockTypeWrapper;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/typed-object/composite-blocktype-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.CompositeBlocktypeWrapper = void 0;
// eslint-disable-next-line import/no-cycle
const blocktype_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/blocktype-wrapper.ts");
class CompositeBlocktypeWrapper extends blocktype_wrapper_1.BlockTypeWrapper {
    constructor(blockTypeDefinition) {
        super(blockTypeDefinition);
        this.blockTypeDefinition = blockTypeDefinition;
    }
    getMissingRequiredPropertyNames(presentPropertyNames = []) {
        const missingRequiredPropertyNames = super.getMissingRequiredPropertyNames(presentPropertyNames);
        // We assume blocktype properties that have an expression as default value can be evaluated during runtime
        return missingRequiredPropertyNames.filter((propertyName) => {
            const blocktypeProperty = this.blockTypeDefinition.properties.find((blocktypeProperty) => blocktypeProperty.name === propertyName);
            return (blocktypeProperty === null || blocktypeProperty === void 0 ? void 0 : blocktypeProperty.defaultValue) === undefined;
        });
    }
}
exports.CompositeBlocktypeWrapper = CompositeBlocktypeWrapper;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/typed-object/constrainttype-wrapper.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ConstraintTypeWrapper = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const services_1 = __webpack_require__("./libs/language-server/src/lib/services/index.ts");
// eslint-disable-next-line import/no-cycle
const evaluation_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluation.ts");
const value_type_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/index.ts");
const typed_object_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/typed-object-wrapper.ts");
class ConstraintTypeWrapper extends typed_object_wrapper_1.TypedObjectWrapper {
    constructor(toBeWrapped) {
        const constraintTypeDefinition = (0, langium_1.isReference)(toBeWrapped)
            ? toBeWrapped.ref
            : toBeWrapped;
        (0, assert_1.strict)(constraintTypeDefinition !== undefined);
        const constraintTypeName = constraintTypeDefinition.name;
        const properties = {};
        for (const property of constraintTypeDefinition.properties) {
            const valuetype = (0, value_type_1.createValuetype)(property.valueType);
            (0, assert_1.strict)(valuetype !== undefined);
            properties[property.name] = {
                type: valuetype,
            };
            const defaultValue = (0, evaluation_1.evaluateExpression)(property.defaultValue, new evaluation_1.EvaluationContext(new services_1.RuntimeParameterProvider()));
            if (defaultValue !== undefined) {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                properties[property.name].defaultValue = defaultValue;
            }
        }
        super(constraintTypeDefinition, constraintTypeName, properties, undefined);
        this.docs = {};
        const valuetype = (0, value_type_1.createValuetype)(constraintTypeDefinition.valuetype);
        (0, assert_1.strict)(valuetype !== undefined);
        this.on = valuetype;
    }
    static canBeWrapped(toBeWrapped) {
        const constraintTypeDefinition = (0, langium_1.isReference)(toBeWrapped)
            ? toBeWrapped.ref
            : toBeWrapped;
        if (constraintTypeDefinition === undefined) {
            return false;
        }
        if (
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        constraintTypeDefinition.properties === undefined ||
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            constraintTypeDefinition.name === undefined ||
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            constraintTypeDefinition.valuetype === undefined) {
            return false;
        }
        if (constraintTypeDefinition.properties.some((property) => {
            return property.valueType.reference.ref === undefined;
        })) {
            return false;
        }
        return true;
    }
}
exports.ConstraintTypeWrapper = ConstraintTypeWrapper;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/typed-object/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
/* eslint-disable import/no-cycle */
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/typed-object-wrapper.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/blocktype-wrapper.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/composite-blocktype-wrapper.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/constrainttype-wrapper.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/typed-object/typed-object-wrapper.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.TypedObjectWrapper = void 0;
class TypedObjectWrapper {
    constructor(astNode, type, properties, validation) {
        this.astNode = astNode;
        this.type = type;
        this.properties = properties;
        this.validation = validation;
    }
    validate(propertyBody, validationContext, evaluationContext) {
        var _a;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        const properties = (_a = propertyBody === null || propertyBody === void 0 ? void 0 : propertyBody.properties) !== null && _a !== void 0 ? _a : [];
        for (const property of properties) {
            const propertySpecification = this.getPropertySpecification(property.name);
            const propertyValidationFn = propertySpecification === null || propertySpecification === void 0 ? void 0 : propertySpecification.validation;
            if (propertyValidationFn === undefined) {
                continue;
            }
            propertyValidationFn(property, validationContext, evaluationContext);
        }
        if (this.validation !== undefined) {
            this.validation(propertyBody, validationContext, evaluationContext);
        }
    }
    getPropertySpecification(name) {
        if (name === undefined) {
            return undefined;
        }
        return this.properties[name];
    }
    getPropertySpecifications() {
        return this.properties;
    }
    hasPropertySpecification(name) {
        return this.getPropertySpecification(name) !== undefined;
    }
    getMissingRequiredPropertyNames(presentPropertyNames = []) {
        return this.getPropertyNames('required', presentPropertyNames);
    }
    getPropertyNames(kind = undefined, excludeNames = []) {
        const resultingPropertyNames = [];
        for (const [name, spec] of Object.entries(this.properties)) {
            if (kind === 'optional' && spec.defaultValue === undefined) {
                continue;
            }
            if (kind === 'required' && spec.defaultValue !== undefined) {
                continue;
            }
            if (excludeNames.includes(name)) {
                continue;
            }
            resultingPropertyNames.push(name);
        }
        return resultingPropertyNames;
    }
}
exports.TypedObjectWrapper = TypedObjectWrapper;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/util/column-id-util.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.columnCharactersAsIndex = exports.columnIndexAsCharacters = void 0;
const firstCharacter = 'A';
const lastCharacter = 'Z';
const characterOffset = firstCharacter.charCodeAt(0);
const numberOfCharacters = lastCharacter.charCodeAt(0) - characterOffset + 1;
function columnIndexAsCharacters(columnIndex) {
    let columnCharacters = '';
    let value = columnIndex + 1;
    do {
        const remainder = value % numberOfCharacters;
        let quotient = Math.floor(value / numberOfCharacters);
        if (remainder !== 0) {
            columnCharacters =
                String.fromCharCode(remainder + characterOffset - 1) + columnCharacters;
        }
        else {
            --quotient;
            columnCharacters = lastCharacter + columnCharacters;
        }
        value = quotient;
    } while (value > 0);
    return columnCharacters;
}
exports.columnIndexAsCharacters = columnIndexAsCharacters;
function columnCharactersAsIndex(columnCharacters) {
    let columnIndex = 0;
    for (let position = 0; position < columnCharacters.length; ++position) {
        const charCode = columnCharacters.charCodeAt(position);
        const characterIndex = charCode - characterOffset + 1;
        const factor = Math.pow(numberOfCharacters, columnCharacters.length - 1 - position);
        columnIndex += characterIndex * factor;
    }
    return columnIndex - 1;
}
exports.columnCharactersAsIndex = columnCharactersAsIndex;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/util/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getTypedObjectWrapper = void 0;
const tslib_1 = __webpack_require__("tslib");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
// eslint-disable-next-line import/no-cycle
const blocktype_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/blocktype-wrapper.ts");
const constrainttype_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/constrainttype-wrapper.ts");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/util/column-id-util.ts"), exports);
/**
 * Creates a @see TypedObjectWrapper wrapper object based on the given type reference.
 */
function getTypedObjectWrapper(typeRef) {
    const type = (0, langium_1.isReference)(typeRef) ? typeRef.ref : typeRef;
    if (type === undefined) {
        return undefined;
    }
    if ((0, ast_1.isReferenceableBlocktypeDefinition)(type)) {
        if (!blocktype_wrapper_1.BlockTypeWrapper.canBeWrapped(type)) {
            return undefined;
        }
        return new blocktype_wrapper_1.BlockTypeWrapper(type);
    }
    else if ((0, ast_1.isBuiltinConstrainttypeDefinition)(type)) {
        if (!constrainttype_wrapper_1.ConstraintTypeWrapper.canBeWrapped(type)) {
            return undefined;
        }
        return new constrainttype_wrapper_1.ConstraintTypeWrapper(type);
    }
    (0, langium_1.assertUnreachable)(type);
}
exports.getTypedObjectWrapper = getTypedObjectWrapper;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/atomic-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isAtomicValuetype = exports.AtomicValuetype = void 0;
const assert_1 = __webpack_require__("assert");
// eslint-disable-next-line import/no-cycle
const evaluation_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/evaluation.ts");
// eslint-disable-next-line import/no-cycle
const primitive_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/index.ts");
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype.ts");
const valuetype_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype-util.ts");
class AtomicValuetype extends valuetype_1.AbstractValuetype {
    constructor(astNode) {
        super();
        this.astNode = astNode;
    }
    acceptVisitor(visitor) {
        return visitor.visitAtomicValuetype(this);
    }
    getConstraints(context) {
        var _a, _b;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        const constraintCollection = (_a = this.astNode) === null || _a === void 0 ? void 0 : _a.constraints;
        (0, assert_1.strict)(constraintCollection !== undefined);
        const constraintCollectionType = new primitive_1.CollectionValuetype(primitive_valuetypes_1.PrimitiveValuetypes.Constraint);
        const constraints = (_b = (0, evaluation_1.evaluateExpression)(constraintCollection, context)) !== null && _b !== void 0 ? _b : [];
        if (!constraintCollectionType.isInternalValueRepresentation(constraints)) {
            return [];
        }
        return constraints;
    }
    isConvertibleTo(target) {
        if (target.equals(this)) {
            return true;
        }
        const supertype = this.getSupertype();
        if (supertype === undefined) {
            return false;
        }
        return supertype.isConvertibleTo(target);
    }
    isReferenceableByUser() {
        const supertype = this.getSupertype();
        if (supertype === undefined) {
            return false;
        }
        return supertype.isReferenceableByUser();
    }
    isAllowedAsRuntimeParameter() {
        const supertype = this.getSupertype();
        if (supertype === undefined) {
            return false;
        }
        return supertype.isAllowedAsRuntimeParameter();
    }
    getName() {
        var _a;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        return (_a = this.astNode.name) !== null && _a !== void 0 ? _a : '';
    }
    doGetSupertype() {
        var _a;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        const supertype = (_a = this.astNode) === null || _a === void 0 ? void 0 : _a.type;
        return (0, valuetype_util_1.createValuetype)(supertype);
    }
    isInternalValueRepresentation(operandValue) {
        const supertype = this.getSupertype();
        if (supertype === undefined) {
            return false;
        }
        return supertype.isInternalValueRepresentation(operandValue);
    }
    equals(target) {
        if (target instanceof AtomicValuetype) {
            return this.astNode === target.astNode;
        }
        return false;
    }
}
exports.AtomicValuetype = AtomicValuetype;
function isAtomicValuetype(v) {
    return v instanceof AtomicValuetype;
}
exports.isAtomicValuetype = isAtomicValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isAtomicValuetype = void 0;
const tslib_1 = __webpack_require__("tslib");
// eslint-disable-next-line import/no-cycle
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype.ts"), exports);
var atomic_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/atomic-valuetype.ts");
Object.defineProperty(exports, "isAtomicValuetype", ({ enumerable: true, get: function () { return atomic_valuetype_1.isAtomicValuetype; } }));
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype-util.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/boolean-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isBooleanValuetype = exports.Boolean = void 0;
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class BooleanValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitBoolean(this);
    }
    isAllowedAsRuntimeParameter() {
        return true;
    }
    getName() {
        return 'boolean';
    }
    isInternalValueRepresentation(operandValue) {
        return typeof operandValue === 'boolean';
    }
    isReferenceableByUser() {
        return true;
    }
    getUserDoc() {
        return `
A boolean value.
Examples: true, false
`.trim();
    }
}
// Only export instance to enforce singleton
exports.Boolean = new BooleanValuetypeImpl();
function isBooleanValuetype(v) {
    return v === exports.Boolean;
}
exports.isBooleanValuetype = isBooleanValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/cell-range-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isCellRangeValuetype = exports.CellRange = void 0;
const cell_range_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts");
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class CellRangeValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitCellRange(this);
    }
    isAllowedAsRuntimeParameter() {
        return false;
    }
    getName() {
        return 'CellRange';
    }
    isInternalValueRepresentation(operandValue) {
        return operandValue instanceof cell_range_wrapper_1.CellRangeWrapper;
    }
}
// Only export instance to enforce singleton
exports.CellRange = new CellRangeValuetypeImpl();
function isCellRangeValuetype(v) {
    return v === exports.CellRange;
}
exports.isCellRangeValuetype = isCellRangeValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/abstract-collection-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.AbstractCollectionValuetype = void 0;
// eslint-disable-next-line import/no-cycle
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class AbstractCollectionValuetype extends primitive_valuetype_1.PrimitiveValuetype {
    isAllowedAsRuntimeParameter() {
        return false;
    }
}
exports.AbstractCollectionValuetype = AbstractCollectionValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/collection-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createCollectionValuetype = exports.isCollectionValuetype = exports.CollectionValuetype = void 0;
const assert_1 = __webpack_require__("assert");
const valuetype_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype-util.ts");
const abstract_collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/abstract-collection-valuetype.ts");
class CollectionValuetype extends abstract_collection_valuetype_1.AbstractCollectionValuetype {
    constructor(elementType) {
        super();
        this.elementType = elementType;
    }
    acceptVisitor(visitor) {
        return visitor.visitCollection(this);
    }
    getName() {
        return `Collection<${this.elementType.getName()}>`;
    }
    equals(target) {
        return (target instanceof CollectionValuetype &&
            target.elementType.equals(this.elementType));
    }
    isInternalValueRepresentation(operandValue) {
        return (Array.isArray(operandValue) &&
            operandValue.every((element) => this.elementType.isInternalValueRepresentation(element)));
    }
}
exports.CollectionValuetype = CollectionValuetype;
function isCollectionValuetype(v, elementType) {
    return v instanceof CollectionValuetype && v.elementType.equals(elementType);
}
exports.isCollectionValuetype = isCollectionValuetype;
function createCollectionValuetype(collectionRef) {
    var _a;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const collectionDefinition = (_a = collectionRef === null || collectionRef === void 0 ? void 0 : collectionRef.reference) === null || _a === void 0 ? void 0 : _a.ref;
    (0, assert_1.strict)((collectionDefinition === null || collectionDefinition === void 0 ? void 0 : collectionDefinition.name) === 'Collection');
    const collectionGenerics = collectionRef.genericRefs;
    if (collectionGenerics.length !== 1) {
        throw new Error("Valuetype Collection needs exactly one generic parameter to define its elements' type");
    }
    const generic = collectionGenerics[0];
    (0, assert_1.strict)(generic !== undefined);
    const elementValuetype = (0, valuetype_util_1.createValuetype)(generic.ref);
    if (elementValuetype === undefined) {
        throw new Error("Could not create valuetype for the elements' type of valuetype Collection");
    }
    return new CollectionValuetype(elementValuetype);
}
exports.createCollectionValuetype = createCollectionValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/empty-collection-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isEmptyCollectionValuetype = exports.EmptyCollection = void 0;
// eslint-disable-next-line import/no-cycle
const abstract_collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/abstract-collection-valuetype.ts");
const collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/collection-valuetype.ts");
class EmptyCollectionValuetypeImpl extends abstract_collection_valuetype_1.AbstractCollectionValuetype {
    isConvertibleTo(target) {
        return (super.isConvertibleTo(target) || target instanceof collection_valuetype_1.CollectionValuetype);
    }
    acceptVisitor(visitor) {
        return visitor.visitCollection(this);
    }
    getName() {
        return `Collection`;
    }
    isInternalValueRepresentation(operandValue) {
        return Array.isArray(operandValue) && operandValue.length === 0;
    }
}
// Only export instance to enforce singleton
exports.EmptyCollection = new EmptyCollectionValuetypeImpl();
function isEmptyCollectionValuetype(v) {
    return v === exports.EmptyCollection;
}
exports.isEmptyCollectionValuetype = isEmptyCollectionValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isEmptyCollectionValuetype = exports.isCollectionValuetype = exports.CollectionValuetype = void 0;
// eslint-disable-next-line import/no-cycle
var collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/collection-valuetype.ts");
Object.defineProperty(exports, "CollectionValuetype", ({ enumerable: true, get: function () { return collection_valuetype_1.CollectionValuetype; } }));
Object.defineProperty(exports, "isCollectionValuetype", ({ enumerable: true, get: function () { return collection_valuetype_1.isCollectionValuetype; } }));
var empty_collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/empty-collection-valuetype.ts");
Object.defineProperty(exports, "isEmptyCollectionValuetype", ({ enumerable: true, get: function () { return empty_collection_valuetype_1.isEmptyCollectionValuetype; } }));


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/constraint-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isConstraintValuetype = exports.Constraint = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class ConstraintValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitConstraint(this);
    }
    isAllowedAsRuntimeParameter() {
        return false;
    }
    getName() {
        return 'Constraint';
    }
    isInternalValueRepresentation(operandValue) {
        return (0, ast_1.isConstraintDefinition)(operandValue);
    }
}
// Only export instance to enforce singleton
exports.Constraint = new ConstraintValuetypeImpl();
function isConstraintValuetype(v) {
    return v === exports.Constraint;
}
exports.isConstraintValuetype = isConstraintValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/decimal-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isDecimalValuetype = exports.Decimal = void 0;
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class DecimalValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitDecimal(this);
    }
    isAllowedAsRuntimeParameter() {
        return true;
    }
    getName() {
        return 'decimal';
    }
    isInternalValueRepresentation(operandValue) {
        return typeof operandValue === 'number' && Number.isFinite(operandValue);
    }
    isReferenceableByUser() {
        return true;
    }
    getUserDoc() {
        return `
A decimal value.
Example: 3.14
`.trim();
    }
}
// Only export instance to enforce singleton
exports.Decimal = new DecimalValuetypeImpl();
function isDecimalValuetype(v) {
    return v === exports.Decimal;
}
exports.isDecimalValuetype = isDecimalValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isTransformValuetype = exports.isValuetypeAssignmentValuetype = exports.isTextValuetype = exports.isRegexValuetype = exports.isIntegerValuetype = exports.isDecimalValuetype = exports.isConstraintValuetype = exports.isCellRangeValuetype = exports.isBooleanValuetype = void 0;
const tslib_1 = __webpack_require__("tslib");
// eslint-disable-next-line import/no-cycle
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts"), exports);
var boolean_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/boolean-valuetype.ts");
Object.defineProperty(exports, "isBooleanValuetype", ({ enumerable: true, get: function () { return boolean_valuetype_1.isBooleanValuetype; } }));
var cell_range_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/cell-range-valuetype.ts");
Object.defineProperty(exports, "isCellRangeValuetype", ({ enumerable: true, get: function () { return cell_range_valuetype_1.isCellRangeValuetype; } }));
var constraint_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/constraint-valuetype.ts");
Object.defineProperty(exports, "isConstraintValuetype", ({ enumerable: true, get: function () { return constraint_valuetype_1.isConstraintValuetype; } }));
var decimal_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/decimal-valuetype.ts");
Object.defineProperty(exports, "isDecimalValuetype", ({ enumerable: true, get: function () { return decimal_valuetype_1.isDecimalValuetype; } }));
var integer_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/integer-valuetype.ts");
Object.defineProperty(exports, "isIntegerValuetype", ({ enumerable: true, get: function () { return integer_valuetype_1.isIntegerValuetype; } }));
var regex_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/regex-valuetype.ts");
Object.defineProperty(exports, "isRegexValuetype", ({ enumerable: true, get: function () { return regex_valuetype_1.isRegexValuetype; } }));
var text_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/text-valuetype.ts");
Object.defineProperty(exports, "isTextValuetype", ({ enumerable: true, get: function () { return text_valuetype_1.isTextValuetype; } }));
var valuetype_assignment_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/valuetype-assignment-valuetype.ts");
Object.defineProperty(exports, "isValuetypeAssignmentValuetype", ({ enumerable: true, get: function () { return valuetype_assignment_valuetype_1.isValuetypeAssignmentValuetype; } }));
var transform_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/transform-valuetype.ts");
Object.defineProperty(exports, "isTransformValuetype", ({ enumerable: true, get: function () { return transform_valuetype_1.isTransformValuetype; } }));


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/integer-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isIntegerValuetype = exports.Integer = void 0;
const decimal_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/decimal-valuetype.ts");
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class IntegerValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    isConvertibleTo(target) {
        return super.isConvertibleTo(target) || target === decimal_valuetype_1.Decimal;
    }
    acceptVisitor(visitor) {
        return visitor.visitInteger(this);
    }
    isAllowedAsRuntimeParameter() {
        return true;
    }
    getName() {
        return 'integer';
    }
    isInternalValueRepresentation(operandValue) {
        return typeof operandValue === 'number' && Number.isInteger(operandValue);
    }
    isReferenceableByUser() {
        return true;
    }
    getUserDoc() {
        return `
An integer value.
Example: 3
`.trim();
    }
}
// Only export instance to enforce singleton
exports.Integer = new IntegerValuetypeImpl();
function isIntegerValuetype(v) {
    return v === exports.Integer;
}
exports.isIntegerValuetype = isIntegerValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isPrimitiveValuetype = exports.PrimitiveValuetype = void 0;
// eslint-disable-next-line import/no-cycle
const valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype.ts");
class PrimitiveValuetype extends valuetype_1.AbstractValuetype {
    constructor() {
        super();
    }
    isConvertibleTo(target) {
        return target.equals(this);
    }
    equals(target) {
        return target === this;
    }
    doGetSupertype() {
        return undefined;
    }
    /**
     * The user documentation for the value type.
     * Text only, no comment characters.
     * Should be given for all user-referenceable value types @see isReferenceableByUser
     */
    getUserDoc() {
        return undefined;
    }
}
exports.PrimitiveValuetype = PrimitiveValuetype;
function isPrimitiveValuetype(v) {
    return v instanceof PrimitiveValuetype;
}
exports.isPrimitiveValuetype = isPrimitiveValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createPrimitiveValuetype = exports.PrimitiveValuetypes = void 0;
const assert_1 = __webpack_require__("assert");
// eslint-disable-next-line import/no-cycle
const boolean_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/boolean-valuetype.ts");
const cell_range_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/cell-range-valuetype.ts");
const constraint_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/constraint-valuetype.ts");
const decimal_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/decimal-valuetype.ts");
const integer_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/integer-valuetype.ts");
const regex_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/regex-valuetype.ts");
const text_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/text-valuetype.ts");
const transform_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/transform-valuetype.ts");
const valuetype_assignment_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/valuetype-assignment-valuetype.ts");
exports.PrimitiveValuetypes = {
    Boolean: boolean_valuetype_1.Boolean,
    Decimal: decimal_valuetype_1.Decimal,
    Integer: integer_valuetype_1.Integer,
    Text: text_valuetype_1.Text,
    Regex: regex_valuetype_1.Regex,
    CellRange: cell_range_valuetype_1.CellRange,
    Constraint: constraint_valuetype_1.Constraint,
    ValuetypeAssignment: valuetype_assignment_valuetype_1.ValuetypeAssignment,
    Transform: transform_valuetype_1.Transform,
};
function createPrimitiveValuetype(builtinValuetype) {
    (0, assert_1.strict)(builtinValuetype.isBuiltin);
    const name = builtinValuetype.name;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (name === undefined) {
        return undefined;
    }
    const matchingPrimitives = Object.values(exports.PrimitiveValuetypes).filter((valuetype) => valuetype.getName() === name);
    if (matchingPrimitives.length === 0) {
        throw new Error(`Found no PrimitiveValuetype for builtin valuetype "${name}"`);
    }
    if (matchingPrimitives.length > 1) {
        throw new Error(`Found multiple ambiguous PrimitiveValuetype for builtin valuetype "${name}"`);
    }
    return matchingPrimitives[0];
}
exports.createPrimitiveValuetype = createPrimitiveValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/regex-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isRegexValuetype = exports.Regex = void 0;
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class RegexValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitRegex(this);
    }
    isAllowedAsRuntimeParameter() {
        return false;
    }
    getName() {
        return 'Regex';
    }
    isInternalValueRepresentation(operandValue) {
        return operandValue instanceof RegExp;
    }
}
// Only export instance to enforce singleton
exports.Regex = new RegexValuetypeImpl();
function isRegexValuetype(v) {
    return v === exports.Regex;
}
exports.isRegexValuetype = isRegexValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/text-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isTextValuetype = exports.Text = void 0;
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class TextValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitText(this);
    }
    isAllowedAsRuntimeParameter() {
        return true;
    }
    getName() {
        return 'text';
    }
    isInternalValueRepresentation(operandValue) {
        return typeof operandValue === 'string';
    }
    isReferenceableByUser() {
        return true;
    }
    getUserDoc() {
        return `
A text value. 
Example: "Hello World"
`.trim();
    }
}
// Only export instance to enforce singleton
exports.Text = new TextValuetypeImpl();
function isTextValuetype(v) {
    return v === exports.Text;
}
exports.isTextValuetype = isTextValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/transform-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isTransformValuetype = exports.Transform = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class TransformValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitTransform(this);
    }
    isAllowedAsRuntimeParameter() {
        return false;
    }
    getName() {
        return 'Transform';
    }
    isInternalValueRepresentation(operandValue) {
        return (0, ast_1.isTransformDefinition)(operandValue);
    }
}
// Only export instance to enforce singleton
exports.Transform = new TransformValuetypeImpl();
function isTransformValuetype(v) {
    return v === exports.Transform;
}
exports.isTransformValuetype = isTransformValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/primitive/valuetype-assignment-valuetype.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isValuetypeAssignmentValuetype = exports.ValuetypeAssignment = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const primitive_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetype.ts");
class ValuetypeAssignmentValuetypeImpl extends primitive_valuetype_1.PrimitiveValuetype {
    acceptVisitor(visitor) {
        return visitor.visitValuetypeAssignment(this);
    }
    isAllowedAsRuntimeParameter() {
        return false;
    }
    getName() {
        return 'ValuetypeAssignment';
    }
    isInternalValueRepresentation(operandValue) {
        return (0, ast_1.isValuetypeAssignment)(operandValue);
    }
}
// Only export instance to enforce singleton
exports.ValuetypeAssignment = new ValuetypeAssignmentValuetypeImpl();
function isValuetypeAssignmentValuetype(v) {
    return v === exports.ValuetypeAssignment;
}
exports.isValuetypeAssignmentValuetype = isValuetypeAssignmentValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/valuetype-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createValuetype = void 0;
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
// eslint-disable-next-line import/no-cycle
const atomic_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/atomic-valuetype.ts");
// eslint-disable-next-line import/no-cycle
const primitive_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/index.ts");
const collection_valuetype_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/collection/collection-valuetype.ts");
/**
 * Returns the matching valuetype instance for a given valuetype keyword or definition.
 * @returns the desired valuetype instance or undefined in case of incomplete AST nodes or a cycle in the hierarchy.
 */
function createValuetype(identifier) {
    var _a;
    if (identifier === undefined) {
        return undefined;
    }
    else if ((0, ast_1.isValuetypeReference)(identifier)) {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        const valuetypeDefinition = (_a = identifier === null || identifier === void 0 ? void 0 : identifier.reference) === null || _a === void 0 ? void 0 : _a.ref;
        if ((valuetypeDefinition === null || valuetypeDefinition === void 0 ? void 0 : valuetypeDefinition.name) === 'Collection') {
            return (0, collection_valuetype_1.createCollectionValuetype)(identifier);
        }
        return createValuetype(valuetypeDefinition);
    }
    else if ((0, ast_1.isValuetypeDefinition)(identifier)) {
        if (identifier.name === 'Collection') {
            // We don't have an object representing a generic collection
            return;
        }
        if (identifier.isBuiltin) {
            return (0, primitive_1.createPrimitiveValuetype)(identifier);
        }
        return new atomic_valuetype_1.AtomicValuetype(identifier);
    }
    (0, langium_1.assertUnreachable)(identifier);
}
exports.createValuetype = createValuetype;


/***/ }),

/***/ "./libs/language-server/src/lib/ast/wrappers/value-type/valuetype.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ValuetypeVisitor = exports.AbstractValuetype = void 0;
class AbstractValuetype {
    isSubtypeOf(other) {
        let othersSupertype = other.getSupertype();
        while (othersSupertype !== undefined) {
            if (othersSupertype === this) {
                return true;
            }
            othersSupertype = othersSupertype.getSupertype();
        }
        return false;
    }
    getSupertype() {
        if (this.hasSupertypeCycle()) {
            return undefined;
        }
        return this.doGetSupertype();
    }
    isReferenceableByUser() {
        return false;
    }
    hasSupertypeCycle(visited = []) {
        const cycleDetected = visited.some((v) => v.equals(this));
        if (cycleDetected) {
            return true;
        }
        visited.push(this);
        const supertype = this.doGetSupertype();
        if (supertype === undefined) {
            return false;
        }
        return supertype.hasSupertypeCycle(visited);
    }
}
exports.AbstractValuetype = AbstractValuetype;
class ValuetypeVisitor {
}
exports.ValuetypeVisitor = ValuetypeVisitor;


/***/ }),

/***/ "./libs/language-server/src/lib/builtin-library/generated/partial-stdlib.ts":
/***/ ((__unused_webpack_module, exports) => {


/******************************************************************************
* This file was generated by tools/scripts/language-server/generate-stdlib.mjs.
* DO NOT EDIT MANUALLY!
******************************************************************************/
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.PartialStdLib = void 0;
exports.PartialStdLib = {
    "builtin:///stdlib/CSVExtractor.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* A CSVExtractor extracts a file from a given URL and interprets it as CSV\n*/\ncomposite blocktype CSVExtractor {\n    property url oftype text;\n    property delimiter oftype text: ',';\n    property enclosing oftype text: '';\n    property enclosingEscape oftype text: '';\n    property encoding oftype text: 'utf-8';\n\n    input inputPort oftype None;\n    output outputPort oftype Sheet;\n\n    inputPort\n        -> FileExtractor\n        -> FileCSVInterpreter\n        -> outputPort;\n\n    block FileExtractor oftype HttpExtractor { \n        url: url; \n    }\n\n\tblock FileCSVInterpreter oftype CSVFileInterpreter {\n\t\tdelimiter: delimiter;\n\t\tenclosing: enclosing;\n\t\tenclosingEscape: enclosingEscape;\n        enoding: encoding;\n\t}\n}\n",
    "builtin:///stdlib/CSVFileInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* A CSVFileInterpreter interprets a file as CSV\n*/\ncomposite blocktype CSVFileInterpreter {\n    property delimiter oftype text: ',';\n    property enclosing oftype text: '';\n    property enclosingEscape oftype text: '';\n    property enoding oftype text: 'utf-8';\n\n    input inputPort oftype File;\n    output outputPort oftype Sheet;\n\n    inputPort\n        -> FileTextInterpreter\n        -> FileCSVInterpreter\n        -> outputPort;\n\n    block FileTextInterpreter oftype TextFileInterpreter {\n        encoding: enoding;\n    }\n\n    block FileCSVInterpreter oftype CSVInterpreter {\n\t    delimiter: delimiter;\n\t    enclosing: enclosing;\n\t    enclosingEscape: enclosingEscape;\n    }\n}\n",
    "builtin:///stdlib/GTFSExtractor.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* A GTFSExtractor extracts a file from a given URL, interprets it as ZIP and extracts it\n*/\ncomposite blocktype GTFSExtractor {\n    property url oftype text;\n\n    input inputPort oftype None;\n    output outputPort oftype FileSystem;\n\n    inputPort\n        -> FileExtractor\n        -> ZipArchiveInterpreter\n        -> outputPort;\n\n    block FileExtractor oftype HttpExtractor { url: url; }\n\n\tblock ZipArchiveInterpreter oftype ArchiveInterpreter { archiveType: \"zip\"; }\n}\n",
    "builtin:///stdlib/percent.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\nvaluetype Percent oftype decimal {\n    constraints: [ZeroToHundredDecimal];\n}\nconstraint ZeroToHundredDecimal on decimal: value >= 0 and value <= 100;",
    "builtin:///stdlib/builtin-blocktypes/ArchiveInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Interprets a `File` as an archive file and converts it to a `FileSystem`. The archive file root is considered the root of the `FileSystem`.\n* \n* @example Interprets a `File` as a ZIP-archive and creates a `FileSystem` of its extracted contents.\n* block ZipArchiveInterpreter oftype ArchiveInterpreter {\n*   archiveType: \"zip\";\n* }\n*/\nbuiltin blocktype ArchiveInterpreter {\n\tinput default oftype File;\n\toutput default oftype FileSystem;\n\t\n\t/**\n\t* The archive type to be interpreted, e.g., \"zip\" or \"gz\". \n\t*/\n\tproperty archiveType oftype text;\n}\n",
    "builtin:///stdlib/builtin-blocktypes/CellRangeSelector.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Selects a subset of a `Sheet` to produce a new `Sheet`.\n* \n* @example Selects the cells in the given range and produces a new `Sheet` containing only the selected cells.\n* block CarsCoreDataSelector oftype CellRangeSelector {\n*   select: range A1:E*;\n* }\n*/\nbuiltin blocktype CellRangeSelector {\n\tinput default oftype Sheet;\n\toutput default oftype Sheet;\n\t\n\t/**\n\t* The cell range to select.\n\t*/\n\tproperty select oftype CellRange;\n}",
    "builtin:///stdlib/builtin-blocktypes/CellWriter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Writes textual values into cells of a `Sheet`. The number of text values needs to match the number of cells to write into.\n* \n* @example Write the value \"Name\" into cell `A1`.\n* block NameHeaderWriter oftype CellWriter {\n*   at: cell A1;\n*   write: [\"Name\"];\n* }\n* \n* @example Write the values \"Name\", \"Age\" into cells `A1` and `A2`.\n* block HeaderSequenceWriter oftype CellWriter {\n*   at: range A1:A2;\n*   write: [\"Name\", \"Age\"];\n* }\n*/\nbuiltin blocktype CellWriter {\n\tinput default oftype Sheet;\n\toutput default oftype Sheet;\n\t\n\t/** \n\t* The values to write.\n\t*/\n\tproperty write oftype Collection<text>;\n\t\n\t/**\n\t* The cells to write into.\n\t*/\n\tproperty at oftype CellRange;\n}",
    "builtin:///stdlib/builtin-blocktypes/ColumnDeleter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Deletes columns from a `Sheet`. Column IDs of subsequent columns will be shifted accordingly, so there will be no gaps.\n* \n* @example Deletes column B (i.e. the second column).\n* block MpgColumnDeleter oftype ColumnDeleter {\n*   delete: [column B];\n* }\n*/\nbuiltin blocktype ColumnDeleter {\n\tinput default oftype Sheet;\n\toutput default oftype Sheet;\n\t\n\t/** \n\t* The columns to delete. Has to be a full column.\n\t*/\n\tproperty delete oftype Collection<CellRange>;\n}",
    "builtin:///stdlib/builtin-blocktypes/CsvInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Interprets an input file as a csv-file containing string-values delimited by `delimiter` and outputs a `Sheet`.\n* \n* @example Interprets an input file as a csv-file containing string-values delimited by `;` and outputs `Sheet`.\n* block AgencyCSVInterpreter oftype CSVInterpreter {  \n* \tdelimiter: \";\";\n* }\n*/\nbuiltin blocktype CSVInterpreter {\n\tinput default oftype TextFile;\n\toutput default oftype Sheet;\n\t\n\t/**\n\t* The delimiter for values in the CSV file.\n\t*/\n\tproperty delimiter oftype text: ',';\n\t\n\t/**\n\t* The enclosing character that may be used for values in the CSV file.\n\t*/\n\tproperty enclosing oftype text: '';\n\t\n\t/**\n\t* The character to escape enclosing characters in values.\n\t*/\n\tproperty enclosingEscape oftype text: '';\n}",
    "builtin:///stdlib/builtin-blocktypes/FilePicker.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Selects one `File` from a `FileSystem` based on its relative path to the root of the `FileSystem`. If no file matches the relative path, no output is created and the execution of the pipeline is aborted.\n* \n* @example Tries to pick the file `agency.txt` from the root of the provided `FileSystem`. If `agency.txt` exists it is passed on as `File`, if it does not exist the execution of the pipeline is aborted.\n* block AgencyFilePicker oftype FilePicker {\n*   path: \"./agency.txt\";\n* }\n*/\nbuiltin blocktype FilePicker {\n\tinput default oftype FileSystem;\n\toutput default oftype File;\n\t\n\t/** \n\t* The path of the file to select, relative to the root of the provided `FileSystem`.\n\t*/\n\tproperty path oftype text;\n}",
    "builtin:///stdlib/builtin-blocktypes/GtfsRtInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Interprets an protobuf file (binary) of type `File` by decoding the file according to `gtfs-realtime.proto`. Outputs the extracted entity defined by `entity` as a `Sheet`\n* \n* @example A file is interpretet as an GTFS-RT file, which contains TripUpdate.\n* block GtfsRTTripUpdateInterpreter oftype GtfsRTInterpreter{\n*   entity: \"trip_update\";\n* }\n*/\nbuiltin blocktype GtfsRTInterpreter {\n\tinput default oftype File;\n\toutput default oftype Sheet;\n\t\n\t/**\n\t* Entity to process from GTFS-RT-feed (`trip_update`, `alert` or `vehicle`).\n\t* \n\t* We currently support following Output-Sheets, each are an equivalent to the flattened Element Index defined in <https://developers.google.com/transit/gtfs-realtime/reference#element-index> (just required fields are included):\n\t* \n\t* Entity TripUpdate:\n\t* ```\n\t* [\n\t* \t'header.gtfs_realtime_version',\n\t* \t'header.timestamp',\n\t* \t'header.incrementality',\n\t* \t'entity.id',\n\t* \t'entity.trip_update.trip.trip_id',\n\t* \t'entity.trip_update.trip.route_id',\n\t* \t'entity.trip_update.stop_time_update.stop_sequence',\n\t* \t'entity.trip_update.stop_time_update.stop_id',\n\t* \t'entity.trip_update.stop_time_update.arrival.time',\n\t* \t'entity.trip_update.stop_time_update.departure.time',\n\t* ];\n\t* \n\t* ```\n\t* Entity VehiclePosition:\n\t* ```\n\t* [\n\t* \t'header.gtfs_realtime_version',\n\t* \t'header.timestamp',\n\t* \t'header.incrementality',\n\t* \t'entity.id',\n\t* \t'entity.vehicle_position.vehicle_descriptor.id',\n\t* \t'entity.vehicle_position.trip.trip_id',\n\t* \t'entity.vehicle_position.trip.route_id',\n\t* \t'entity.vehicle_position.position.latitude',\n\t* \t'entity.vehicle_position.position.longitude',\n\t* \t'entity.vehicle_position.timestamp',\n\t* ];\n\t* ```\n\t* \n\t* Entity Alert:\n\t* ```\n\t* [\n\t* \t'header.gtfs_realtime_version',\n\t* \t'header.timestamp',\n\t* \t'header.incrementality',\n\t* \t'entity.id',\n\t* \t'entity.alert.informed_entity.route_id',\n\t* \t'entity.alert.header_text',\n\t* \t'entity.alert.description_text',\n\t* ];\n\t* ```\n\t*/\n\tproperty entity oftype text;\n}\n",
    "builtin:///stdlib/builtin-blocktypes/HttpExtractor.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Extracts a `File` from the web.\n* \n* @example Fetches a file from the given URL.\n* block CarsFileExtractor oftype HttpExtractor {\n*   url: \"tinyurl.com/4ub9spwz\";\n* }\n*/\nbuiltin blocktype HttpExtractor {\n\tinput default oftype None;\n\toutput default oftype File;\n\t\n\t/**\n\t* The URL to the file in the web to extract.\n\t*/\n\tproperty url oftype text;\n\t\n\t/**\n\t* Configures how many retries should be executed after a failure fetching the data.\n\t*/\n\tproperty retries oftype integer: 0;\n\t\n\t/**\n\t* Configures the wait time in milliseconds before executing a retry.\n\t*/\n\tproperty retryBackoffMilliseconds oftype integer: 1000;\n\t\n\t/**\n\t* Configures the wait strategy before executing a retry. Can have values \"exponential\" or \"linear\".\n\t*/\n\tproperty retryBackoffStrategy oftype text: \"exponential\";\n\t\n\t/**\n\t* Indicates, whether to follow redirects on get requests. If `false`, redirects are not followed. Default `true`\n\t*/\n\tproperty followRedirects oftype boolean: true;\n}\n",
    "builtin:///stdlib/builtin-blocktypes/PostgresLoader.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Loads a `Table` into a PostgreSQL database sink.\n* \n* @example A local Postgres instance is filled with table data about cars.\n* block CarsLoader oftype PostgresLoader {\n*   host: \"localhost\";\n*   port: 5432;\n*   username: \"postgres\";\n*   password: \"postgres\";\n*   database: \"CarsDB\";\n*   table: \"Cars\";\n* }\n*/\nbuiltin blocktype PostgresLoader {\n\tinput default oftype Table;\n\toutput default oftype None;\n\t\n\t/**\n\t* The hostname or IP address of the Postgres database.\n\t*/\n\tproperty host oftype text;\n\t\n\t/**\n\t* The port of the Postgres database.\n\t*/\n\tproperty port oftype integer;\n\t\n\t/**\n\t* The username to login to the Postgres database.\n\t*/\n\tproperty username oftype text;\n\t\n\t/**\n\t* The password to login to the Postgres database.\n\t*/\n\tproperty password oftype text;\n\t\n\t/**\n\t* The database to use.\n\t*/\n\tproperty database oftype text;\n\t\n\t/**\n\t* The name of the table to write into.\n\t*/\n\tproperty table oftype text;\n}",
    "builtin:///stdlib/builtin-blocktypes/RowDeleter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Deletes one or more rows from a `Sheet`. Row IDs of subsequent rows will be shifted accordingly, so there will be no gaps.\n* \n* @example Deletes row 2 (i.e. the second row).\n* block SecondRowDeleter oftype RowDeleter {\n*   delete: [row 2];\n* }\n*/\nbuiltin blocktype RowDeleter {\n\tinput default oftype Sheet;\n\toutput default oftype Sheet;\n\t\n\t/**\n\t* The rows to delete.\n\t*/\n\tproperty delete oftype Collection<CellRange>;\n}",
    "builtin:///stdlib/builtin-blocktypes/SheetPicker.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Selects one `Sheet` from a `Workbook` based on its `sheetName`. If no sheet matches the name, no output is created and the execution of the pipeline is aborted.\n* \n* @example Tries to pick the sheet `AgencyNames` from the provided `Workbook`. If `AgencyNames` exists it is passed on as `Sheet`, if it does not exist the execution of the pipeline is aborted.\n* block AgencySheetPicker oftype SheetPicker {\n*   sheetName: \"AgencyNames\";\n* }\n*/\nbuiltin blocktype SheetPicker {\n\tinput default oftype Workbook;\n\toutput default oftype Sheet;\n\t\n\t/**\n\t* The name of the sheet to select.\n\t*/\n\tproperty sheetName oftype text;\n}",
    "builtin:///stdlib/builtin-blocktypes/SqliteLoader.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Loads a `Table` into a SQLite database sink.\n* \n* @example A SQLite file `cars.db` is created in the working directory. Incoming data is written to the table `cars`.\n* block CarsLoader oftype SQLiteLoader {\n*   table: \"cars\";\n*   file: \"./cars.db\";\n* }\n*/\nbuiltin blocktype SQLiteLoader {\n\tinput default oftype Table;\n\toutput default oftype None;\n\t\n\t/**\n\t* The name of the table to write into.\n\t*/\n\tproperty table oftype text;\n\t\n\t/**\n\t* The path to a SQLite file that will be created if it does not exist. Usual file extensions are `.sqlite` and `.db`.\n\t*/\n\tproperty file oftype text;\n\t\n\t/**\n\t* Indicates, whether to drop the table before loading data into it. If `false`, data is appended to the table instead of dropping it.\n\t*/\n\tproperty dropTable oftype boolean: true;\n}",
    "builtin:///stdlib/builtin-blocktypes/TableInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Interprets a `Sheet` as a `Table`. In case a header row is present in the sheet, its names can be matched with the provided column names. Otherwise, the provided column names are assigned in order.\n* \n* @example Interprets a `Sheet` about cars with a topmost header row and interprets it as a `Table` by assigning a primitive valuetype to each column. The column names are matched to the header, so the order of the type assignments does not matter.\n* block CarsTableInterpreter oftype TableInterpreter {\n*   header: true;\n*   columns: [\n*     \"name\" oftype text,\n*     \"mpg\" oftype decimal,\n*     \"cyl\" oftype integer,\n*   ];\n* }\n* \n* @example Interprets a `Sheet` about cars without a topmost header row and interprets it as a `Table` by sequentially assigning a name and a primitive valuetype to each column of the sheet. Note that the order of columns matters here. The first column (column `A`) will be named \"name\", the second column (column `B`) will be named \"mpg\" etc.\n* block CarsTableInterpreter oftype TableInterpreter {\n*   header: false;\n*   columns: [\n*     \"name\" oftype text,\n*     \"mpg\" oftype decimal,\n*     \"cyl\" oftype integer,\n*   ];\n* }\n*/\nbuiltin blocktype TableInterpreter {\n\tinput default oftype Sheet;\n\toutput default oftype Table;\n\t\n  /**\n  * Whether the first row should be interpreted as header row.\n\t*/\n  property header oftype boolean: true;\n\t\n  /**\n  * Collection of valuetype assignments. Uses column names (potentially matched with the header or by sequence depending on the `header` property) to assign a primitive valuetype to each column.\n\t*/\n  property columns oftype Collection<ValuetypeAssignment>;\n}",
    "builtin:///stdlib/builtin-blocktypes/TableTransformer.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Applies a transform on each value of a column. The input port type of the used transform has to match the type of the input column.\n* \n* @example Given a column \"temperature\" with temperature values in Celsius, it overwrites the column with computed values in Fahrenheit by using the `CelsiusToFahrenheit` transform. The transform itself is defined elsewhere in the model.\n* \n* transform CelsiusToFahrenheit {\n*   from Celsius oftype decimal;\n*   to Fahrenheit oftype decimal;\n* \n*   Fahrenheit: (Celsius * 9/5) + 32;\n* }\n* \n* block CelsiusToFahrenheitTransformer oftype TableTransformer {\n*   inputColumns: ['temperature'];\n*   outputColumn: 'temperature';\n*   use: CelsiusToFahrenheit;\n* }\n* \n* @example Given a column \"temperatureCelsius\" with temperature values in Celsius, it adds a new column \"temperatureFahrenheit\" with computed values in Fahrenheit by using the `CelsiusToFahrenheit` transform. The transform itself is defined elsewhere in the model.\n* \n* transform CelsiusToFahrenheit {\n*   from Celsius oftype decimal;\n*   to Fahrenheit oftype decimal;\n* \n*   Fahrenheit: (Celsius * 9/5) + 32;\n* }\n* \n* block CelsiusToFahrenheitTransformer oftype TableTransformer {\n*   inputColumns: ['temperatureCelsius'];\n*   outputColumn: 'temperatureFahrenheit';\n*   use: CelsiusToFahrenheit;\n* }\n*/\nbuiltin blocktype TableTransformer {\n\tinput default oftype Table;\n\toutput default oftype Table;\n\t\n  /**\n  * The names of the input columns. The columns have to be present in the table and match with the transform's input port types.\n\t*/\n  property inputColumns oftype Collection<text>;\n\t\n  /**\n  * The name of the output column. Overwrites the column if it already exists, or otherwise creates a new one.\n\t*/\n  property outputColumn oftype text;\n\t\n  /**\n  * Reference to the transform that is applied to the column.\n\t*/\n  property use oftype Transform; \n}",
    "builtin:///stdlib/builtin-blocktypes/TextFileInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Interprets a `File` as a `TextFile`.\n*/\nbuiltin blocktype TextFileInterpreter {\n\tinput default oftype File;\n\toutput default oftype TextFile;\n\t\n\t/**\n\t* The encoding used for decoding the file contents.\n\t*/\n\tproperty encoding oftype text: 'utf-8';\n\t\n\t/**\n\t* The regex for identifying line breaks.\n\t*/\n\tproperty lineBreak oftype Regex: /\\r?\\n/;\n}\n",
    "builtin:///stdlib/builtin-blocktypes/TextLineDeleter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Deletes individual lines from a `TextFile`.\n*/\nbuiltin blocktype TextLineDeleter {\n\tinput default oftype TextFile;\n\toutput default oftype TextFile;\n\t\n\t/**\n\t* The line numbers to delete.\n\t*/\n\tproperty lines oftype Collection<integer>;\n}\n",
    "builtin:///stdlib/builtin-blocktypes/TextRangeSelector.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Selects a range of lines from a `TextFile`.\n*/\nbuiltin blocktype TextRangeSelector {\n\tinput default oftype TextFile;\n\toutput default oftype TextFile;\n\t\n\t/**\n\t* Inclusive beginning line number for the selection.\n\t*/\n\tproperty lineFrom oftype integer: 1;\n\t\n\t/**\n\t* Inclusive ending line number for the selection.\n\t* The default value is the biggest usable integer.\n\t*/\n\tproperty lineTo oftype integer: 9007199254740991;\n}",
    "builtin:///stdlib/builtin-blocktypes/XlsInterpreter.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Interprets an input file as a XLSX-file and outputs a `Workbook` containing `Sheet`s.\n* \n* @example Interprets an input file as a XLSX-file and outputs a `Workbook` containing `Sheet`s.\n* block AgencyXLSXInterpreter oftype XLSXInterpreter { }\n*/\nbuiltin blocktype XLSXInterpreter {\n\tinput default oftype File;\n\toutput default oftype Workbook;\n}",
    "builtin:///stdlib/builtin-constrainttypes/AllowlistConstraint.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Limits the values to a defined a set of allowed values. Only values in the list are valid.\n* \n* @example Only allows the common abbreviations for millisecond, second, minute, etc..\n* constraint TimeUnitString oftype AllowlistConstraint {\n*   allowlist: [\"ms\", \"s\", \"min\", \"h\", \"d\", \"m\", \"y\"];\n* }\n*/\nbuiltin constrainttype AllowlistConstraint on text {\n\tproperty allowlist oftype Collection<text> ;\n}",
    "builtin:///stdlib/builtin-constrainttypes/DenylistConstraint.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Defines a set of forbidden values. All values in the list are considered invalid.\n*\n* @example Denies all primary colors.\n* constraint NoPrimaryColors oftype DenylistConstraint {\n*   denylist: [\"red\", \"blue\", \"yellow\"];\n* }\n*/\nbuiltin constrainttype DenylistConstraint on text {\n\tproperty denylist oftype Collection<text> ;\n}",
    "builtin:///stdlib/builtin-constrainttypes/LengthConstraint.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Limits the length of a string with an upper and/or lower boundary. \n* Only values with a length within the given range are valid.\n*\n* @example A text constraint with 0 to 20 characters.\n* constraint ShortAnswerConstraint oftype LengthConstraint {\n*   minLength: 0;\n*   maxLength: 20;\n* }\n*/\nbuiltin constrainttype LengthConstraint on text {\n\t/**\n    * Inclusive minimum of the valid text length.\n\t*/\n\tproperty minLength oftype integer : 0;\n\n\t/**\n    * Inclusive maximum of the valid text length.\n\t* The default value is the biggest usable integer.\n\t*/\n\tproperty maxLength oftype integer : 9007199254740991;\n}",
    "builtin:///stdlib/builtin-constrainttypes/RangeConstraint.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Limits the range of a number value with an upper and/or lower boundary which can be inclusive or exclusive. Only values within the given range are considered valid.\n*\n* @example A scale between (and including) 1 and 100.\n* constraint HundredScale oftype RangeConstraint {\n*   lowerBound: 1;\n*   upperBound: 100;\t\t\n* }\n*\n* @example A scale for numbers strictly larger than 1 and less or equal to 100.\n* constraint HundredScale oftype RangeConstraint {\n*   lowerBound: 1;\n*   lowerBoundInclusive: false;\n*   upperBound: 100;\t\t\n* }\n*/\nbuiltin constrainttype RangeConstraint on decimal {\n\t/**\n\t* Lower bound for the valid value range.\n\t* @see lowerBoundInclusive whether inclusive or exclusive.\n\t* The default value is the smallest usable integer.\n\t*/\n\tproperty lowerBound oftype decimal : -9007199254740991;\n\n\t/**\n\t* Boolean flag whether @see lowerBound is inclusive or exclusive.\n\t*/\n\tproperty lowerBoundInclusive oftype boolean : true;\n\n\t/**\n\t* Upper bound for the valid value range.\n\t* @see upperBoundInclusive whether inclusive or exclusive.\n\t* The default value is the biggest usable integer.\n\t*/\n\tproperty upperBound oftype decimal : 9007199254740991;\n\n\t/**\n\t* Boolean flag whether @see upperBound is inclusive or exclusive.\n\t*/\n\tproperty upperBoundInclusive oftype boolean : true;\n}",
    "builtin:///stdlib/builtin-constrainttypes/RegexConstraint.jv": "// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg\n//\n// SPDX-License-Identifier: AGPL-3.0-only\n\n/**\n* Limits the values complying with a regex. \n* Only values that comply with the regex are considered valid.\n*\n* @example Text that complies with the IPv4 address format.\n* constraint IPv4Format oftype RegexConstraint {\n*   regex: /^((25[0-5]|(2[0-4]|1\\d|[1-9]|)\\d)\\.?\\b){4}$/;\n* }\n*/\nbuiltin constrainttype RegexConstraint on text {\n\tproperty regex oftype Regex ;\n}"
};


/***/ }),

/***/ "./libs/language-server/src/lib/builtin-library/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getStdLib = void 0;
const tslib_1 = __webpack_require__("tslib");
var stdlib_1 = __webpack_require__("./libs/language-server/src/lib/builtin-library/stdlib.ts");
Object.defineProperty(exports, "getStdLib", ({ enumerable: true, get: function () { return stdlib_1.getStdLib; } }));
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/builtin-library/jayvee-workspace-manager.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/builtin-library/jayvee-workspace-manager.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.initializeWorkspace = exports.JayveeWorkspaceManager = void 0;
const tslib_1 = __webpack_require__("tslib");
const langium_1 = __webpack_require__("langium");
const vscode_uri_1 = __webpack_require__("vscode-uri");
const stdlib_1 = __webpack_require__("./libs/language-server/src/lib/builtin-library/stdlib.ts");
class JayveeWorkspaceManager extends langium_1.DefaultWorkspaceManager {
    constructor(services) {
        super(services);
        this.documentFactory = services.workspace.LangiumDocumentFactory;
    }
    loadAdditionalDocuments(folders, collector) {
        const _super = Object.create(null, {
            loadAdditionalDocuments: { get: () => super.loadAdditionalDocuments }
        });
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield _super.loadAdditionalDocuments.call(this, folders, collector);
            Object.entries((0, stdlib_1.getStdLib)()).forEach(([libName, libCode]) => {
                collector(this.documentFactory.fromString(libCode, vscode_uri_1.URI.parse(libName)));
            });
        });
    }
}
exports.JayveeWorkspaceManager = JayveeWorkspaceManager;
/**
 * Initializes the workspace with all workspace folders.
 * Also loads additional required files, e.g., the standard library
 */
function initializeWorkspace(services, workspaceFolders = []) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield services.shared.workspace.WorkspaceManager.initializeWorkspace(workspaceFolders);
    });
}
exports.initializeWorkspace = initializeWorkspace;


/***/ }),

/***/ "./libs/language-server/src/lib/builtin-library/stdlib.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.getStdLib = exports.IOtypesLib = exports.getBuiltinValuetypesLib = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const partial_stdlib_1 = __webpack_require__("./libs/language-server/src/lib/builtin-library/generated/partial-stdlib.ts");
function getBuiltinValuetypesLib() {
    const primitiveValuetypes = Object.values(primitive_valuetypes_1.PrimitiveValuetypes).map(parseBuiltinValuetypeToJayvee);
    const collectionValuetype = `${parseAsComment('For internal use only.')}
builtin valuetype Collection<ElementType>;`;
    return {
        'builtin:///stdlib/builtin-valuetypes.jv': [
            ...primitiveValuetypes,
            collectionValuetype,
        ].join('\n\n'),
    };
}
exports.getBuiltinValuetypesLib = getBuiltinValuetypesLib;
exports.IOtypesLib = {
    'builtin:///stdlib/iotypes.jv': Object.values(ast_1.IOType)
        .map((iotype) => `builtin iotype ${iotype};`)
        .join('\n\n'),
};
function getStdLib() {
    return Object.assign(Object.assign(Object.assign({}, partial_stdlib_1.PartialStdLib), getBuiltinValuetypesLib()), exports.IOtypesLib);
}
exports.getStdLib = getStdLib;
function parseBuiltinValuetypeToJayvee(valuetype) {
    const lines = [];
    const userDoc = valuetype.getUserDoc();
    if (userDoc !== undefined) {
        lines.push(parseAsComment(userDoc));
    }
    if (!valuetype.isReferenceableByUser()) {
        lines.push(parseAsComment('For internal use only.'));
    }
    lines.push(`builtin valuetype ${valuetype.getName()};`);
    return lines.join('\n');
}
function parseAsComment(text, indents = 0) {
    return text
        .split('\n')
        .map((l) => `// ${l}`)
        .map((l) => '\t'.repeat(indents) + l)
        .join('\n');
}


/***/ }),

/***/ "./libs/language-server/src/lib/completion/jayvee-completion-provider.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeCompletionProvider = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const vscode_languageserver_1 = __webpack_require__("vscode-languageserver");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const ast_2 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const lsp_doc_generator_1 = __webpack_require__("./libs/language-server/src/lib/docs/lsp-doc-generator.ts");
const RIGHT_ARROW_SYMBOL = '\u{2192}';
class JayveeCompletionProvider extends langium_1.DefaultCompletionProvider {
    constructor(services) {
        super(services);
        this.langiumDocumentService = services.shared.workspace.LangiumDocuments;
    }
    completionFor(context, next, acceptor) {
        const astNode = context.node;
        if (astNode !== undefined) {
            const isBlockTypeCompletion = (0, ast_2.isBlockDefinition)(astNode) && next.property === 'type';
            if (isBlockTypeCompletion) {
                return this.completionForBlockType(acceptor);
            }
            const isConstraintTypeCompletion = (0, ast_2.isConstraintDefinition)(astNode) && next.property === 'type';
            if (isConstraintTypeCompletion) {
                return this.completionForConstraintType(acceptor);
            }
            const isValuetypeDefinitionCompletion = next.type === ast_2.ValuetypeReference;
            if (isValuetypeDefinitionCompletion) {
                return this.completionForValuetype(context, acceptor);
            }
            const isFirstPropertyCompletion = (0, ast_2.isPropertyBody)(astNode) && next.type === ast_2.PropertyAssignment;
            const isOtherPropertyCompletion = (0, ast_2.isPropertyAssignment)(astNode) && next.type === ast_2.PropertyAssignment;
            if (isFirstPropertyCompletion || isOtherPropertyCompletion) {
                return this.completionForPropertyName(astNode, acceptor);
            }
        }
        return super.completionFor(context, next, acceptor);
    }
    completionForBlockType(acceptor) {
        const blockTypes = (0, model_util_1.getAllBuiltinBlocktypes)(this.langiumDocumentService);
        blockTypes.forEach((blockType) => {
            const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
            const markdownDoc = lspDocBuilder.generateBlockTypeDoc(blockType);
            acceptor({
                label: blockType.type,
                labelDetails: {
                    detail: ` ${blockType.inputType} ${RIGHT_ARROW_SYMBOL} ${blockType.outputType}`,
                },
                kind: vscode_languageserver_1.CompletionItemKind.Class,
                detail: `(block type)`,
                documentation: {
                    kind: 'markdown',
                    value: markdownDoc,
                },
            });
        });
    }
    completionForConstraintType(acceptor) {
        const constraintTypes = (0, model_util_1.getAllBuiltinConstraintTypes)(this.langiumDocumentService);
        constraintTypes.forEach((constraintType) => {
            const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
            const markdownDoc = lspDocBuilder.generateConstraintTypeDoc(constraintType);
            acceptor({
                label: constraintType.type,
                labelDetails: {
                    detail: ` on ${constraintType.on.getName()}`,
                },
                kind: vscode_languageserver_1.CompletionItemKind.Class,
                detail: `(constraint type)`,
                documentation: {
                    kind: 'markdown',
                    value: markdownDoc,
                },
            });
        });
    }
    completionForValuetype(context, acceptor) {
        this.langiumDocumentService.all
            .map((document) => document.parseResult.value)
            .forEach((parsedDocument) => {
            if (!(0, ast_2.isJayveeModel)(parsedDocument)) {
                throw new Error('Expected parsed document to be a JayveeModel');
            }
            parsedDocument.valuetypes.forEach((valuetypeDefinition) => {
                const valuetype = (0, ast_1.createValuetype)(valuetypeDefinition);
                if (valuetype !== undefined && valuetype.isReferenceableByUser()) {
                    acceptor({
                        label: valuetypeDefinition.name,
                        kind: vscode_languageserver_1.CompletionItemKind.Class,
                        detail: `(valuetype)`,
                    });
                }
            });
        });
    }
    completionForPropertyName(astNode, acceptor) {
        let container;
        if ((0, ast_2.isPropertyBody)(astNode)) {
            container = astNode.$container;
        }
        else {
            container = astNode.$container.$container;
        }
        const wrapper = (0, ast_1.getTypedObjectWrapper)(container.type);
        if (wrapper === undefined) {
            return;
        }
        const presentPropertyNames = container.body.properties.map((attr) => attr.name);
        const propertyKinds = [
            'required',
            'optional',
        ];
        for (const propertyKind of propertyKinds) {
            const propertyNames = wrapper.getPropertyNames(propertyKind, presentPropertyNames);
            this.constructPropertyCompletionValueItems(wrapper, propertyNames, propertyKind).forEach(acceptor);
        }
    }
    constructPropertyCompletionValueItems(wrapper, propertyNames, kind) {
        return propertyNames.map((propertyName) => {
            const propertySpec = wrapper.getPropertySpecification(propertyName);
            (0, assert_1.strict)(propertySpec !== undefined);
            const completionValueItem = {
                label: propertyName,
                labelDetails: {
                    detail: ` ${propertySpec.type.getName()}`,
                },
                kind: vscode_languageserver_1.CompletionItemKind.Field,
                detail: `(${kind} property)`,
                sortText: kind === 'required' ? '1' : '2',
            };
            if (propertySpec.defaultValue !== undefined) {
                const defaultValueString = JSON.stringify(propertySpec.defaultValue);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                completionValueItem.labelDetails.detail += ` = ${defaultValueString}`;
            }
            const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
            const markdownDoc = lspDocBuilder.generatePropertyDoc(wrapper, propertyName);
            if (markdownDoc !== undefined) {
                completionValueItem.documentation = {
                    kind: 'markdown',
                    value: markdownDoc,
                };
            }
            return completionValueItem;
        });
    }
}
exports.JayveeCompletionProvider = JayveeCompletionProvider;


/***/ }),

/***/ "./libs/language-server/src/lib/docs/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/jayvee-doc-generator.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/lsp-doc-generator.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/markdown-builder.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/docs/jayvee-doc-generator.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/language-server/src/lib/docs/lsp-doc-generator.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.LspDocGenerator = void 0;
const markdown_builder_1 = __webpack_require__("./libs/language-server/src/lib/docs/markdown-builder.ts");
class LspDocGenerator {
    generateBlockTypeDoc(blockType) {
        const markdownBuilder = new markdown_builder_1.MarkdownBuilder();
        return markdownBuilder.line(blockType.docs.description).build();
    }
    generateConstraintTypeDoc(constraintType) {
        const markdownBuilder = new markdown_builder_1.MarkdownBuilder();
        return markdownBuilder.line(constraintType.docs.description).build();
    }
    generatePropertyDoc(wrapper, propertyName) {
        var _a, _b;
        const markdownBuilder = new markdown_builder_1.MarkdownBuilder();
        const propertySpec = wrapper.getPropertySpecification(propertyName);
        if (propertySpec === undefined) {
            return undefined;
        }
        return markdownBuilder
            .line((_a = propertySpec.docs) === null || _a === void 0 ? void 0 : _a.description)
            .newLine()
            .line((_b = propertySpec.docs) === null || _b === void 0 ? void 0 : _b.validation)
            .build();
    }
}
exports.LspDocGenerator = LspDocGenerator;


/***/ }),

/***/ "./libs/language-server/src/lib/docs/markdown-builder.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.MarkdownBuilder = void 0;
class MarkdownBuilder {
    constructor() {
        this.markdownTextLines = [];
    }
    getHeaderPrefix(depth) {
        return '#'.repeat(depth);
    }
    heading(title, depth) {
        this.markdownTextLines.push(`${this.getHeaderPrefix(depth)} ${title}`);
        this.newLine();
        return this;
    }
    line(text) {
        if (text !== undefined) {
            this.markdownTextLines.push(text);
        }
        return this;
    }
    code(code, language) {
        return this.line('```' + (language !== null && language !== void 0 ? language : ''))
            .line(code)
            .line('```')
            .newLine();
    }
    newLine() {
        this.markdownTextLines.push('');
        return this;
    }
    comment(comment) {
        this.markdownTextLines.push(`<!-- ${comment} -->`);
        return this;
    }
    build() {
        return this.markdownTextLines.join('\n');
    }
}
exports.MarkdownBuilder = MarkdownBuilder;


/***/ }),

/***/ "./libs/language-server/src/lib/hover/jayvee-hover-provider.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeHoverProvider = void 0;
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const lsp_doc_generator_1 = __webpack_require__("./libs/language-server/src/lib/docs/lsp-doc-generator.ts");
class JayveeHoverProvider extends langium_1.AstNodeHoverProvider {
    getAstNodeHoverContent(astNode) {
        let doc = undefined;
        if ((0, ast_1.isBuiltinBlocktypeDefinition)(astNode)) {
            doc = this.getBlockTypeMarkdownDoc(astNode);
        }
        if ((0, ast_1.isBuiltinConstrainttypeDefinition)(astNode)) {
            doc = this.getConstraintTypeMarkdownDoc(astNode);
        }
        if ((0, ast_1.isPropertyAssignment)(astNode)) {
            doc = this.getPropertyMarkdownDoc(astNode);
        }
        if (doc === undefined) {
            return undefined;
        }
        const hover = {
            contents: {
                kind: 'markdown',
                value: doc,
            },
        };
        return hover;
    }
    getBlockTypeMarkdownDoc(blockTypeDefinition) {
        if (!ast_1.BlockTypeWrapper.canBeWrapped(blockTypeDefinition)) {
            return;
        }
        const blockType = new ast_1.BlockTypeWrapper(blockTypeDefinition);
        const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
        return lspDocBuilder.generateBlockTypeDoc(blockType);
    }
    getConstraintTypeMarkdownDoc(constraintTypeDefinition) {
        if (!ast_1.ConstraintTypeWrapper.canBeWrapped(constraintTypeDefinition)) {
            return;
        }
        const constraintType = new ast_1.ConstraintTypeWrapper(constraintTypeDefinition);
        const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
        return lspDocBuilder.generateConstraintTypeDoc(constraintType);
    }
    getPropertyMarkdownDoc(property) {
        const container = property.$container.$container;
        const wrapper = (0, ast_1.getTypedObjectWrapper)(container.type);
        if (wrapper === undefined) {
            return;
        }
        const lspDocBuilder = new lsp_doc_generator_1.LspDocGenerator();
        return lspDocBuilder.generatePropertyDoc(wrapper, property.name);
    }
}
exports.JayveeHoverProvider = JayveeHoverProvider;


/***/ }),

/***/ "./libs/language-server/src/lib/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/ast/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/builtin-library/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/docs/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/services/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/util/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/index.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/jayvee-module.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/jayvee-module.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.createJayveeServices = exports.JayveeSharedModule = exports.JayveeModule = void 0;
const langium_1 = __webpack_require__("langium");
const module_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/module.ts");
const jayvee_workspace_manager_1 = __webpack_require__("./libs/language-server/src/lib/builtin-library/jayvee-workspace-manager.ts");
const jayvee_completion_provider_1 = __webpack_require__("./libs/language-server/src/lib/completion/jayvee-completion-provider.ts");
const jayvee_hover_provider_1 = __webpack_require__("./libs/language-server/src/lib/hover/jayvee-hover-provider.ts");
const jayvee_value_converter_1 = __webpack_require__("./libs/language-server/src/lib/jayvee-value-converter.ts");
const runtime_parameter_provider_1 = __webpack_require__("./libs/language-server/src/lib/services/runtime-parameter-provider.ts");
const validation_registry_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-registry.ts");
/**
 * Dependency injection module that overrides Langium default services and contributes the
 * declared custom services. The Langium defaults can be partially specified to override only
 * selected services, while the custom services must be fully specified.
 */
exports.JayveeModule = {
    parser: {
        ValueConverter: () => new jayvee_value_converter_1.JayveeValueConverter(),
    },
    validation: {
        ValidationRegistry: (services) => new validation_registry_1.JayveeValidationRegistry(services),
    },
    lsp: {
        CompletionProvider: (services) => new jayvee_completion_provider_1.JayveeCompletionProvider(services),
        HoverProvider: (services) => new jayvee_hover_provider_1.JayveeHoverProvider(services),
    },
    RuntimeParameterProvider: () => new runtime_parameter_provider_1.RuntimeParameterProvider(),
};
exports.JayveeSharedModule = {
    workspace: {
        WorkspaceManager: (services) => new jayvee_workspace_manager_1.JayveeWorkspaceManager(services),
    },
};
/**
 * Create the full set of services required by Langium.
 *
 * First inject the shared services by merging two modules:
 *  - Langium default shared services
 *  - Services generated by langium-cli
 *
 * Then inject the language-specific services by merging three modules:
 *  - Langium default language-specific services
 *  - Services generated by langium-cli
 *  - Services specified in this file
 *
 * @param context Optional module context with the LSP connection
 * @returns An object wrapping the shared services and the language-specific services
 */
function createJayveeServices(context) {
    const shared = (0, langium_1.inject)((0, langium_1.createDefaultSharedModule)(context), module_1.JayveeGeneratedSharedModule, exports.JayveeSharedModule);
    const Jayvee = (0, langium_1.inject)((0, langium_1.createDefaultModule)({ shared }), module_1.JayveeGeneratedModule, exports.JayveeModule);
    shared.ServiceRegistry.register(Jayvee);
    return { shared, Jayvee };
}
exports.createJayveeServices = createJayveeServices;


/***/ }),

/***/ "./libs/language-server/src/lib/jayvee-value-converter.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeValueConverter = void 0;
const langium_1 = __webpack_require__("langium");
class JayveeValueConverter extends langium_1.DefaultValueConverter {
    runConverter(rule, input, cstNode) {
        if (rule.name === 'REGEX') {
            // Trim leading and trailing '/' character:
            return input.substring(1, input.length - 1);
        }
        return super.runConverter(rule, input, cstNode);
    }
}
exports.JayveeValueConverter = JayveeValueConverter;


/***/ }),

/***/ "./libs/language-server/src/lib/services/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/services/runtime-parameter-provider.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/services/runtime-parameter-provider.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.RuntimeParameterProvider = void 0;
class RuntimeParameterProvider {
    constructor() {
        this.runtimeParameters = new Map();
        this.valueParser = undefined;
    }
    setValueParser(valueParser) {
        this.valueParser = valueParser;
    }
    getParsedValue(key, valuetype) {
        var _a;
        const stringValue = this.getRawValue(key);
        if (stringValue === undefined) {
            return undefined;
        }
        return (_a = this.valueParser) === null || _a === void 0 ? void 0 : _a.call(this, stringValue, valuetype);
    }
    getRawValue(key) {
        return this.runtimeParameters.get(key);
    }
    setValue(key, value) {
        this.runtimeParameters.set(key, value);
    }
    hasValue(key) {
        return this.runtimeParameters.has(key);
    }
    getReadonlyMap() {
        return this.runtimeParameters;
    }
}
exports.RuntimeParameterProvider = RuntimeParameterProvider;


/***/ }),

/***/ "./libs/language-server/src/lib/util/constructor-class.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));


/***/ }),

/***/ "./libs/language-server/src/lib/util/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/util/constructor-class.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/util/registry.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/util/registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.Registry = void 0;
const assert_1 = __webpack_require__("assert");
class Registry {
    constructor() {
        this.registry = new Map();
    }
    register(key, classToRegister) {
        (0, assert_1.strict)(!this.registry.has(key), `Multiple keys "${key}" were registered, expected at most one register call per key`);
        this.registry.set(key, classToRegister);
    }
    getAll() {
        return [...this.registry.values()];
    }
    getAllEntries() {
        return [...this.registry.entries()].map(([k, v]) => {
            return { key: k, value: v };
        });
    }
    get(key) {
        return this.registry.get(key);
    }
    clear() {
        this.registry.clear();
    }
}
exports.Registry = Registry;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/block-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateBlockDefinition = void 0;
/**
 * See https://jvalue.github.io/jayvee/docs/dev/guides/working-with-the-ast/ for why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const blocktype_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/blocktype-wrapper.ts");
function validateBlockDefinition(block, context) {
    checkPipesOfBlock(block, 'input', context);
    checkPipesOfBlock(block, 'output', context);
}
exports.validateBlockDefinition = validateBlockDefinition;
function checkPipesOfBlock(block, whatToCheck, context) {
    var _a, _b;
    if (!blocktype_wrapper_1.BlockTypeWrapper.canBeWrapped(block === null || block === void 0 ? void 0 : block.type)) {
        return;
    }
    const blockType = new blocktype_wrapper_1.BlockTypeWrapper(block === null || block === void 0 ? void 0 : block.type);
    const pipes = collectPipes(block, whatToCheck);
    const isStartOrEnd = (whatToCheck === 'input' && !blockType.hasInput()) ||
        (whatToCheck === 'output' && !blockType.hasOutput());
    if (isStartOrEnd) {
        if (pipes.length > 0) {
            for (const pipe of pipes) {
                context.accept('error', `Blocks of type ${blockType.type} do not have an ${whatToCheck}`, whatToCheck === 'input'
                    ? pipe.getToDiagnostic()
                    : pipe.getFromDiagnostic());
            }
        }
        return;
    }
    const hasMultipleInputPorts = pipes.length > 1 && whatToCheck === 'input';
    if (hasMultipleInputPorts) {
        for (const pipe of pipes) {
            context.accept('error', `At most one pipe can be connected to the ${whatToCheck} of a ${blockType.type}`, pipe.getToDiagnostic());
        }
        return;
    }
    // above: there should be pipes (defined by blocktype)
    // but there aren't any
    if (pipes.length === 0) {
        const isLastBlockOfCompositeBlocktype = (0, ast_1.isCompositeBlocktypeDefinition)(block.$container) &&
            ((_a = block.$container.blocks.at(-1)) === null || _a === void 0 ? void 0 : _a.name) === block.name;
        const isFirstBlockOfCompositeBlocktype = (0, ast_1.isCompositeBlocktypeDefinition)(block.$container) &&
            ((_b = block.$container.blocks.at(0)) === null || _b === void 0 ? void 0 : _b.name) === block.name;
        // exception: the first block in a composite block is connected to the input, not another block
        // exception: The last block in a composite block is connected to the output, not another block
        if (!isLastBlockOfCompositeBlocktype && !isFirstBlockOfCompositeBlocktype) {
            context.accept('warning', `A pipe should be connected to the ${whatToCheck} of this block`, {
                node: block,
                property: 'name',
            });
        }
    }
}
function collectPipes(block, whatToCheck) {
    let pipes;
    switch (whatToCheck) {
        case 'input': {
            pipes = (0, model_util_1.collectIngoingPipes)(block);
            break;
        }
        case 'output': {
            pipes = (0, model_util_1.collectOutgoingPipes)(block);
            break;
        }
        default: {
            (0, langium_1.assertUnreachable)(whatToCheck);
        }
    }
    return pipes;
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/blocktype-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateBlocktypeDefinition = void 0;
const assert_1 = __webpack_require__("assert");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
function validateBlocktypeDefinition(blocktype, validationContext, evaluationContext) {
    checkNoMultipleInputs(blocktype, validationContext);
    checkNoMultipleOutputs(blocktype, validationContext);
    checkOneInput(blocktype, validationContext);
    checkOneOutput(blocktype, validationContext);
    checkNoDuplicateProperties(blocktype, validationContext);
    checkPropertiesDefaultValuesHaveCorrectType(blocktype, validationContext, evaluationContext);
}
exports.validateBlocktypeDefinition = validateBlocktypeDefinition;
function checkNoMultipleInputs(blocktype, context) {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (blocktype.inputs === undefined) {
        return;
    }
    if (blocktype.inputs.length > 1) {
        blocktype.inputs.forEach((inputDefinition) => {
            context.accept('error', `Found more than one input definition in blocktype '${blocktype.name}'`, {
                node: inputDefinition,
            });
        });
    }
}
function checkNoMultipleOutputs(blocktype, context) {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (blocktype.outputs === undefined) {
        return;
    }
    if (blocktype.outputs.length > 1) {
        blocktype.outputs.forEach((outputDefinition) => {
            context.accept('error', `Found more than one output definition in blocktype '${blocktype.name}'`, {
                node: outputDefinition,
            });
        });
    }
}
function checkOneInput(blocktype, context) {
    var _a, _b;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const numberOfInputs = (_b = (_a = blocktype.inputs) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
    if (numberOfInputs < 1) {
        context.accept('error', `Found no input in blocktype '${blocktype.name}' - consider using iotype "none" if the blocktype consumes no input`, {
            node: blocktype,
        });
    }
}
function checkOneOutput(blocktype, context) {
    var _a, _b;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const numberOfOutputs = (_b = (_a = blocktype.outputs) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0;
    if (numberOfOutputs < 1) {
        context.accept('error', `Found no output in blocktype '${blocktype.name}' - consider using iotype "none" if the blocktype produces no output`, {
            node: blocktype,
        });
    }
}
function checkNoDuplicateProperties(blocktype, context) {
    var _a;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (blocktype.properties === undefined) {
        return;
    }
    const propertyMap = new Map();
    for (const property of blocktype.properties) {
        const propertyName = property.name;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (propertyName === undefined) {
            continue;
        }
        const trackedPropertiesWithName = (_a = propertyMap.get(propertyName)) !== null && _a !== void 0 ? _a : [];
        propertyMap.set(propertyName, [...trackedPropertiesWithName, property]);
    }
    [...propertyMap.values()]
        .filter((properties) => properties.length > 1)
        .forEach((properties) => {
        properties.forEach((property) => {
            context.accept('error', `Property '${property.name}' in blocktype '${blocktype.name}' is defined multiple times`, {
                node: property,
                property: 'name',
            });
        });
    });
}
function checkPropertiesDefaultValuesHaveCorrectType(blocktype, context, evaluationContext) {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (blocktype.properties === undefined) {
        return;
    }
    blocktype.properties
        .filter((property) => property.defaultValue !== undefined)
        .forEach((property) => checkPropertyDefaultValuesHasCorrectType(property, context, evaluationContext));
}
function checkPropertyDefaultValuesHasCorrectType(property, validationContext, evaluationContext) {
    const defaultValueExpression = property.defaultValue;
    if (defaultValueExpression === undefined) {
        return;
    }
    const evaluatedExpression = (0, ast_1.evaluateExpression)(defaultValueExpression, evaluationContext, validationContext);
    if (evaluatedExpression === undefined) {
        validationContext.accept('error', `Could not evaluate this expression.`, {
            node: property,
            property: 'defaultValue',
        });
        return;
    }
    const expectedValuetype = (0, ast_1.createValuetype)(property.valueType);
    (0, assert_1.strict)(expectedValuetype !== undefined);
    if (!expectedValuetype.isInternalValueRepresentation(evaluatedExpression)) {
        validationContext.accept('error', `This default value is not compatible with valuetype ${expectedValuetype.getName()}`, {
            node: property,
            property: 'defaultValue',
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/blocktype-specific/property-assignment.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.checkBlocktypeSpecificProperties = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
function checkBlocktypeSpecificProperties(property, propertySpec, validationContext, evaluationContext) {
    var _a;
    const propName = property.name;
    const propValue = (0, ast_1.evaluatePropertyValue)(property, evaluationContext, propertySpec.type);
    if (propValue === undefined) {
        return;
    }
    switch ((_a = property.$container.$container.type.ref) === null || _a === void 0 ? void 0 : _a.name) {
        case 'ArchiveInterpreter':
            return checkArchiveInterpreterProperty(propName, propValue, property, validationContext);
        case 'CellWriter':
            return checkCellWriterProperty(propName, property, validationContext, evaluationContext);
        case 'ColumnDeleter':
            return checkColumnDeleterProperty(propName, property, validationContext, evaluationContext);
        case 'GtfsRTInterpreter':
            return checkGtfsRTInterpreterProperty(propName, propValue, property, validationContext);
        case 'HttpExtractor':
            return checkHttpExtractorProperty(propName, propValue, property, validationContext);
        case 'RowDeleter':
            return checkRowDeleterProperty(propName, property, validationContext, evaluationContext);
        case 'TableInterpreter':
            return checkTableInterpreterProperty(propName, property, validationContext, evaluationContext);
        case 'TextFileInterpreter':
            return checkTextFileInterpreterProperty(propName, propValue, property, validationContext);
        case 'TextLineDeleter':
            return checkTextLineDeleterProperty(propName, property, validationContext, evaluationContext);
        case 'TextRangeSelector':
            return checkTextRangeSelectorProperty(propName, propValue, property, validationContext);
        default:
    }
}
exports.checkBlocktypeSpecificProperties = checkBlocktypeSpecificProperties;
function checkArchiveInterpreterProperty(propName, propValue, property, validationContext) {
    const allowedArchiveTypes = ['zip', 'gz'];
    if (propName === 'archiveType') {
        checkPropertyValueOneOf(propValue, allowedArchiveTypes, propName, property, validationContext);
    }
}
function checkCellWriterProperty(propName, property, validationContext, evaluationContext) {
    if (propName === 'at') {
        const cellRange = (0, ast_1.evaluatePropertyValue)(property, evaluationContext, ast_1.PrimitiveValuetypes.CellRange);
        if (cellRange === undefined) {
            return;
        }
        if (!cellRange.isOneDimensional()) {
            validationContext.accept('error', 'The cell range needs to be one-dimensional', {
                node: cellRange.astNode,
            });
        }
    }
}
function checkColumnDeleterProperty(propName, property, validationContext, evaluationContext) {
    if (propName === 'delete') {
        const cellRanges = (0, ast_1.evaluatePropertyValue)(property, evaluationContext, new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.CellRange));
        cellRanges === null || cellRanges === void 0 ? void 0 : cellRanges.forEach((cellRange) => {
            if (!(0, ast_1.isColumnWrapper)(cellRange)) {
                validationContext.accept('error', 'An entire column needs to be selected', {
                    node: cellRange.astNode,
                });
            }
        });
    }
}
function checkGtfsRTInterpreterProperty(propName, propValue, property, validationContext) {
    const allowedEntities = [
        'trip_update',
        'alert',
        'vehicle',
    ];
    if (propName === 'entity') {
        checkPropertyValueOneOf(propValue, allowedEntities, propName, property, validationContext);
    }
}
function checkHttpExtractorProperty(propName, propValue, property, validationContext) {
    if (propName === 'retries') {
        const minRetryValue = 0;
        if (typeof propValue === 'number' && propValue < minRetryValue) {
            validationContext.accept('error', `The value of property "${propName}" must not be smaller than ${minRetryValue}`, {
                node: property,
                property: 'value',
            });
        }
    }
    if (propName === 'retryBackoffMilliseconds') {
        const minBackoffValue = 1000;
        if (typeof propValue === 'number' && propValue < minBackoffValue) {
            validationContext.accept('error', `The value of property "${propName}" must not be smaller than ${minBackoffValue}`, {
                node: property,
                property: 'value',
            });
        }
    }
    if (propName === 'retryBackoffStrategy') {
        const allowedRetryStrategies = [
            'exponential',
            'linear',
        ];
        checkPropertyValueOneOf(propValue, allowedRetryStrategies, propName, property, validationContext);
    }
}
function checkRowDeleterProperty(propName, property, validationContext, evaluationContext) {
    if (propName === 'delete') {
        const cellRanges = (0, ast_1.evaluatePropertyValue)(property, evaluationContext, new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.CellRange));
        cellRanges === null || cellRanges === void 0 ? void 0 : cellRanges.forEach((cellRange) => {
            if (!(0, ast_1.isRowWrapper)(cellRange)) {
                validationContext.accept('error', 'An entire row needs to be selected', {
                    node: cellRange.astNode,
                });
            }
        });
    }
}
function checkTableInterpreterProperty(propName, property, validationContext, evaluationContext) {
    if (propName === 'columns') {
        const valuetypeAssignments = (0, ast_1.evaluatePropertyValue)(property, evaluationContext, new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.ValuetypeAssignment));
        if (valuetypeAssignments === undefined) {
            return;
        }
        (0, validation_util_1.checkUniqueNames)(valuetypeAssignments, validationContext, 'column');
    }
}
function checkTextFileInterpreterProperty(propName, propValue, property, validationContext) {
    // https://developer.mozilla.org/en-US/docs/Web/API/Encoding_API/Encodings
    const allowedEncodings = [
        'utf8',
        'ibm866',
        'latin2',
        'latin3',
        'latin4',
        'cyrillic',
        'arabic',
        'greek',
        'hebrew',
        'logical',
        'latin6',
        'utf-16',
    ];
    if (propName === 'encoding') {
        checkPropertyValueOneOf(propValue, allowedEncodings, propName, property, validationContext);
    }
}
function checkTextLineDeleterProperty(propName, property, validationContext, evaluationContext) {
    if (propName === 'lines') {
        const minTextLineIndex = 1;
        const lines = (0, ast_1.evaluatePropertyValue)(property, evaluationContext, new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.Integer));
        lines === null || lines === void 0 ? void 0 : lines.forEach((value, index) => {
            if (value < minTextLineIndex) {
                validationContext.accept('error', `Line numbers need to be greater than zero`, {
                    node: property.value,
                    property: 'values',
                    index: index,
                });
            }
        });
    }
}
function checkTextRangeSelectorProperty(propName, propValue, property, validationContext) {
    const minLineIndex = 1;
    if (propName === 'lineFrom' || propName === 'lineTo') {
        if (typeof propValue === 'number' && propValue < minLineIndex) {
            validationContext.accept('error', `The value of property "${propName}" must not be smaller than ${minLineIndex}`, {
                node: property,
                property: 'value',
            });
        }
    }
}
function checkPropertyValueOneOf(propValue, allowedValues, propName, property, validationContext) {
    if (!allowedValues.includes(propValue)) {
        validationContext.accept('error', `The value of property "${propName}" must be one of the following values: ${allowedValues
            .map((v) => `"${v.toString()}"`)
            .join(', ')}`, {
            node: property,
            property: 'value',
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/blocktype-specific/property-body.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.checkBlocktypeSpecificPropertyBody = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
function checkBlocktypeSpecificPropertyBody(propertyBody, validationContext, evaluationContext) {
    var _a;
    switch ((_a = propertyBody.$container.type.ref) === null || _a === void 0 ? void 0 : _a.name) {
        case 'TextRangeSelector':
            return checkTextRangeSelectorPropertyBody(propertyBody, validationContext, evaluationContext);
        case 'CellWriter':
            return checkCellWriterPropertyBody(propertyBody, validationContext, evaluationContext);
        case 'TableTransformer':
            return checkTableTransformerPropertyBody(propertyBody, validationContext, evaluationContext);
        default:
    }
}
exports.checkBlocktypeSpecificPropertyBody = checkBlocktypeSpecificPropertyBody;
function checkTextRangeSelectorPropertyBody(propertyBody, validationContext, evaluationContext) {
    const lineFromProperty = propertyBody.properties.find((p) => p.name === 'lineFrom');
    const lineToProperty = propertyBody.properties.find((p) => p.name === 'lineTo');
    if (lineFromProperty === undefined || lineToProperty === undefined) {
        return;
    }
    const lineFrom = (0, ast_1.evaluatePropertyValue)(lineFromProperty, evaluationContext, ast_1.PrimitiveValuetypes.Integer);
    const lineTo = (0, ast_1.evaluatePropertyValue)(lineToProperty, evaluationContext, ast_1.PrimitiveValuetypes.Integer);
    if (lineFrom === undefined || lineTo === undefined) {
        return;
    }
    if (lineFrom > lineTo) {
        [lineFromProperty, lineToProperty].forEach((property) => {
            validationContext.accept('error', 'The lower line number needs to be smaller or equal to the upper line number', { node: property.value });
        });
    }
}
function checkCellWriterPropertyBody(propertyBody, validationContext, evaluationContext) {
    const writeProperty = propertyBody.properties.find((p) => p.name === 'write');
    const atProperty = propertyBody.properties.find((p) => p.name === 'at');
    if (writeProperty === undefined || atProperty === undefined) {
        return;
    }
    const writeValues = (0, ast_1.evaluatePropertyValue)(writeProperty, evaluationContext, new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.Text));
    const atValue = (0, ast_1.evaluatePropertyValue)(atProperty, evaluationContext, ast_1.PrimitiveValuetypes.CellRange);
    if (writeValues === undefined || atValue === undefined) {
        return;
    }
    const numberOfValuesToWrite = writeValues.length;
    const numberOfCells = atValue.numberOfCells();
    if (numberOfCells !== numberOfValuesToWrite) {
        [writeProperty, atProperty].forEach((propertyNode) => {
            validationContext.accept('warning', `The number of values to write (${numberOfValuesToWrite}) does not match the number of cells (${numberOfCells})`, { node: propertyNode.value });
        });
    }
}
function checkTableTransformerPropertyBody(propertyBody, validationContext, evaluationContext) {
    checkInputColumnsMatchTransformationPorts(propertyBody, validationContext, evaluationContext);
}
function checkInputColumnsMatchTransformationPorts(propertyBody, validationContext, evaluationContext) {
    var _a, _b;
    const useProperty = propertyBody.properties.find((x) => x.name === 'use');
    const inputColumnsProperty = propertyBody.properties.find((x) => x.name === 'inputColumns');
    if (useProperty === undefined || inputColumnsProperty === undefined) {
        return;
    }
    const transform = (0, ast_1.evaluatePropertyValue)(useProperty, evaluationContext, ast_1.PrimitiveValuetypes.Transform);
    const inputColumns = (0, ast_1.evaluatePropertyValue)(inputColumnsProperty, evaluationContext, new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.Text));
    if (transform === undefined || inputColumns === undefined) {
        return;
    }
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const transformInputPorts = (_b = (_a = transform === null || transform === void 0 ? void 0 : transform.body) === null || _a === void 0 ? void 0 : _a.ports) === null || _b === void 0 ? void 0 : _b.filter((x) => x.kind === 'from');
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (transformInputPorts === undefined) {
        return;
    }
    const numberTransformPorts = transformInputPorts.length;
    const numberInputColumns = inputColumns.length;
    if (numberTransformPorts !== numberInputColumns) {
        validationContext.accept('error', `Expected ${numberTransformPorts} columns but only got ${numberInputColumns}`, {
            node: inputColumnsProperty,
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/column-id.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateColumnId = void 0;
function validateColumnId(columnId, context) {
    checkColumnIdSyntax(columnId, context);
}
exports.validateColumnId = validateColumnId;
function checkColumnIdSyntax(columnId, context) {
    if ((columnId === null || columnId === void 0 ? void 0 : columnId.value) === undefined || (columnId === null || columnId === void 0 ? void 0 : columnId.value) === '*') {
        return;
    }
    const columnIdRegex = /^[A-Z]+$/;
    if (!columnIdRegex.test(columnId.value)) {
        context.accept('error', 'Columns need to be denoted via capital letters or the * character', {
            node: columnId,
            property: 'value',
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/composite-blocktype-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateCompositeBlockTypeDefinition = void 0;
const blocktype_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/blocktype-definition.ts");
function validateCompositeBlockTypeDefinition(blockType, validationContext, evaluationContext) {
    (0, blocktype_definition_1.validateBlocktypeDefinition)(blockType, validationContext, evaluationContext);
    checkHasPipeline(blockType, validationContext);
    checkExactlyOnePipeline(blockType, validationContext);
}
exports.validateCompositeBlockTypeDefinition = validateCompositeBlockTypeDefinition;
function checkHasPipeline(blockType, context) {
    if (blockType.pipes.length === 0) {
        context.accept('error', `Composite blocktypes must define one pipeline '${blockType.name}'`, {
            node: blockType,
            property: 'name',
        });
    }
}
function checkExactlyOnePipeline(blockType, context) {
    if (blockType.pipes.length > 1) {
        blockType.pipes.forEach((pipe) => {
            context.accept('error', `Found more than one pipeline definition in composite blocktype '${blockType.name}'`, {
                node: pipe,
            });
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/constrainttype-specific/property-body.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.checkConstraintTypeSpecificPropertyBody = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
function checkConstraintTypeSpecificPropertyBody(propertyBody, validationContext, evaluationContext) {
    var _a;
    switch ((_a = propertyBody.$container.type.ref) === null || _a === void 0 ? void 0 : _a.name) {
        case 'LengthConstraint':
            return checkLengthConstraintPropertyBody(propertyBody, validationContext, evaluationContext);
        case 'RangeConstraint':
            return checkRangeConstraintPropertyBody(propertyBody, validationContext, evaluationContext);
        default:
    }
}
exports.checkConstraintTypeSpecificPropertyBody = checkConstraintTypeSpecificPropertyBody;
function checkLengthConstraintPropertyBody(propertyBody, validationContext, evaluationContext) {
    const minLengthProperty = propertyBody.properties.find((p) => p.name === 'minLength');
    const maxLengthProperty = propertyBody.properties.find((p) => p.name === 'maxLength');
    if (minLengthProperty === undefined || maxLengthProperty === undefined) {
        return;
    }
    const minLength = (0, ast_1.evaluatePropertyValue)(minLengthProperty, evaluationContext, ast_1.PrimitiveValuetypes.Integer);
    const maxLength = (0, ast_1.evaluatePropertyValue)(maxLengthProperty, evaluationContext, ast_1.PrimitiveValuetypes.Integer);
    if (minLength === undefined || maxLength === undefined) {
        return;
    }
    if (minLength > maxLength) {
        [minLengthProperty, maxLengthProperty].forEach((property) => {
            validationContext.accept('error', 'The minimum length needs to be smaller or equal to the maximum length', { node: property.value });
        });
    }
}
function checkRangeConstraintPropertyBody(propertyBody, validationContext, evaluationContext) {
    const lowerBoundProperty = propertyBody.properties.find((p) => p.name === 'lowerBound');
    const upperBoundProperty = propertyBody.properties.find((p) => p.name === 'upperBound');
    if (lowerBoundProperty === undefined || upperBoundProperty === undefined) {
        return;
    }
    const lowerBound = (0, ast_1.evaluatePropertyValue)(lowerBoundProperty, evaluationContext, ast_1.PrimitiveValuetypes.Decimal);
    const upperBound = (0, ast_1.evaluatePropertyValue)(upperBoundProperty, evaluationContext, ast_1.PrimitiveValuetypes.Decimal);
    if (lowerBound === undefined || upperBound === undefined) {
        return;
    }
    if (lowerBound > upperBound) {
        [lowerBoundProperty, upperBoundProperty].forEach((property) => {
            validationContext.accept('error', 'The lower bound needs to be smaller or equal to the upper bound', { node: property.value });
        });
        return;
    }
    const lowerBoundInclusiveProperty = propertyBody.properties.find((p) => p.name === 'lowerBoundInclusive');
    const upperBoundInclusiveProperty = propertyBody.properties.find((p) => p.name === 'upperBoundInclusive');
    if (lowerBound === upperBound) {
        let lowerBoundInclusive = true;
        if (lowerBoundInclusiveProperty !== undefined) {
            const expressionValue = (0, ast_1.evaluatePropertyValue)(lowerBoundInclusiveProperty, evaluationContext, ast_1.PrimitiveValuetypes.Boolean);
            if (expressionValue === undefined) {
                return;
            }
            lowerBoundInclusive = expressionValue;
        }
        let upperBoundInclusive = true;
        if (upperBoundInclusiveProperty !== undefined) {
            const expressionValue = (0, ast_1.evaluatePropertyValue)(upperBoundInclusiveProperty, evaluationContext, ast_1.PrimitiveValuetypes.Boolean);
            if (expressionValue === undefined) {
                return;
            }
            upperBoundInclusive = expressionValue;
        }
        const errorMessage = 'Lower and upper bounds need to be inclusive if they are identical';
        if (!lowerBoundInclusive) {
            validationContext.accept('error', errorMessage, {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                node: lowerBoundInclusiveProperty.value,
            });
        }
        if (!upperBoundInclusive) {
            validationContext.accept('error', errorMessage, {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                node: upperBoundInclusiveProperty.value,
            });
        }
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/expression-constraint-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateExpressionConstraintDefinition = void 0;
const type_inference_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-inference.ts");
const primitive_valuetypes_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/primitive/primitive-valuetypes.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
function validateExpressionConstraintDefinition(constraint, validationContext, evaluationContext) {
    checkConstraintExpression(constraint, validationContext, evaluationContext);
}
exports.validateExpressionConstraintDefinition = validateExpressionConstraintDefinition;
function checkConstraintExpression(constraint, validationContext, evaluationContext) {
    const expression = constraint === null || constraint === void 0 ? void 0 : constraint.expression;
    const inferredType = (0, type_inference_1.inferExpressionType)(expression, validationContext);
    if (inferredType === undefined) {
        return;
    }
    const expectedType = primitive_valuetypes_1.PrimitiveValuetypes.Boolean;
    if (!inferredType.isConvertibleTo(expectedType)) {
        validationContext.accept('error', `The value needs to be of type ${expectedType.getName()} but is of type ${inferredType.getName()}`, {
            node: expression,
        });
        return;
    }
    (0, validation_util_1.checkExpressionSimplification)(expression, validationContext, evaluationContext);
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/jayvee-model.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateJayveeModel = void 0;
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
function validateJayveeModel(model, context) {
    (0, validation_util_1.checkUniqueNames)(model.pipelines, context);
    (0, validation_util_1.checkUniqueNames)(model.transforms, context);
    (0, validation_util_1.checkUniqueNames)(model.valuetypes, context);
    (0, validation_util_1.checkUniqueNames)(model.constraints, context);
    (0, validation_util_1.checkUniqueNames)(model.blocktypes, context);
    (0, validation_util_1.checkUniqueNames)(model.constrainttypes, context);
    (0, validation_util_1.checkUniqueNames)(model.iotypes, context);
}
exports.validateJayveeModel = validateJayveeModel;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/pipe-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validatePipeDefinition = void 0;
const pipe_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/pipe-wrapper.ts");
const blocktype_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/blocktype-wrapper.ts");
function validatePipeDefinition(pipe, context) {
    checkBlockCompatibility(pipe, context);
}
exports.validatePipeDefinition = validatePipeDefinition;
function checkBlockCompatibility(pipe, context) {
    var _a, _b;
    const pipeWrappers = (0, pipe_wrapper_1.createWrappersFromPipeChain)(pipe);
    for (const pipeWrapper of pipeWrappers) {
        const fromBlockTypeDefinition = (_a = pipeWrapper.from) === null || _a === void 0 ? void 0 : _a.type;
        const toBlockTypeDefinition = (_b = pipeWrapper.to) === null || _b === void 0 ? void 0 : _b.type;
        if (!blocktype_wrapper_1.BlockTypeWrapper.canBeWrapped(fromBlockTypeDefinition) ||
            !blocktype_wrapper_1.BlockTypeWrapper.canBeWrapped(toBlockTypeDefinition)) {
            continue;
        }
        const fromBlockType = new blocktype_wrapper_1.BlockTypeWrapper(fromBlockTypeDefinition);
        const toBlockType = new blocktype_wrapper_1.BlockTypeWrapper(toBlockTypeDefinition);
        if (fromBlockType.hasOutput() && toBlockType.hasInput()) {
            if (!fromBlockType.canBeConnectedTo(toBlockType)) {
                const errorMessage = `The output type "${fromBlockType.outputType}" of ${fromBlockType.type} is incompatible with the input type "${toBlockType.inputType}" of ${toBlockType.type}`;
                context.accept('error', errorMessage, pipeWrapper.getFromDiagnostic());
                context.accept('error', errorMessage, pipeWrapper.getToDiagnostic());
            }
        }
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/pipeline-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validatePipelineDefinition = void 0;
const model_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/model-util.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
function validatePipelineDefinition(pipeline, context) {
    checkStartingBlocks(pipeline, context);
    (0, validation_util_1.checkUniqueNames)(pipeline.blocks, context);
    (0, validation_util_1.checkUniqueNames)(pipeline.transforms, context);
    (0, validation_util_1.checkUniqueNames)(pipeline.valuetypes, context);
    (0, validation_util_1.checkUniqueNames)(pipeline.constraints, context);
}
exports.validatePipelineDefinition = validatePipelineDefinition;
function checkStartingBlocks(pipeline, context) {
    const startingBlocks = (0, model_util_1.collectStartingBlocks)(pipeline);
    if (startingBlocks.length === 0) {
        context.accept('error', `An extractor block is required for this pipeline`, {
            node: pipeline,
            property: 'name',
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/property-assignment.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validatePropertyAssignment = void 0;
/**
 * See https://jvalue.github.io/jayvee/docs/dev/guides/working-with-the-ast/ for why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const ast_2 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
const property_assignment_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/blocktype-specific/property-assignment.ts");
function validatePropertyAssignment(property, wrapper, validationContext, evaluationContext) {
    const propertySpec = wrapper.getPropertySpecification(property === null || property === void 0 ? void 0 : property.name);
    checkPropertyNameValidity(property, propertySpec, validationContext);
    if (propertySpec === undefined) {
        return;
    }
    checkPropertyValueTyping(property, propertySpec, validationContext, evaluationContext);
    if (validationContext.hasErrorOccurred()) {
        return;
    }
    (0, property_assignment_1.checkBlocktypeSpecificProperties)(property, propertySpec, validationContext, evaluationContext);
}
exports.validatePropertyAssignment = validatePropertyAssignment;
function checkPropertyNameValidity(property, propertySpec, context) {
    var _a;
    if (propertySpec === undefined) {
        context.accept('error', `Invalid property name "${(_a = property === null || property === void 0 ? void 0 : property.name) !== null && _a !== void 0 ? _a : ''}".`, {
            node: property,
            property: 'name',
        });
    }
}
function checkPropertyValueTyping(property, propertySpec, validationContext, evaluationContext) {
    const propertyType = propertySpec.type;
    const propertyValue = property === null || property === void 0 ? void 0 : property.value;
    if (propertyValue === undefined) {
        return;
    }
    if ((0, ast_2.isRuntimeParameterLiteral)(propertyValue)) {
        if (!propertyType.isAllowedAsRuntimeParameter()) {
            validationContext.accept('error', `Runtime parameters are not allowed for properties of type ${propertyType.getName()}`, {
                node: propertyValue,
            });
        }
        return;
    }
    if ((0, ast_2.isBlocktypeProperty)(propertyValue)) {
        return;
    }
    const inferredType = (0, ast_1.inferExpressionType)(propertyValue, validationContext);
    if (inferredType === undefined) {
        return;
    }
    if (!inferredType.isConvertibleTo(propertyType)) {
        validationContext.accept('error', `The value of property "${property.name}" needs to be of type ${propertyType.getName()} but is of type ${inferredType.getName()}`, {
            node: propertyValue,
        });
        return;
    }
    (0, validation_util_1.checkExpressionSimplification)(propertyValue, validationContext, evaluationContext);
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/property-body.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validatePropertyBody = void 0;
/**
 * See https://jvalue.github.io/jayvee/docs/dev/guides/working-with-the-ast/ for why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const ast_2 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
const property_body_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/blocktype-specific/property-body.ts");
const property_body_2 = __webpack_require__("./libs/language-server/src/lib/validation/checks/constrainttype-specific/property-body.ts");
const property_assignment_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/property-assignment.ts");
function validatePropertyBody(propertyBody, validationContext, evaluationContext) {
    var _a;
    const properties = (_a = propertyBody === null || propertyBody === void 0 ? void 0 : propertyBody.properties) !== null && _a !== void 0 ? _a : [];
    (0, validation_util_1.checkUniqueNames)(properties, validationContext);
    const wrapper = inferTypedObjectWrapper(propertyBody);
    if (wrapper === undefined) {
        return;
    }
    checkPropertyCompleteness(propertyBody, properties, wrapper, validationContext);
    for (const property of propertyBody.properties) {
        (0, property_assignment_1.validatePropertyAssignment)(property, wrapper, validationContext, evaluationContext);
    }
    if (validationContext.hasErrorOccurred()) {
        return;
    }
    checkCustomPropertyValidation(propertyBody, wrapper, validationContext, evaluationContext);
}
exports.validatePropertyBody = validatePropertyBody;
function inferTypedObjectWrapper(propertyBody) {
    var _a;
    const type = (_a = propertyBody.$container) === null || _a === void 0 ? void 0 : _a.type.ref;
    return (0, ast_1.getTypedObjectWrapper)(type);
}
function checkPropertyCompleteness(propertyBody, properties, wrapper, context) {
    const presentPropertyNames = properties.map((property) => property.name);
    const missingRequiredPropertyNames = wrapper.getMissingRequiredPropertyNames(presentPropertyNames);
    if (missingRequiredPropertyNames.length > 0) {
        context.accept('error', `The following required properties are missing: ${missingRequiredPropertyNames
            .map((name) => `"${name}"`)
            .join(', ')}`, {
            node: propertyBody.$container,
            property: 'type',
        });
    }
}
function checkCustomPropertyValidation(propertyBody, wrapper, validationContext, evaluationContext) {
    wrapper.validate(propertyBody, validationContext, evaluationContext);
    if ((0, ast_2.isBlockDefinition)(propertyBody.$container)) {
        (0, property_body_1.checkBlocktypeSpecificPropertyBody)(propertyBody, validationContext, evaluationContext);
    }
    else if ((0, ast_2.isTypedConstraintDefinition)(propertyBody.$container)) {
        (0, property_body_2.checkConstraintTypeSpecificPropertyBody)(propertyBody, validationContext, evaluationContext);
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/range-literal.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateRangeLiteral = void 0;
const cell_range_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/cell-range-wrapper.ts");
function validateRangeLiteral(range, context) {
    if (!cell_range_wrapper_1.CellRangeWrapper.canBeWrapped(range)) {
        return;
    }
    const wrappedRange = new cell_range_wrapper_1.CellRangeWrapper(range);
    checkRangeLimits(wrappedRange, context);
}
exports.validateRangeLiteral = validateRangeLiteral;
function checkRangeLimits(range, context) {
    if (range.from.columnIndex > range.to.columnIndex ||
        range.from.rowIndex > range.to.rowIndex) {
        context.accept('error', `Cell ranges need to be spanned from top-left to bottom-right`, {
            node: range.astNode,
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/regex-literal.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateRegexLiteral = void 0;
/**
 * See https://jvalue.github.io/jayvee/docs/dev/guides/working-with-the-ast/ for why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
function validateRegexLiteral(regex, context) {
    checkRegexParsability(regex, context);
}
exports.validateRegexLiteral = validateRegexLiteral;
function checkRegexParsability(regex, context) {
    const regexValue = regex === null || regex === void 0 ? void 0 : regex.value;
    if (regexValue === undefined) {
        return;
    }
    try {
        new RegExp(regexValue);
    }
    catch (error) {
        if (error instanceof SyntaxError) {
            context.accept('error', `A parsing error occurred: ${error.message}`, {
                node: regex,
            });
        }
        else {
            context.accept('error', `An unknown parsing error occurred: ${JSON.stringify(error)}.`, {
                node: regex,
            });
        }
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/transform-body.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateTransformBody = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
const transform_output_assigment_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/transform-output-assigment.ts");
function validateTransformBody(transformBody, validationContext, evaluationContext) {
    (0, validation_util_1.checkUniqueNames)(transformBody.ports, validationContext, 'transform port');
    checkUniqueOutputAssignments(transformBody, validationContext);
    checkSingleOutputPort(transformBody, validationContext);
    checkAreInputsUsed(transformBody, validationContext);
    for (const property of transformBody.outputAssignments) {
        (0, transform_output_assigment_1.validateTransformOutputAssignment)(property, validationContext, evaluationContext);
    }
}
exports.validateTransformBody = validateTransformBody;
function checkUniqueOutputAssignments(transformBody, context) {
    var _a, _b, _c;
    const assignedOutputPorts = (_a = transformBody === null || transformBody === void 0 ? void 0 : transformBody.outputAssignments) !== null && _a !== void 0 ? _a : [];
    const definedOutputPorts = (_c = (_b = transformBody === null || transformBody === void 0 ? void 0 : transformBody.ports) === null || _b === void 0 ? void 0 : _b.filter((x) => (x === null || x === void 0 ? void 0 : x.kind) === 'to')) !== null && _c !== void 0 ? _c : [];
    for (const definedOutputPort of definedOutputPorts) {
        const usedInAssignments = assignedOutputPorts.filter((x) => { var _a, _b; return ((_b = (_a = x === null || x === void 0 ? void 0 : x.outPortName) === null || _a === void 0 ? void 0 : _a.ref) === null || _b === void 0 ? void 0 : _b.name) === definedOutputPort.name; });
        if (usedInAssignments.length === 0) {
            context.accept('error', 'An output assignment is required for this port', { node: definedOutputPort, property: 'name' });
        }
        if (usedInAssignments.length > 1) {
            usedInAssignments.forEach((usedAssignment) => {
                context.accept('error', 'At most one assignment per output port', {
                    node: usedAssignment,
                    property: 'outPortName',
                });
            });
        }
    }
}
function checkSingleOutputPort(transformBody, context) {
    var _a;
    const ports = (_a = transformBody.ports) === null || _a === void 0 ? void 0 : _a.filter((x) => x.kind === 'to');
    if (ports === undefined) {
        return undefined;
    }
    if (ports.length > 1) {
        ports.forEach((port) => {
            context.accept('error', `More than one output port is defined`, {
                node: port,
            });
        });
    }
    if (ports.length === 0) {
        context.accept('error', `There has to be a single output port`, {
            node: transformBody.$container,
            property: 'name',
        });
    }
}
function checkAreInputsUsed(transformBody, context) {
    var _a;
    const inputs = (_a = transformBody.ports) === null || _a === void 0 ? void 0 : _a.filter((x) => x.kind === 'from');
    const outputAssignments = transformBody === null || transformBody === void 0 ? void 0 : transformBody.outputAssignments;
    if (inputs === undefined || outputAssignments === undefined) {
        return undefined;
    }
    const referencedPorts = [];
    outputAssignments.forEach((outputAssignment) => {
        const referenceLiterals = (0, transform_output_assigment_1.extractReferenceLiterals)(outputAssignment === null || outputAssignment === void 0 ? void 0 : outputAssignment.expression);
        referencedPorts.push(...referenceLiterals
            .map((x) => { var _a; return (_a = x === null || x === void 0 ? void 0 : x.value) === null || _a === void 0 ? void 0 : _a.ref; })
            .filter(ast_1.isTransformPortDefinition));
    });
    const referencedPortNames = referencedPorts.map((x) => x === null || x === void 0 ? void 0 : x.name);
    inputs.forEach((input) => {
        if (input.name === undefined) {
            return;
        }
        if (!referencedPortNames.includes(input.name)) {
            if (isOutputPortComplete(input)) {
                context.accept('warning', 'This input port is never used', {
                    node: input,
                });
            }
        }
    });
}
function isOutputPortComplete(portDefinition) {
    var _a, _b;
    const valueType = (_b = (_a = portDefinition === null || portDefinition === void 0 ? void 0 : portDefinition.valueType) === null || _a === void 0 ? void 0 : _a.reference) === null || _b === void 0 ? void 0 : _b.ref;
    if (valueType === undefined) {
        return false;
    }
    return (valueType === null || valueType === void 0 ? void 0 : valueType.name) !== undefined;
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/transform-output-assigment.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.extractReferenceLiterals = exports.validateTransformOutputAssignment = void 0;
/**
 * See https://jvalue.github.io/jayvee/docs/dev/guides/working-with-the-ast/ for why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const langium_1 = __webpack_require__("langium");
const type_inference_1 = __webpack_require__("./libs/language-server/src/lib/ast/expressions/type-inference.ts");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
const valuetype_util_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/value-type/valuetype-util.ts");
const validation_util_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts");
function validateTransformOutputAssignment(outputAssignment, validationContext, evaluationContext) {
    checkOutputValueTyping(outputAssignment, validationContext, evaluationContext);
    checkOutputNotInAssignmentExpression(outputAssignment, validationContext);
}
exports.validateTransformOutputAssignment = validateTransformOutputAssignment;
function checkOutputValueTyping(outputAssignment, validationContext, evaluationContext) {
    var _a, _b;
    const assignmentExpression = outputAssignment === null || outputAssignment === void 0 ? void 0 : outputAssignment.expression;
    if (assignmentExpression === undefined) {
        return;
    }
    const outputType = (_b = (_a = outputAssignment === null || outputAssignment === void 0 ? void 0 : outputAssignment.outPortName) === null || _a === void 0 ? void 0 : _a.ref) === null || _b === void 0 ? void 0 : _b.valueType;
    if (outputType === undefined) {
        return;
    }
    const inferredType = (0, type_inference_1.inferExpressionType)(assignmentExpression, validationContext);
    if (inferredType === undefined) {
        return;
    }
    const expectedType = (0, valuetype_util_1.createValuetype)(outputType);
    if (expectedType === undefined) {
        return;
    }
    if (!inferredType.isConvertibleTo(expectedType)) {
        validationContext.accept('error', `The value needs to be of type ${expectedType.getName()} but is of type ${inferredType.getName()}`, {
            node: assignmentExpression,
        });
        return;
    }
    (0, validation_util_1.checkExpressionSimplification)(assignmentExpression, validationContext, evaluationContext);
}
function checkOutputNotInAssignmentExpression(outputAssignment, context) {
    const referenceLiterals = extractReferenceLiterals(outputAssignment === null || outputAssignment === void 0 ? void 0 : outputAssignment.expression);
    referenceLiterals.forEach((referenceLiteral) => {
        var _a;
        const referenced = (_a = referenceLiteral === null || referenceLiteral === void 0 ? void 0 : referenceLiteral.value) === null || _a === void 0 ? void 0 : _a.ref;
        if (!(0, ast_1.isTransformPortDefinition)(referenced)) {
            return;
        }
        if ((referenced === null || referenced === void 0 ? void 0 : referenced.kind) === 'to') {
            context.accept('error', 'Output ports are not allowed in this expression', {
                node: referenceLiteral,
            });
        }
    });
}
function extractReferenceLiterals(expression) {
    if (expression === undefined) {
        return [];
    }
    if ((0, ast_1.isExpressionLiteral)(expression)) {
        if ((0, ast_1.isReferenceLiteral)(expression)) {
            return [expression];
        }
        return [];
    }
    else if ((0, ast_1.isBinaryExpression)(expression)) {
        return [
            ...extractReferenceLiterals(expression.left),
            ...extractReferenceLiterals(expression.right),
        ];
    }
    else if ((0, ast_1.isUnaryExpression)(expression)) {
        return extractReferenceLiterals(expression.expression);
    }
    (0, langium_1.assertUnreachable)(expression);
}
exports.extractReferenceLiterals = extractReferenceLiterals;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/typed-constraint-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


/* eslint-disable @typescript-eslint/no-unnecessary-condition */
// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateTypedConstraintDefinition = void 0;
const constrainttype_wrapper_1 = __webpack_require__("./libs/language-server/src/lib/ast/wrappers/typed-object/constrainttype-wrapper.ts");
function validateTypedConstraintDefinition(constraint, context) {
    checkConstraintType(constraint, context);
    // TODO: add custom validations
}
exports.validateTypedConstraintDefinition = validateTypedConstraintDefinition;
function checkConstraintType(constraint, context) {
    var _a;
    const constraintType = constraint === null || constraint === void 0 ? void 0 : constraint.type;
    if (constraintType === undefined) {
        return undefined;
    }
    const canCreateWrapper = constrainttype_wrapper_1.ConstraintTypeWrapper.canBeWrapped(constraintType);
    if (!canCreateWrapper) {
        context.accept('error', `Unknown constraint type '${(_a = constraintType.$refText) !== null && _a !== void 0 ? _a : ''}'`, {
            node: constraint,
            property: 'type',
        });
    }
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/valuetype-definition.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateValuetypeDefinition = void 0;
/**
 * See https://jvalue.github.io/jayvee/docs/dev/guides/working-with-the-ast/ for why the following ESLint rule is disabled for this file.
 */
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
function validateValuetypeDefinition(valuetype, validationContext, evaluationContext) {
    checkSupertypeCycle(valuetype, validationContext);
    checkConstraintsCollectionValues(valuetype, validationContext, evaluationContext);
    checkGenericsHaveNoDuplicate(valuetype, validationContext);
}
exports.validateValuetypeDefinition = validateValuetypeDefinition;
function checkSupertypeCycle(valuetypeDefinition, context) {
    var _a, _b;
    const hasCycle = (_b = (_a = (0, ast_1.createValuetype)(valuetypeDefinition)) === null || _a === void 0 ? void 0 : _a.hasSupertypeCycle()) !== null && _b !== void 0 ? _b : false;
    if (hasCycle) {
        (0, assert_1.strict)(!valuetypeDefinition.isBuiltin);
        context.accept('error', 'Could not construct this valuetype since there is a cycle in the (transitive) "oftype" relation.', {
            node: valuetypeDefinition,
            property: 'type',
        });
    }
}
function checkConstraintsCollectionValues(valuetype, validationContext, evaluationContext) {
    const constraintCollection = valuetype === null || valuetype === void 0 ? void 0 : valuetype.constraints;
    if (constraintCollection === undefined) {
        return;
    }
    const inferredCollectionType = (0, ast_1.inferExpressionType)(constraintCollection, validationContext);
    const expectedType = new ast_1.CollectionValuetype(ast_1.PrimitiveValuetypes.Constraint);
    if (inferredCollectionType === undefined) {
        return;
    }
    if (!inferredCollectionType.isConvertibleTo(expectedType)) {
        validationContext.accept('error', `The value needs to be of type ${expectedType.getName()} but is of type ${inferredCollectionType.getName()}`, {
            node: constraintCollection,
        });
        return;
    }
    const constraints = (0, ast_1.evaluateExpression)(constraintCollection, evaluationContext, validationContext);
    (0, assert_1.strict)(expectedType.isInternalValueRepresentation(constraints));
    constraints.forEach((constraint, index) => {
        checkConstraintMatchesValuetype(valuetype, constraint, constraintCollection, index, validationContext);
    });
}
function checkConstraintMatchesValuetype(valuetypeDefinition, constraint, diagnosticNode, diagnosticIndex, context) {
    const actualValuetype = (0, ast_1.createValuetype)(valuetypeDefinition);
    const compatibleValuetype = getCompatibleValuetype(constraint);
    if (actualValuetype === undefined || compatibleValuetype === undefined) {
        return;
    }
    if (!actualValuetype.isConvertibleTo(compatibleValuetype)) {
        context.accept('error', `This valuetype ${actualValuetype.getName()} is not convertible to the type ${compatibleValuetype.getName()} of the constraint "${constraint.name}"`, {
            node: diagnosticNode,
            property: 'values',
            index: diagnosticIndex,
        });
    }
}
function getCompatibleValuetype(constraint) {
    if ((0, ast_1.isTypedConstraintDefinition)(constraint)) {
        if (ast_1.ConstraintTypeWrapper.canBeWrapped(constraint.type)) {
            return undefined;
        }
        return new ast_1.ConstraintTypeWrapper(constraint.type).on;
    }
    else if ((0, ast_1.isExpressionConstraintDefinition)(constraint)) {
        return (0, ast_1.createValuetype)(constraint === null || constraint === void 0 ? void 0 : constraint.valuetype);
    }
    (0, langium_1.assertUnreachable)(constraint);
}
function checkGenericsHaveNoDuplicate(valuetypeDefinition, context) {
    var _a;
    const generics = (_a = valuetypeDefinition.genericDefinition) === null || _a === void 0 ? void 0 : _a.generics;
    if (generics === undefined) {
        return;
    }
    const duplicates = getDuplicateGenerics(generics);
    duplicates.forEach((generic) => {
        context.accept('error', `Generic parameter ${generic.name} is not unique`, {
            node: generic,
            property: 'name',
        });
    });
}
function getDuplicateGenerics(generics) {
    const countPerGenericName = generics
        .map((generic) => {
        return { name: generic.name, count: 1 };
    })
        .reduce((result, current) => {
        var _a;
        const currentName = current.name;
        result[currentName] = ((_a = result[currentName]) !== null && _a !== void 0 ? _a : 0) + 1;
        return result;
    }, {});
    const duplicateGenericNames = Object.entries(countPerGenericName)
        .filter(([, occurences]) => occurences > 1)
        .map(([generic]) => generic);
    const duplicates = [];
    duplicateGenericNames.forEach((genericName) => {
        duplicates.push(...generics.filter((x) => x.name === genericName));
    });
    return duplicates;
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/checks/valuetype-reference.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.validateValuetypeReference = void 0;
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const ast_2 = __webpack_require__("./libs/language-server/src/lib/ast/generated/ast.ts");
function validateValuetypeReference(valuetypeRef, validationContext) {
    checkGenericsMatchDefinition(valuetypeRef, validationContext);
    checkIsValuetypeReferenceable(valuetypeRef, validationContext);
}
exports.validateValuetypeReference = validateValuetypeReference;
function checkGenericsMatchDefinition(valuetypeRef, context) {
    var _a, _b, _c, _d, _e, _f;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const valuetypeDefinition = (_a = valuetypeRef.reference) === null || _a === void 0 ? void 0 : _a.ref;
    if (valuetypeDefinition === undefined) {
        return;
    }
    const requiredGenerics = 
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    (_d = (_c = (_b = valuetypeDefinition.genericDefinition) === null || _b === void 0 ? void 0 : _b.generics) === null || _c === void 0 ? void 0 : _c.length) !== null && _d !== void 0 ? _d : 0;
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    const givenGenerics = (_f = (_e = valuetypeRef === null || valuetypeRef === void 0 ? void 0 : valuetypeRef.genericRefs) === null || _e === void 0 ? void 0 : _e.length) !== null && _f !== void 0 ? _f : 0;
    if (givenGenerics !== requiredGenerics) {
        context.accept('error', `The referenced valuetype ${valuetypeDefinition.name} requires ${requiredGenerics} generic parameters but found ${givenGenerics}.`, {
            node: valuetypeRef,
        });
    }
}
function checkIsValuetypeReferenceable(valuetypeRef, context) {
    const valuetype = (0, ast_1.createValuetype)(valuetypeRef);
    if (valuetype === undefined) {
        return;
    }
    const isUsedInBuiltinDefinition = (0, ast_2.isBuiltinBlocktypeDefinition)(valuetypeRef.$container.$container) ||
        (0, ast_2.isBuiltinConstrainttypeDefinition)(valuetypeRef.$container.$container);
    if (isUsedInBuiltinDefinition) {
        return;
    }
    if (valuetype.isReferenceableByUser()) {
        return;
    }
    context.accept('error', `Valuetype ${valuetype.getName()} cannot be referenced in this context`, {
        node: valuetypeRef,
    });
}


/***/ }),

/***/ "./libs/language-server/src/lib/validation/index.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const tslib_1 = __webpack_require__("tslib");
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/validation-context.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/validation-registry.ts"), exports);
tslib_1.__exportStar(__webpack_require__("./libs/language-server/src/lib/validation/validation-util.ts"), exports);


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validation-context.ts":
/***/ ((__unused_webpack_module, exports) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.ValidationContext = void 0;
class ValidationContext {
    constructor(validationAcceptor) {
        this.validationAcceptor = validationAcceptor;
        this.errorOccurred = false;
        this.accept = (severity, message, info) => {
            if (severity === 'error') {
                this.errorOccurred = true;
            }
            this.validationAcceptor(severity, message, info);
        };
    }
    hasErrorOccurred() {
        return this.errorOccurred;
    }
}
exports.ValidationContext = ValidationContext;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validation-registry.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.JayveeValidationRegistry = void 0;
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
const block_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/block-definition.ts");
const blocktype_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/blocktype-definition.ts");
const column_id_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/column-id.ts");
const composite_blocktype_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/composite-blocktype-definition.ts");
const expression_constraint_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/expression-constraint-definition.ts");
const jayvee_model_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/jayvee-model.ts");
const pipe_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/pipe-definition.ts");
const pipeline_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/pipeline-definition.ts");
const property_body_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/property-body.ts");
const range_literal_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/range-literal.ts");
const regex_literal_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/regex-literal.ts");
const transform_body_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/transform-body.ts");
const typed_constraint_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/typed-constraint-definition.ts");
const valuetype_definition_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/valuetype-definition.ts");
const valuetype_reference_1 = __webpack_require__("./libs/language-server/src/lib/validation/checks/valuetype-reference.ts");
const validation_context_1 = __webpack_require__("./libs/language-server/src/lib/validation/validation-context.ts");
/**
 * Registry for validation checks.
 */
class JayveeValidationRegistry extends langium_1.ValidationRegistry {
    constructor(services) {
        super(services);
        this.runtimeParameterProvider = services.RuntimeParameterProvider;
        this.registerJayveeValidationChecks({
            BuiltinBlocktypeDefinition: blocktype_definition_1.validateBlocktypeDefinition,
            BlockDefinition: block_definition_1.validateBlockDefinition,
            CompositeBlocktypeDefinition: composite_blocktype_definition_1.validateCompositeBlockTypeDefinition,
            ColumnId: column_id_1.validateColumnId,
            TypedConstraintDefinition: typed_constraint_definition_1.validateTypedConstraintDefinition,
            ExpressionConstraintDefinition: expression_constraint_definition_1.validateExpressionConstraintDefinition,
            JayveeModel: jayvee_model_1.validateJayveeModel,
            PipeDefinition: pipe_definition_1.validatePipeDefinition,
            PipelineDefinition: pipeline_definition_1.validatePipelineDefinition,
            PropertyBody: property_body_1.validatePropertyBody,
            RangeLiteral: range_literal_1.validateRangeLiteral,
            RegexLiteral: regex_literal_1.validateRegexLiteral,
            ValuetypeDefinition: valuetype_definition_1.validateValuetypeDefinition,
            ValuetypeReference: valuetype_reference_1.validateValuetypeReference,
            TransformBody: transform_body_1.validateTransformBody,
        });
    }
    registerJayveeValidationChecks(checksRecord) {
        for (const [type, check] of Object.entries(checksRecord)) {
            const wrappedCheck = this.wrapJayveeValidationCheck(check, this.runtimeParameterProvider);
            this.doRegister(type, this.wrapValidationException(wrappedCheck, this));
        }
    }
    wrapJayveeValidationCheck(check, runtimeParameterProvider) {
        return (node, accept) => {
            const validationContext = new validation_context_1.ValidationContext(accept);
            const evaluationContext = new ast_1.EvaluationContext(runtimeParameterProvider);
            return check(node, validationContext, evaluationContext);
        };
    }
}
exports.JayveeValidationRegistry = JayveeValidationRegistry;


/***/ }),

/***/ "./libs/language-server/src/lib/validation/validation-util.ts":
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {


// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.checkExpressionSimplification = exports.checkUniqueNames = void 0;
const assert_1 = __webpack_require__("assert");
const langium_1 = __webpack_require__("langium");
const ast_1 = __webpack_require__("./libs/language-server/src/lib/ast/index.ts");
function checkUniqueNames(nodes, context, nodeKind) {
    const nodesByName = groupNodesByName(nodes);
    for (const [nodeName, nodes] of nodesByName.entriesGroupedByKey()) {
        if (nodes.length > 1) {
            for (const node of nodes) {
                context.accept('error', `The ${nodeKind !== null && nodeKind !== void 0 ? nodeKind : node.$type.toLowerCase()} name "${nodeName}" needs to be unique.`, {
                    node,
                    property: 'name',
                });
            }
        }
    }
}
exports.checkUniqueNames = checkUniqueNames;
function groupNodesByName(nodes) {
    const nodesByName = new langium_1.MultiMap();
    for (const node of nodes) {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if ((node === null || node === void 0 ? void 0 : node.name) !== undefined) {
            nodesByName.add(node.name, node);
        }
    }
    return nodesByName;
}
function checkExpressionSimplification(expression, validationContext, evaluationContext) {
    const simplifiableSubExpressions = collectSubExpressionsWithoutFreeVariables(expression).filter(isSimplifiableExpression);
    simplifiableSubExpressions.forEach((expression) => {
        const evaluatedExpression = (0, ast_1.evaluateExpression)(expression, evaluationContext, validationContext, ast_1.EvaluationStrategy.EXHAUSTIVE);
        (0, assert_1.strict)(evaluatedExpression !== undefined);
        validationContext.accept('info', 
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
        `The expression can be simplified to ${evaluatedExpression}`, { node: expression });
    });
}
exports.checkExpressionSimplification = checkExpressionSimplification;
function collectSubExpressionsWithoutFreeVariables(expression) {
    if (expression === undefined) {
        return [];
    }
    if ((0, ast_1.isExpressionLiteral)(expression)) {
        if ((0, ast_1.isFreeVariableLiteral)(expression)) {
            return [];
        }
        return [expression];
    }
    else if ((0, ast_1.isUnaryExpression)(expression)) {
        const innerExpression = expression.expression;
        const innerSubExpressions = collectSubExpressionsWithoutFreeVariables(innerExpression);
        const innerExpressionHasNoFreeVariables = innerSubExpressions.length === 1 &&
            innerSubExpressions[0] === innerExpression;
        if (innerExpressionHasNoFreeVariables) {
            return [expression];
        }
        return innerSubExpressions;
    }
    else if ((0, ast_1.isBinaryExpression)(expression)) {
        const leftExpression = expression.left;
        const rightExpression = expression.right;
        const leftSubExpressions = collectSubExpressionsWithoutFreeVariables(leftExpression);
        const rightSubExpressions = collectSubExpressionsWithoutFreeVariables(rightExpression);
        const leftExpressionHasNoFreeVariables = leftSubExpressions.length === 1 &&
            leftSubExpressions[0] === leftExpression;
        const rightExpressionHasNoFreeVariables = rightSubExpressions.length === 1 &&
            rightSubExpressions[0] === rightExpression;
        if (leftExpressionHasNoFreeVariables && rightExpressionHasNoFreeVariables) {
            return [expression];
        }
        return [...leftSubExpressions, ...rightSubExpressions];
    }
    (0, langium_1.assertUnreachable)(expression);
}
function isSimplifiableExpression(expression) {
    return (!(0, ast_1.isExpressionLiteral)(expression) && !isNegativeNumberExpression(expression));
}
function isNegativeNumberExpression(expression) {
    return ((0, ast_1.isUnaryExpression)(expression) &&
        expression.operator === '-' &&
        (0, ast_1.isValueLiteral)(expression.expression));
}


/***/ }),

/***/ "@fast-csv/parse":
/***/ ((module) => {

module.exports = require("@fast-csv/parse");

/***/ }),

/***/ "assert":
/***/ ((module) => {

module.exports = require("assert");

/***/ }),

/***/ "assert/strict":
/***/ ((module) => {

module.exports = require("assert/strict");

/***/ }),

/***/ "chalk":
/***/ ((module) => {

module.exports = require("chalk");

/***/ }),

/***/ "commander":
/***/ ((module) => {

module.exports = require("commander");

/***/ }),

/***/ "exceljs":
/***/ ((module) => {

module.exports = require("exceljs");

/***/ }),

/***/ "follow-redirects":
/***/ ((module) => {

module.exports = require("follow-redirects");

/***/ }),

/***/ "fp-ts/lib/Either":
/***/ ((module) => {

module.exports = require("fp-ts/lib/Either");

/***/ }),

/***/ "gtfs-realtime-bindings":
/***/ ((module) => {

module.exports = require("gtfs-realtime-bindings");

/***/ }),

/***/ "jszip":
/***/ ((module) => {

module.exports = require("jszip");

/***/ }),

/***/ "langium":
/***/ ((module) => {

module.exports = require("langium");

/***/ }),

/***/ "langium/lib/utils/errors":
/***/ ((module) => {

module.exports = require("langium/lib/utils/errors");

/***/ }),

/***/ "langium/node":
/***/ ((module) => {

module.exports = require("langium/node");

/***/ }),

/***/ "mime-types":
/***/ ((module) => {

module.exports = require("mime-types");

/***/ }),

/***/ "pg":
/***/ ((module) => {

module.exports = require("pg");

/***/ }),

/***/ "sqlite3":
/***/ ((module) => {

module.exports = require("sqlite3");

/***/ }),

/***/ "tslib":
/***/ ((module) => {

module.exports = require("tslib");

/***/ }),

/***/ "util":
/***/ ((module) => {

module.exports = require("util");

/***/ }),

/***/ "vscode-languageserver":
/***/ ((module) => {

module.exports = require("vscode-languageserver");

/***/ }),

/***/ "vscode-languageserver-protocol":
/***/ ((module) => {

module.exports = require("vscode-languageserver-protocol");

/***/ }),

/***/ "vscode-languageserver-types":
/***/ ((module) => {

module.exports = require("vscode-languageserver-types");

/***/ }),

/***/ "vscode-uri":
/***/ ((module) => {

module.exports = require("vscode-uri");

/***/ }),

/***/ "fs":
/***/ ((module) => {

module.exports = require("fs");

/***/ }),

/***/ "node:zlib":
/***/ ((module) => {

module.exports = require("node:zlib");

/***/ }),

/***/ "path":
/***/ ((module) => {

module.exports = require("path");

/***/ }),

/***/ "process":
/***/ ((module) => {

module.exports = require("process");

/***/ }),

/***/ "./apps/interpreter/package.json":
/***/ ((module) => {

module.exports = JSON.parse('{"name":"@jvalue/jayvee-interpreter","description":"Interpreter for .jv files","main":"main.js","bin":{"jv":"main.js","jayvee":"main.js"},"bugs":{"url":"https://github.com/jvalue/jayvee/issues"},"repository":{"type":"git","url":"https://github.com/jvalue/jayvee.git"},"homepage":"https://github.com/jvalue/jayvee","publishConfig":{"access":"public","registry":"https://registry.npmjs.org"},"version":"0.3.0"}');

/***/ })

/******/ 	});
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId](module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it need to be isolated against other modules in the chunk.
(() => {
var exports = __webpack_exports__;

// SPDX-FileCopyrightText: 2023 Friedrich-Alexander-Universitat Erlangen-Nurnberg
//
// SPDX-License-Identifier: AGPL-3.0-only
Object.defineProperty(exports, "__esModule", ({ value: true }));
const jayvee_execution_1 = __webpack_require__("./libs/execution/src/index.ts");
const commander_1 = __webpack_require__("commander");
const package_json_1 = __webpack_require__("./apps/interpreter/package.json");
const prerequisites_1 = __webpack_require__("./apps/interpreter/src/prerequisites.ts");
const run_action_1 = __webpack_require__("./apps/interpreter/src/run-action.ts");
const runtimeParameterRegex = /^([_a-zA-Z][\w_]*)=(.*)$/;
function collectRuntimeParameters(optionValue, previous) {
    const regexMatch = optionValue.match(runtimeParameterRegex);
    if (regexMatch == null) {
        throw new Error(`Encountered runtime parameter with invalid syntax: ${optionValue}`);
    }
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const parameter = regexMatch[1];
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const value = regexMatch[2];
    previous.set(parameter, value);
    return previous;
}
(0, prerequisites_1.assertNodeVersion)();
const program = new commander_1.Command();
const version = package_json_1.version;
program.version(version);
program
    .argument('<file>', `path to the .jv source file`)
    .option('-e, --env <parameter>=<value>', 'provide a runtime parameters', collectRuntimeParameters, new Map())
    .option('-d, --debug', 'enable debug logging', false)
    .option('-dg, --debug-granularity <granularity>', `Sets the granularity of block debug logging. Can have values ${jayvee_execution_1.DebugGranularityValues.join(', ')} (default).`, jayvee_execution_1.DebugGranularityValues[jayvee_execution_1.DebugGranularityValues.length - 1])
    .option('-dt, --debug-target <block name>', `Sets the target blocks of the of block debug logging, separated by comma. If not given, all blocks are targeted.`, undefined)
    .option('-po, --parse-only', 'Only parses the model without running it. Exits with 0 if the model is valid, with 1 otherwise.', false)
    .description('Run a Jayvee file')
    .action(run_action_1.runAction);
program.showHelpAfterError();
program.parse(process.argv);

})();

var __webpack_export_target__ = exports;
for(var i in __webpack_exports__) __webpack_export_target__[i] = __webpack_exports__[i];
if(__webpack_exports__.__esModule) Object.defineProperty(__webpack_export_target__, "__esModule", { value: true });
/******/ })()
;
//# sourceMappingURL=main.js.map